@@ -1,8 +1,8 @@
 /*
  *      htmlchars.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2007 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2006-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: htmlchars.c 1826 2007-08-25 14:16:52Z eht16 $
+ * $Id: htmlchars.c 2195 2008-01-28 17:27:39Z ntrel $
  */
 
 /* HTML Characters plugin (Inserts HTML character entities like '&amp;') */
@@ -28,21 +28,17 @@
 #include "support.h"
 #include "plugindata.h"
 #include "document.h"
+#include "pluginmacros.h"
 
 
 PluginFields	*plugin_fields;
 GeanyData		*geany_data;
 
-#define doc_array	geany_data->doc_array
-// can't use document as a macro because it's currently a typename
-#define documents	geany_data->document
-#define scintilla	geany_data->sci
-#define ui			geany_data->ui
-
 
 VERSION_CHECK(7)
 
-PLUGIN_INFO(_("HTML Characters"), _("Inserts HTML character entities like '&amp;'."), "0.1")
+PLUGIN_INFO(_("HTML Characters"), _("Inserts HTML character entities like '&amp;'."), VERSION,
+	_("The Geany developer team"))
 
 
 enum
@@ -75,10 +71,10 @@ static void tools_show_dialog_insert_special_chars()
 		GtkWidget *swin, *vbox, *label;
 
 		sc_dialog = gtk_dialog_new_with_buttons(
-					_("Special Characters"), GTK_WINDOW(geany_data->app->window),
+					_("Special Characters"), GTK_WINDOW(app->window),
 					GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 					_("_Insert"), GTK_RESPONSE_OK, NULL);
-		vbox = ui->dialog_vbox_new(GTK_DIALOG(sc_dialog));
+		vbox = p_ui->dialog_vbox_new(GTK_DIALOG(sc_dialog));
 		gtk_box_set_spacing(GTK_BOX(vbox), 6);
 		gtk_widget_set_name(sc_dialog, "GeanyDialog");
 
@@ -429,18 +425,18 @@ static void sc_fill_store(GtkTreeStore *store)
  * returns only TRUE if a valid selection(i.e. no category) could be found */
 static gboolean sc_insert(GtkTreeModel *model, GtkTreeIter *iter)
 {
-	gint idx = documents->get_cur_idx();
+	gint idx = p_document->get_cur_idx();
 	gboolean result = FALSE;
 
 	if (DOC_IDX_VALID(idx))
 	{
 		gchar *str;
-		gint pos = scintilla->get_current_position(doc_list[idx].sci);
+		gint pos = p_sci->get_current_position(doc_list[idx].sci);
 
 		gtk_tree_model_get(model, iter, COLUMN_HTML_NAME, &str, -1);
 		if (str && *str)
 		{
-			scintilla->insert_text(doc_list[idx].sci, pos, str);
+			p_sci->insert_text(doc_list[idx].sci, pos, str);
 			g_free(str);
 			result = TRUE;
 		}
@@ -499,7 +495,7 @@ static void
 item_activate(GtkMenuItem *menuitem, gpointer gdata)
 {
 	// refuse opening the dialog if we don't have an active tab
-	gint idx = documents->get_cur_idx();
+	gint idx = p_document->get_cur_idx();
 
 	if (idx == -1 || ! doc_list[idx].is_valid) return;
 
