@@ -13,11 +13,14 @@
 #include "Platform.h"
 
 #include "Scintilla.h"
-#include "SVector.h"
 #include "SplitVector.h"
 #include "Partitioning.h"
 #include "CellBuffer.h"
 
+#ifdef SCI_NAMESPACE
+using namespace Scintilla;
+#endif
+
 MarkerHandleSet::MarkerHandleSet() {
 	root = 0;
 }
@@ -468,6 +471,7 @@ void UndoHistory::BeginUndoAction() {
 }
 
 void UndoHistory::EndUndoAction() {
+	PLATFORM_ASSERT(undoSequenceDepth > 0);
 	EnsureUndoRoom();
 	undoSequenceDepth--;
 	if (0 == undoSequenceDepth) {
@@ -559,7 +563,7 @@ CellBuffer::CellBuffer() {
 CellBuffer::~CellBuffer() {
 }
 
-char CellBuffer::CharAt(int position) {
+char CellBuffer::CharAt(int position) const {
 	return substance.ValueAt(position);
 }
 
@@ -649,7 +653,7 @@ const char *CellBuffer::DeleteChars(int position, int deleteLength, bool &startS
 	return data;
 }
 
-int CellBuffer::Length() {
+int CellBuffer::Length() const {
 	return substance.Length();
 }
 
@@ -658,11 +662,11 @@ void CellBuffer::Allocate(int newSize) {
 	style.ReAllocate(newSize);
 }
 
-int CellBuffer::Lines() {
+int CellBuffer::Lines() const {
 	return lv.Lines();
 }
 
-int CellBuffer::LineStart(int line) {
+int CellBuffer::LineStart(int line) const {
 	if (line < 0)
 		return 0;
 	else if (line >= Lines())
@@ -722,6 +726,20 @@ int CellBuffer::LineFromHandle(int markerHandle) {
 
 // Without undo
 
+void CellBuffer::InsertLine(int line, int position) {
+	lv.InsertLine(line, position);
+	if (lineStates.Length()) {
+		lineStates.Insert(line, 0);
+	}
+}
+
+void CellBuffer::RemoveLine(int line) {
+	lv.RemoveLine(line);
+	if (lineStates.Length()) {
+		lineStates.Delete(line);
+	}
+}
+
 void CellBuffer::BasicInsertString(int position, const char *s, int insertLength) {
 	if (insertLength == 0)
 		return;
@@ -737,21 +755,21 @@ void CellBuffer::BasicInsertString(int position, const char *s, int insertLength
 	char chAfter = substance.ValueAt(position + insertLength);
 	if (chPrev == '\r' && chAfter == '\n') {
 		// Splitting up a crlf pair at position
-		lv.InsertLine(lineInsert, position);
+		InsertLine(lineInsert, position);
 		lineInsert++;
 	}
 	char ch = ' ';
 	for (int i = 0; i < insertLength; i++) {
 		ch = s[i];
 		if (ch == '\r') {
-			lv.InsertLine(lineInsert, (position + i) + 1);
+			InsertLine(lineInsert, (position + i) + 1);
 			lineInsert++;
 		} else if (ch == '\n') {
 			if (chPrev == '\r') {
 				// Patch up what was end of line
 				lv.SetLineStart(lineInsert - 1, (position + i) + 1);
 			} else {
-				lv.InsertLine(lineInsert, (position + i) + 1);
+				InsertLine(lineInsert, (position + i) + 1);
 				lineInsert++;
 			}
 		}
@@ -761,7 +779,7 @@ void CellBuffer::BasicInsertString(int position, const char *s, int insertLength
 	if (chAfter == '\n') {
 		if (ch == '\r') {
 			// End of line already in buffer so drop the newly created one
-			lv.RemoveLine(lineInsert - 1);
+			RemoveLine(lineInsert - 1);
 		}
 	}
 }
@@ -796,13 +814,13 @@ void CellBuffer::BasicDeleteChars(int position, int deleteLength) {
 			chNext = substance.ValueAt(position + i + 1);
 			if (ch == '\r') {
 				if (chNext != '\n') {
-					lv.RemoveLine(lineRemove);
+					RemoveLine(lineRemove);
 				}
 			} else if (ch == '\n') {
 				if (ignoreNL) {
 					ignoreNL = false; 	// Further \n are real deletions
 				} else {
-					lv.RemoveLine(lineRemove);
+					RemoveLine(lineRemove);
 				}
 			}
 
@@ -813,7 +831,7 @@ void CellBuffer::BasicDeleteChars(int position, int deleteLength) {
 		char chAfter = substance.ValueAt(position + deleteLength);
 		if (chBefore == '\r' && chAfter == '\n') {
 			// Using lineRemove-1 as cr ended line before start of deletion
-			lv.RemoveLine(lineRemove - 1);
+			RemoveLine(lineRemove - 1);
 			lv.SetLineStart(lineRemove - 1, position + 1);
 		}
 	}
@@ -888,12 +906,14 @@ void CellBuffer::PerformRedoStep() {
 }
 
 int CellBuffer::SetLineState(int line, int state) {
+	lineStates.EnsureLength(line + 1);
 	int stateOld = lineStates[line];
 	lineStates[line] = state;
 	return stateOld;
 }
 
 int CellBuffer::GetLineState(int line) {
+	lineStates.EnsureLength(line + 1);
 	return lineStates[line];
 }
 
