@@ -1,8 +1,8 @@
 /*
  *      callbacks.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: callbacks.c 1932 2007-10-08 12:10:03Z eht16 $
+ * $Id: callbacks.c 2190 2008-01-25 13:14:38Z ntrel $
  */
 
 /*
@@ -61,6 +61,8 @@
 #include "tools.h"
 #include "project.h"
 #include "navqueue.h"
+#include "printing.h"
+#include "plugins.h"
 
 #include "geanyobject.h"
 
@@ -143,14 +145,17 @@ static void quit_app()
 
 	configuration_save();
 
-	// force close all tabs
+	// ignore changes for all tabs (already asked user in on_exit_clicked)
 	for (i = 0; i < doc_array->len; i++)
 	{
 		if (doc_list[i].is_valid && doc_list[i].changed)
 		{
-			doc_list[i].changed = FALSE;	// ignore changes (already asked user in on_exit_clicked)
+			doc_list[i].changed = FALSE;
 		}
 	}
+	if (app->project != NULL)
+		project_close(FALSE);	// save project session files
+
 	on_close_all1_activate(NULL, NULL);
 
 	main_quit();
@@ -297,6 +302,12 @@ on_file1_activate                      (GtkMenuItem     *menuitem,
 {
 	gtk_widget_set_sensitive(ui_widgets.recent_files_menuitem,
 						g_queue_get_length(ui_prefs.recent_queue) > 0);
+#if GTK_CHECK_VERSION(2, 10, 0)
+	// hide Page setup when GTK printing is not used
+	// (on GTK < 2.10 the menu item is hidden completely)
+	ui_widget_show_hide(ui_widgets.print_page_setup,
+		printing_prefs.use_gtk_printing || gtk_check_version(2, 10, 0) != NULL);
+#endif
 }
 
 
@@ -806,7 +817,18 @@ on_tv_notebook_switch_page             (GtkNotebook     *notebook,
                                         guint            page_num,
                                         gpointer         user_data)
 {
-	//switch_tv_notebook_page = TRUE;
+	// suppress selection changed signal when switching to the open files list
+	app->ignore_callback = TRUE;
+}
+
+
+void
+on_tv_notebook_switch_page_after       (GtkNotebook     *notebook,
+                                        GtkNotebookPage *page,
+                                        guint            page_num,
+                                        gpointer         user_data)
+{
+	app->ignore_callback = FALSE;
 }
 
 
@@ -986,9 +1008,8 @@ on_line_breaking1_toggled              (GtkCheckMenuItem *checkmenuitem,
 	if (! app->ignore_callback)
 	{
 		gint idx = document_get_cur_idx();
-		if (idx == -1 || ! doc_list[idx].is_valid) return;
-		doc_list[idx].line_wrapping = ! doc_list[idx].line_wrapping;
-		sci_set_lines_wrapped(doc_list[idx].sci, doc_list[idx].line_wrapping);
+		if (! DOC_IDX_VALID(idx)) return;
+		document_set_line_wrapping(idx, ! doc_list[idx].line_wrapping);
 	}
 }
 
@@ -1000,7 +1021,7 @@ on_set_file_readonly1_toggled          (GtkCheckMenuItem *checkmenuitem,
 	if (! app->ignore_callback)
 	{
 		gint idx = document_get_cur_idx();
-		if (idx == -1 || ! doc_list[idx].is_valid) return;
+		if (! DOC_IDX_VALID(idx)) return;
 		doc_list[idx].readonly = ! doc_list[idx].readonly;
 		sci_set_readonly(doc_list[idx].sci, doc_list[idx].readonly);
 		ui_update_tab_status(idx);
@@ -1016,7 +1037,7 @@ on_use_auto_indentation1_toggled       (GtkCheckMenuItem *checkmenuitem,
 	if (! app->ignore_callback)
 	{
 		gint idx = document_get_cur_idx();
-		if (idx == -1 || ! doc_list[idx].is_valid) return;
+		if (! DOC_IDX_VALID(idx)) return;
 		doc_list[idx].auto_indent = ! doc_list[idx].auto_indent;
 	}
 }
@@ -1168,7 +1189,7 @@ void
 on_find_in_files1_activate             (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	search_show_find_in_files_dialog();
+	search_show_find_in_files_dialog(NULL);
 }
 
 
@@ -1300,7 +1321,7 @@ on_comments_function_activate          (GtkMenuItem     *menuitem,
 
 	if (doc_list[idx].file_type == NULL)
 	{
-		ui_set_statusbar(_("Please set the filetype for the current file before using this function."));
+		ui_set_statusbar(FALSE, _("Please set the filetype for the current file before using this function."));
 		return;
 	}
 
@@ -1324,7 +1345,7 @@ on_comments_multiline_activate         (GtkMenuItem     *menuitem,
 
 	if (! DOC_IDX_VALID(idx) || doc_list[idx].file_type == NULL)
 	{
-		ui_set_statusbar(_("Please set the filetype for the current file before using this function."));
+		ui_set_statusbar(FALSE, _("Please set the filetype for the current file before using this function."));
 		return;
 	}
 
@@ -1406,25 +1427,10 @@ on_comments_fileheader_activate        (GtkMenuItem     *menuitem,
 }
 
 
-void
-on_custom_date_dialog_response         (GtkDialog *dialog,
-                                        gint response,
-                                        gpointer user_data)
+static void
+on_custom_date_input_response(const gchar *input)
 {
-	if (response == GTK_RESPONSE_ACCEPT)
-	{
-		g_free(ui_prefs.custom_date_format);
-		ui_prefs.custom_date_format = g_strdup(gtk_entry_get_text(GTK_ENTRY(user_data)));
-	}
-	gtk_widget_destroy(GTK_WIDGET(dialog));
-}
-
-
-void
-on_custom_date_entry_activate          (GtkEntry        *entry,
-                                        gpointer         user_data)
-{
-	on_custom_date_dialog_response(GTK_DIALOG(user_data), GTK_RESPONSE_ACCEPT, entry);
+	setptr(ui_prefs.custom_date_format, g_strdup(input));
 }
 
 
@@ -1453,7 +1459,7 @@ on_insert_date_activate                (GtkMenuItem     *menuitem,
 		format = "%m.%d.%Y %H:%M:%S";
 	else if (utils_str_equal(_("yyyy/mm/dd hh:mm:ss"), (gchar*) user_data))
 		format = "%Y/%m/%d %H:%M:%S";
-	else if (utils_str_equal(_("Use Custom Date Format"), (gchar*) user_data))
+	else if (utils_str_equal(_("_Use Custom Date Format"), (gchar*) user_data))
 		format = ui_prefs.custom_date_format;
 	else
 	{
@@ -1466,9 +1472,7 @@ on_insert_date_activate                (GtkMenuItem     *menuitem,
 
 		dialogs_show_input(_("Custom Date Format"),
 			_("Enter here a custom date and time format. You can use any conversion specifiers which can be used with the ANSI C strftime function."),
-			ui_prefs.custom_date_format,
-			G_CALLBACK(on_custom_date_dialog_response),
-			G_CALLBACK(on_custom_date_entry_activate));
+			ui_prefs.custom_date_format, FALSE, &on_custom_date_input_response);
 		return;
 	}
 
@@ -1485,7 +1489,7 @@ on_insert_date_activate                (GtkMenuItem     *menuitem,
 	else
 	{
 		utils_beep();
-		msgwin_status_add(
+		ui_set_statusbar(TRUE,
 				_("Date format string could not be converted (possibly too long)."));
 	}
 }
@@ -1604,8 +1608,9 @@ on_print1_activate                     (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	gint idx = document_get_cur_idx();
-
-	document_print(idx);
+	if (! DOC_IDX_VALID(idx))
+		return;
+	printing_print_doc(idx);
 }
 
 
@@ -1644,7 +1649,7 @@ on_menu_show_sidebar1_toggled          (GtkCheckMenuItem *checkmenuitem,
 		prefs.sidebar_symbol_visible = TRUE;
 	}
 
-	ui_treeviews_show_hide(TRUE);
+	ui_sidebar_show_hide();
 	gtk_notebook_set_current_page(GTK_NOTEBOOK(app->treeview_notebook), active_page);
 }
 
@@ -1731,7 +1736,7 @@ on_menu_increase_indent1_activate      (GtkMenuItem     *menuitem,
 		line = sci_get_line_from_position(doc_list[idx].sci, old_pos);
 		ind_pos = sci_get_line_indent_position(doc_list[idx].sci, line);
 		// when using tabs increase cur pos by 1, when using space increase it by tab_width
-		step = (editor_prefs.use_tabs) ? 1 : editor_prefs.tab_width;
+		step = (doc_list[idx].use_tabs) ? 1 : editor_prefs.tab_width;
 		new_pos = (old_pos > ind_pos) ? old_pos + step : old_pos;
 
 		sci_set_current_position(doc_list[idx].sci, ind_pos, TRUE);
@@ -1759,7 +1764,7 @@ on_menu_decrease_indent1_activate      (GtkMenuItem     *menuitem,
 		old_pos = sci_get_current_position(doc_list[idx].sci);
 		line = sci_get_line_from_position(doc_list[idx].sci, old_pos);
 		ind_pos = sci_get_line_indent_position(doc_list[idx].sci, line);
-		step = (editor_prefs.use_tabs) ? 1 : editor_prefs.tab_width;
+		step = (doc_list[idx].use_tabs) ? 1 : editor_prefs.tab_width;
 		new_pos = (old_pos >= ind_pos) ? old_pos - step : old_pos;
 
 		if (ind_pos == sci_get_position_from_line(doc_list[idx].sci, line))
@@ -1783,7 +1788,7 @@ on_next_message1_activate              (GtkMenuItem     *menuitem,
 {
 	if (! ui_tree_view_find_next(GTK_TREE_VIEW(msgwindow.tree_msg),
 		msgwin_goto_messages_file_line))
-		ui_set_statusbar(_("No more message items."));
+		ui_set_statusbar(FALSE, _("No more message items."));
 }
 
 
@@ -1852,7 +1857,7 @@ void
 on_project_close1_activate             (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	project_close();
+	project_close(TRUE);
 }
 
 
@@ -1887,43 +1892,32 @@ on_menu_open_selected_file1_activate   (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	gint idx = document_get_cur_idx();
-	gchar *filename = NULL;
+	gchar *sel = NULL;
 
-	if (idx == -1 || ! doc_list[idx].is_valid) return;
+	if (! DOC_IDX_VALID(idx)) return;
 
-	if (sci_get_lines_selected(doc_list[idx].sci) == 1)
+	sel = editor_get_default_selection(idx, GEANY_WORDCHARS"./");
+
+	if (sel != NULL)
 	{
-		gint len = sci_get_selected_text_length(doc_list[idx].sci);
+		gchar *locale_filename, *filename;
 
-		filename = g_malloc(len + 1);
-		sci_get_selected_text(doc_list[idx].sci, filename);
-	}
-	else if (sci_get_lines_selected(doc_list[idx].sci) == 0)
-	{	// use the word at current cursor position
-		gchar word[GEANY_MAX_WORD_LENGTH];
-
-		editor_find_current_word(doc_list[idx].sci, -1, word, sizeof(word), GEANY_WORDCHARS"./");
-		if (word[0] != '\0')
-			filename = g_strdup(word);
-	}
-
-	if (filename != NULL)
-	{
-		gchar *locale_filename;
-
-		if (! g_path_is_absolute(filename))
+		if (g_path_is_absolute(sel))
+			filename = g_strdup(sel);
+		else
 		{	// relative filename, add the path of the current file
 			gchar *path;
-			gchar *tmp = filename;
 
-			// use the projects base path if we have an open project (useful?)
-			if (app->project != NULL && app->project->base_path != NULL)
-				path = g_strdup(app->project->base_path);
-			else
-				path = g_path_get_dirname(doc_list[idx].file_name);
+			path = g_path_get_dirname(doc_list[idx].file_name);
+			filename = g_build_path(G_DIR_SEPARATOR_S, path, sel, NULL);
 
-			filename = g_strconcat(path, G_DIR_SEPARATOR_S, filename, NULL);
-			g_free(tmp);
+			if (! g_file_test(filename, G_FILE_TEST_EXISTS) &&
+				app->project != NULL && NZV(app->project->base_path))
+			{
+				// try the project's base path
+				setptr(path, project_get_base_path());
+				setptr(filename, g_build_path(G_DIR_SEPARATOR_S, path, sel, NULL));
+			}
 			g_free(path);
 		}
 
@@ -1932,6 +1926,7 @@ on_menu_open_selected_file1_activate   (GtkMenuItem     *menuitem,
 
 		g_free(filename);
 		g_free(locale_filename);
+		g_free(sel);
 	}
 }
 
@@ -1998,7 +1993,7 @@ on_context_action1_activate            (GtkMenuItem     *menuitem,
 
 		if (! g_spawn_command_line_async(command, &error))
 		{
-			msgwin_status_add("Context action command failed: %s", error->message);
+			ui_set_statusbar(TRUE, "Context action command failed: %s", error->message);
 			g_error_free(error);
 		}
 	}
@@ -2085,3 +2080,55 @@ gboolean on_motion_event(GtkWidget *widget, GdkEventMotion *event, gpointer user
 	return FALSE;
 }
 
+
+void
+on_tabs1_activate                      (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	gint idx = document_get_cur_idx();
+
+	document_set_use_tabs(idx, TRUE);
+	ui_update_statusbar(idx, -1);
+}
+
+
+void
+on_spaces1_activate                    (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	gint idx = document_get_cur_idx();
+
+	document_set_use_tabs(idx, FALSE);
+	ui_update_statusbar(idx, -1);
+}
+
+
+void
+on_strip_trailing_spaces1_activate     (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	gint idx = document_get_cur_idx();
+
+	document_strip_trailing_spaces(idx);
+}
+
+
+void
+on_page_setup1_activate                (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+#if GTK_CHECK_VERSION(2, 10, 0)
+	printing_page_setup_gtk();
+#endif
+}
+
+
+void
+on_tools1_activate                     (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+#ifdef HAVE_PLUGINS
+	plugins_update_tools_menu();
+#endif
+}
+
