@@ -1,8 +1,8 @@
 /*
  *      callbacks.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: callbacks.h 1860 2007-09-07 19:01:55Z eht16 $
+ * $Id: callbacks.h 2150 2008-01-06 18:11:57Z eht16 $
  */
 
 typedef struct
@@ -572,3 +572,30 @@ gboolean
 on_entry_goto_line_key_press_event     (GtkWidget       *widget,
                                         GdkEventKey     *event,
                                         gpointer         user_data);
+
+void
+on_tv_notebook_switch_page_after       (GtkNotebook     *notebook,
+                                        GtkNotebookPage *page,
+                                        guint            page_num,
+                                        gpointer         user_data);
+
+void
+on_tabs1_activate                      (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_spaces1_activate                    (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+
+void
+on_strip_trailing_spaces1_activate     (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_page_setup1_activate                (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_tools1_activate                     (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
