@@ -1,8 +1,8 @@
 /*
  *      dialogs.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,30 +18,30 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: dialogs.h 1663 2007-07-04 17:08:53Z eht16 $
+ * $Id: dialogs.h 2150 2008-01-06 18:11:57Z eht16 $
  */
 
 
 #ifndef GEANY_DIALOGS_H
 #define GEANY_DIALOGS_H 1
 
-/* This shows the file selection dialog to open a file. */
+typedef void (*InputCallback)(const gchar *);
+
+
 void dialogs_show_open_file();
 
-/* This shows the file selection dialog to save a file. */
 gboolean dialogs_show_save_as();
 
 gboolean dialogs_show_unsaved_file(gint idx);
 
-/* This shows the font selection dialog to choose a font. */
 void dialogs_show_open_font();
 
 void dialogs_show_word_count();
 
 void dialogs_show_color(gchar *colour);
 
-void dialogs_show_input(const gchar *title, const gchar *label_text, const gchar *default_text,
-						GCallback cb_dialog, GCallback cb_entry);
+GtkWidget *dialogs_show_input(const gchar *title, const gchar *label_text,
+	const gchar *default_text, gboolean persistent, InputCallback input_cb);
 
 void dialogs_show_goto_line();
 
@@ -49,10 +49,11 @@ void dialogs_show_file_properties(gint idx);
 
 gboolean dialogs_show_question(const gchar *text, ...) G_GNUC_PRINTF (1, 2);
 
-/* extra_text can be NULL; otherwise it is displayed below main_text. */
 gboolean dialogs_show_question_full(GtkWidget *parent, const gchar *yes_btn, const gchar *no_btn,
 	const gchar *extra_text, const gchar *main_text, ...) G_GNUC_PRINTF (5, 6);
 
 void dialogs_show_msgbox(gint type, const gchar *text, ...) G_GNUC_PRINTF (2, 3);
 
+void dialogs_show_msgbox_with_secondary(gint type, const gchar *text, const gchar *secondary);
+
 #endif
