@@ -1,8 +1,8 @@
 /*
  *      document.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: document.c 1934 2007-10-09 12:04:15Z eht16 $
+ * $Id: document.c 2164 2008-01-11 17:09:23Z ntrel $
  */
 
 /*
@@ -70,6 +70,7 @@
 #include "symbols.h"
 #include "callbacks.h"
 #include "geanyobject.h"
+#include "highlighting.h"
 
 
 /* dynamic array of document elements to hold all information of the notebook tabs */
@@ -80,8 +81,6 @@ GArray *doc_array;
 static gboolean delay_colourise = FALSE;
 
 
-void msgwin_status_add_new(const gchar *format, ...) G_GNUC_PRINTF(1, 2);	// temporary for v0.12
-
 static void document_undo_clear(gint idx);
 static void document_redo_add(gint idx, guint type, gpointer data);
 
@@ -251,19 +250,40 @@ void document_set_text_changed(gint idx)
 }
 
 
+void document_set_use_tabs(gint idx, gboolean use_tabs)
+{
+	document *doc = &doc_list[idx];
+
+	g_return_if_fail(DOC_IDX_VALID(idx));
+
+	doc->use_tabs = use_tabs;
+	sci_set_use_tabs(doc->sci, use_tabs);
+	// remove indent spaces on backspace, if using spaces to indent
+	SSM(doc->sci, SCI_SETBACKSPACEUNINDENTS, ! use_tabs, 0);
+}
+
+
+void document_set_line_wrapping(gint idx, gboolean wrap)
+{
+	document *doc = &doc_list[idx];
+
+	g_return_if_fail(DOC_IDX_VALID(idx));
+
+	doc->line_wrapping = wrap;
+	sci_set_lines_wrapped(doc->sci, wrap);
+}
+
+
 // Apply just the prefs that can change in the Preferences dialog
 void document_apply_update_prefs(gint idx)
 {
 	ScintillaObject *sci = doc_list[idx].sci;
+
 	sci_set_mark_long_lines(sci, editor_prefs.long_line_type, editor_prefs.long_line_column, editor_prefs.long_line_color);
 
 	sci_set_tab_width(sci, editor_prefs.tab_width);
 
-	sci_set_use_tabs(sci, editor_prefs.use_tabs);
-	// remove indent spaces on backspace, if using spaces to indent
-	SSM(sci, SCI_SETBACKSPACEUNINDENTS, ! editor_prefs.use_tabs, 0);
-
-	sci_set_autoc_max_height(sci, editor_prefs.autocompletion_max_height);
+	sci_set_autoc_max_height(sci, editor_prefs.symbolcompletion_max_height);
 
 	sci_set_indentation_guides(sci, editor_prefs.show_indent_guide);
 	sci_set_visible_white_spaces(sci, editor_prefs.show_white_space);
@@ -344,6 +364,8 @@ static void setup_sci_keys(ScintillaObject *sci)
 	sci_assign_cmdkey(sci, SCK_UP | (SCMOD_CTRL << 16) | (SCMOD_SHIFT << 16), SCI_PARAUPEXTEND);
 	sci_assign_cmdkey(sci, SCK_DOWN | (SCMOD_CTRL << 16), SCI_PARADOWN);
 	sci_assign_cmdkey(sci, SCK_DOWN | (SCMOD_CTRL << 16) | (SCMOD_SHIFT << 16), SCI_PARADOWNEXTEND);
+
+	sci_clear_cmdkey(sci, SCK_BACK | (SCMOD_ALT << 16)); // clear Alt-Backspace (Undo)
 }
 
 
@@ -399,6 +421,8 @@ static gint document_create_new_sci(const gchar *filename)
 	sci_set_scrollbar_mode(sci, editor_prefs.show_scrollbars);
 	sci_set_caret_policy_x(sci, CARET_JUMPS | CARET_EVEN, 0);
 	//sci_set_caret_policy_y(sci, CARET_JUMPS | CARET_EVEN, 0);
+	SSM(sci, SCI_AUTOCSETSEPARATOR, '\n', 0);
+	SSM(sci, SCI_SETENDATLASTLINE, FALSE, 0);	// allow scrolling past end of document
 
 	// signal for insert-key(works without too, but to update the right status bar)
 	//g_signal_connect((GtkWidget*) sci, "key-press-event",
@@ -466,11 +490,11 @@ gboolean document_remove(guint page_num)
 		}
 		notebook_remove_page(page_num);
 		treeviews_remove_document(idx);
-		msgwin_status_add_new(_("File %s closed."), DOC_FILENAME(idx));
+		msgwin_status_add(_("File %s closed."), DOC_FILENAME(idx));
 		g_free(doc_list[idx].encoding);
 		g_free(doc_list[idx].saved_encoding.encoding);
 		g_free(doc_list[idx].file_name);
-		tm_workspace_remove_object(doc_list[idx].tm_file, TRUE);
+		tm_workspace_remove_object(doc_list[idx].tm_file, TRUE, TRUE);
 
 		doc_list[idx].is_valid = FALSE;
 		doc_list[idx].sci = NULL;
@@ -510,7 +534,7 @@ static void store_saved_encoding(gint idx)
  * filename is either the UTF-8 file name, or NULL.
  * If ft is NULL and filename is not NULL, then the filetype will be guessed
  * from the given filename.
- * text is the contents of the new file, or NULL.
+ * text is the contents of the new file in valid UTF-8 encoding, or NULL.
  * Returns: idx of new file in doc_list. */
 gint document_new_file(const gchar *filename, filetype *ft, const gchar *text)
 {
@@ -529,6 +553,7 @@ gint document_new_file(const gchar *filename, filetype *ft, const gchar *text)
 #else
 	sci_set_eol_mode(doc_list[idx].sci, SC_EOL_LF);
 #endif
+	document_set_use_tabs(idx, editor_prefs.use_tabs);
 	sci_set_undo_collection(doc_list[idx].sci, TRUE);
 	sci_empty_undo_buffer(doc_list[idx].sci);
 
@@ -545,7 +570,7 @@ gint document_new_file(const gchar *filename, filetype *ft, const gchar *text)
 
 	document_set_filetype(idx, ft);	// also clears taglist
 	if (ft == NULL)
-		filetypes[GEANY_FILETYPES_ALL]->style_func_ptr(doc_list[idx].sci);
+		highlighting_set_styles(doc_list[idx].sci, GEANY_FILETYPES_ALL);
 	ui_set_window_title(idx);
 	build_menu_update(idx);
 	document_update_tag_list(idx, FALSE);
@@ -564,7 +589,7 @@ gint document_new_file(const gchar *filename, filetype *ft, const gchar *text)
 		g_signal_emit_by_name(geany_object, "document-new", idx);
 	}
 
-	msgwin_status_add_new(_("New file \"%s\" opened."),
+	msgwin_status_add(_("New file \"%s\" opened."),
 		(doc_list[idx].file_name != NULL) ? doc_list[idx].file_name : GEANY_STRING_UNTITLED);
 
 	return idx;
@@ -730,7 +755,7 @@ static gboolean load_text_file(const gchar *locale_filename, const gchar *utf8_f
 
 	if (g_stat(locale_filename, &st) != 0)
 	{
-		msgwin_status_add(_("Could not open file %s (%s)"), utf8_filename, g_strerror(errno));
+		ui_set_statusbar(TRUE, _("Could not open file %s (%s)"), utf8_filename, g_strerror(errno));
 		return FALSE;
 	}
 
@@ -738,7 +763,7 @@ static gboolean load_text_file(const gchar *locale_filename, const gchar *utf8_f
 
 	if (! g_file_get_contents(locale_filename, &filedata->data, NULL, &err))
 	{
-		msgwin_status_add(err->message);
+		ui_set_statusbar(TRUE, err->message);
 		g_error_free(err);
 		return FALSE;
 	}
@@ -764,7 +789,7 @@ static gboolean load_text_file(const gchar *locale_filename, const gchar *utf8_f
 		if (main_status.main_window_realized)
 			dialogs_show_msgbox(GTK_MESSAGE_WARNING, warn_msg, utf8_filename);
 
-		msgwin_status_add(warn_msg, utf8_filename);
+		ui_set_statusbar(TRUE, warn_msg, utf8_filename);
 
 		// set the file to read-only mode because saving it is probably dangerous
 		filedata->readonly = TRUE;
@@ -781,7 +806,7 @@ static gboolean load_text_file(const gchar *locale_filename, const gchar *utf8_f
 		}
 		else if (! handle_forced_encoding(filedata, forced_enc))
 		{
-			msgwin_status_add(_("The file \"%s\" is not valid %s."), utf8_filename, forced_enc);
+			ui_set_statusbar(TRUE, _("The file \"%s\" is not valid %s."), utf8_filename, forced_enc);
 			utils_beep();
 			g_free(filedata->data);
 			return FALSE;
@@ -789,7 +814,7 @@ static gboolean load_text_file(const gchar *locale_filename, const gchar *utf8_f
 	}
 	else if (! handle_encoding(filedata))
 	{
-		msgwin_status_add(
+		ui_set_statusbar(TRUE,
 			_("The file \"%s\" does not look like a text file or the file encoding is not supported."),
 			utf8_filename);
 		utils_beep();
@@ -830,6 +855,38 @@ static void set_cursor_position(gint idx, gint pos)
 }
 
 
+static gboolean detect_use_tabs(ScintillaObject *sci)
+{
+	gint line;
+	gsize tabs = 0, spaces = 0;
+
+	for (line = 0; line < sci_get_line_count(sci); line++)
+	{
+		gint pos = sci_get_position_from_line(sci, line);
+		gchar c;
+
+		c = sci_get_char_at(sci, pos);
+		if (c == '\t')
+			tabs++;
+		else
+		if (c == ' ')
+		{
+			// check at least 2 spaces
+			if (sci_get_char_at(sci, pos + 1) == ' ')
+				spaces++;
+		}
+	}
+	if (spaces == 0 && tabs == 0)
+		return editor_prefs.use_tabs;
+
+	// Skew comparison by a factor of 2 in favour of default editor pref
+	if (editor_prefs.use_tabs)
+		return ! (spaces > tabs * 2);
+	else
+		return (tabs > spaces * 2);
+}
+
+
 /* To open a new file, set idx to -1; filename should be locale encoded.
  * To reload a file, set the idx for the document to be reloaded; filename should be NULL.
  * pos is the cursor position, which can be overridden by --line and --column.
@@ -867,7 +924,7 @@ gint document_open_file_full(gint idx, const gchar *filename, gint pos, gboolean
 		// filename must not be NULL when opening a file
 		if (filename == NULL)
 		{
-			ui_set_statusbar(_("Invalid filename"));
+			ui_set_statusbar(FALSE, _("Invalid filename"));
 			return -1;
 		}
 
@@ -917,6 +974,21 @@ gint document_open_file_full(gint idx, const gchar *filename, gint pos, gboolean
 	sci_set_eol_mode(doc_list[idx].sci, editor_mode);
 	g_free(filedata.data);
 
+	if (reload)
+		document_set_use_tabs(idx, doc_list[idx].use_tabs); // resetup sci
+	else
+	if (! editor_prefs.detect_tab_mode)
+		document_set_use_tabs(idx, editor_prefs.use_tabs);
+	else
+	{	// detect & set tabs/spaces
+		gboolean use_tabs = detect_use_tabs(doc_list[idx].sci);
+
+		if (use_tabs != editor_prefs.use_tabs)
+			ui_set_statusbar(TRUE, _("Setting %s indentation mode."),
+				(use_tabs) ? _("Tabs") : _("Spaces"));
+		document_set_use_tabs(idx, use_tabs);
+	}
+
 	sci_set_undo_collection(doc_list[idx].sci, TRUE);
 
 	doc_list[idx].mtime = filedata.mtime; // get the modification time from file and keep it
@@ -962,9 +1034,9 @@ gint document_open_file_full(gint idx, const gchar *filename, gint pos, gboolean
 		g_signal_emit_by_name(geany_object, "document-open", idx);
 
 	if (reload)
-		msgwin_status_add(_("File %s reloaded."), utf8_filename);
+		ui_set_statusbar(TRUE, _("File %s reloaded."), utf8_filename);
 	else
-		msgwin_status_add_new(_("File %s opened(%d%s)."),
+		msgwin_status_add(_("File %s opened(%d%s)."),
 				utf8_filename, gtk_notebook_get_n_pages(GTK_NOTEBOOK(app->notebook)),
 				(readonly) ? _(", read-only") : "");
 
@@ -1056,15 +1128,10 @@ static gboolean document_update_timestamp(gint idx)
 
 	g_return_val_if_fail(DOC_IDX_VALID(idx), FALSE);
 
-#ifdef G_OS_WIN32
-	// don't try to convert the filename on Windows, it should be already in UTF8
-	locale_filename = g_strdup(doc_list[idx].file_name);
-#else
 	locale_filename = utils_get_locale_from_utf8(doc_list[idx].file_name);
-#endif
 	if (g_stat(locale_filename, &st) != 0)
 	{
-		msgwin_status_add(_("Could not open file %s (%s)"), doc_list[idx].file_name,
+		ui_set_statusbar(TRUE, _("Could not open file %s (%s)"), doc_list[idx].file_name,
 			g_strerror(errno));
 		g_free(locale_filename);
 		return FALSE;
@@ -1076,6 +1143,32 @@ static gboolean document_update_timestamp(gint idx)
 }
 
 
+/* Sets line and column to the given position byte_pos in the document.
+ * byte_pos is the position counted in bytes, not characters */
+static void get_line_column_from_pos(gint idx, guint byte_pos, gint *line, gint *column)
+{
+	gint i;
+	gint line_start;
+
+	// for some reason we can use byte count instead of character count here
+	*line = sci_get_line_from_position(doc_list[idx].sci, byte_pos);
+	line_start = sci_get_position_from_line(doc_list[idx].sci, *line);
+	// get the column in the line
+	*column = byte_pos - line_start;
+
+	// any non-ASCII characters are encoded with two bytes(UTF-8, always in Scintilla), so
+	// skip one byte(i++) and decrease the column number which is based on byte count
+	for (i = line_start; i < (line_start + *column); i++)
+	{
+		if (sci_get_char_at(doc_list[idx].sci, i) < 0)
+		{
+			(*column)--;
+			i++;
+		}
+	}
+}
+
+
 /* This saves the file.
  * When force is set then it is always saved, even if it is unchanged(useful when using Save As)
  * It returns whether the file could be saved or not. */
@@ -1084,17 +1177,15 @@ gboolean document_save_file(gint idx, gboolean force)
 	gchar *data;
 	FILE *fp;
 	gint bytes_written, len;
-#ifndef G_OS_WIN32
 	gchar *locale_filename = NULL;
-#endif
 
 	if (! DOC_IDX_VALID(idx)) return FALSE;
-	// the changed flag should exclude the readonly flag, but check it anyway for safety
+	// the "changed" flag should exclude the "readonly" flag, but check it anyway for safety
 	if (! force && (! doc_list[idx].changed || doc_list[idx].readonly)) return FALSE;
 
 	if (doc_list[idx].file_name == NULL)
 	{
-		msgwin_status_add(_("Error saving file."));
+		ui_set_statusbar(TRUE, _("Error saving file."));
 		utils_beep();
 		return FALSE;
 	}
@@ -1132,21 +1223,50 @@ gboolean document_save_file(gint idx, gboolean force)
 	{
 		GError *conv_error = NULL;
 		gchar* conv_file_contents = NULL;
+		gsize bytes_read;
 		gsize conv_len;
 
 		// try to convert it from UTF-8 to original encoding
 		conv_file_contents = g_convert(data, len-1, doc_list[idx].encoding, "UTF-8",
-													NULL, &conv_len, &conv_error);
+													&bytes_read, &conv_len, &conv_error);
 
 		if (conv_error != NULL)
 		{
-			dialogs_show_msgbox(GTK_MESSAGE_ERROR,
-	_("An error occurred while converting the file from UTF-8 in \"%s\". The file remains unsaved."
-	  "\nError message: %s\n"),
-			doc_list[idx].encoding, conv_error->message);
-			geany_debug("encoding error: %s)", conv_error->message);
+			gchar *text = g_strdup_printf(
+				_("An error occurred while converting the file from UTF-8 in \"%s\". The file remains unsaved."),
+				doc_list[idx].encoding);
+			gchar *error_text;
+
+			if (conv_error->code == G_CONVERT_ERROR_ILLEGAL_SEQUENCE)
+			{
+				gchar *context = NULL;
+				gint line, column;
+				gint context_len;
+				gunichar unic;
+				gint max_len = MIN((gint)bytes_read + 6, len - 1); // don't read over the doc length
+				context = g_malloc(7); // read 6 bytes from Sci + '\0'
+				sci_get_text_range(doc_list[idx].sci, bytes_read, max_len, context);
+
+				// take only one valid Unicode character from the context and discard the leftover
+				unic = g_utf8_get_char_validated(context, -1);
+				context_len = g_unichar_to_utf8(unic, context);
+				context[context_len] = '\0';
+				get_line_column_from_pos(idx, bytes_read, &line, &column);
+
+				error_text = g_strdup_printf(
+					_("Error message: %s\nThe error occurred at \"%s\" (line: %d, column: %d)."),
+					conv_error->message, context, line + 1, column);
+				g_free(context);
+			}
+			else
+				error_text = g_strdup_printf(_("Error message: %s."), conv_error->message);
+
+			geany_debug("encoding error: %s", conv_error->message);
+			dialogs_show_msgbox_with_secondary(GTK_MESSAGE_ERROR, text, error_text);
 			g_error_free(conv_error);
 			g_free(data);
+			g_free(text);
+			g_free(error_text);
 			return FALSE;
 		}
 		else
@@ -1160,16 +1280,14 @@ gboolean document_save_file(gint idx, gboolean force)
 	{
 		len = strlen(data);
 	}
-#ifdef G_OS_WIN32
-	fp = g_fopen(doc_list[idx].file_name, "wb"); // this should fix the windows \n problem
-#else
+
 	locale_filename = utils_get_locale_from_utf8(doc_list[idx].file_name);
-	fp = g_fopen(locale_filename, "w");
+	fp = g_fopen(locale_filename, "wb");
 	g_free(locale_filename);
-#endif
+
 	if (fp == NULL)
 	{
-		msgwin_status_add(_("Error saving file (%s)."), g_strerror(errno));
+		ui_set_statusbar(TRUE, _("Error saving file (%s)."), g_strerror(errno));
 		utils_beep();
 		g_free(data);
 		return FALSE;
@@ -1181,7 +1299,7 @@ gboolean document_save_file(gint idx, gboolean force)
 
 	if (len != bytes_written)
 	{
-		msgwin_status_add(_("Error saving file."));
+		ui_set_statusbar(TRUE, _("Error saving file."));
 		utils_beep();
 		return FALSE;
 	}
@@ -1220,7 +1338,7 @@ gboolean document_save_file(gint idx, gboolean force)
 		tm_workspace_update(TM_WORK_OBJECT(app->tm_workspace), TRUE, TRUE, FALSE);
 		gtk_label_set_text(GTK_LABEL(doc_list[idx].tab_label), base_name);
 		gtk_label_set_text(GTK_LABEL(doc_list[idx].tabmenu_label), base_name);
-		msgwin_status_add_new(_("File %s saved."), doc_list[idx].file_name);
+		msgwin_status_add(_("File %s saved."), doc_list[idx].file_name);
 		ui_update_statusbar(idx, -1);
 		g_free(base_name);
 #ifdef HAVE_VTE
@@ -1284,7 +1402,7 @@ gboolean document_search_bar_find(gint idx, const gchar *text, gint flags, gbool
 	{
 		if (! inc)
 		{
-			ui_set_statusbar(_("\"%s\" was not found."), text);
+			ui_set_statusbar(FALSE, _("\"%s\" was not found."), text);
 		}
 		utils_beep();
 		sci_goto_pos(doc_list[idx].sci, start_pos, FALSE);	// clear selection
@@ -1338,7 +1456,7 @@ gint document_find_text(gint idx, const gchar *text, gint flags, gboolean search
 		if ((selection_end == 0 && ! search_backwards) ||
 			(selection_end == sci_len && search_backwards))
 		{
-			ui_set_statusbar(_("\"%s\" was not found."), text);
+			ui_set_statusbar(FALSE, _("\"%s\" was not found."), text);
 			utils_beep();
 			return -1;
 		}
@@ -1421,7 +1539,7 @@ static void show_replace_summary(gint idx, gint count, const gchar *find_text,
 
 	if (count == 0)
 	{
-		ui_set_statusbar(_("No matches found for \"%s\"."), find_text);
+		ui_set_statusbar(FALSE, _("No matches found for \"%s\"."), find_text);
 		return;
 	}
 
@@ -1431,14 +1549,14 @@ static void show_replace_summary(gint idx, gint count, const gchar *find_text,
 	{	// escape special characters for showing
 		escaped_find_text = g_strescape(find_text, NULL);
 		escaped_replace_text = g_strescape(replace_text, NULL);
-		msgwin_status_add(_("%s: replaced %d occurrence(s) of \"%s\" with \"%s\"."),
+		ui_set_statusbar(TRUE, _("%s: replaced %d occurrence(s) of \"%s\" with \"%s\"."),
 						filename, count, escaped_find_text, escaped_replace_text);
 		g_free(escaped_find_text);
 		g_free(escaped_replace_text);
 	}
 	else
 	{
-		msgwin_status_add(_("%s: replaced %d occurrence(s) of \"%s\" with \"%s\"."),
+		ui_set_statusbar(TRUE, _("%s: replaced %d occurrence(s) of \"%s\" with \"%s\"."),
 						filename, count, find_text, replace_text);
 	}
 	g_free(filename);
@@ -1667,8 +1785,14 @@ void document_set_font(gint idx, const gchar *font_name, gint size)
 
 void document_update_tag_list(gint idx, gboolean update)
 {
+	/* We must call treeviews_update_tag_list() before returning,
+	 * to ensure that the symbol list is always updated properly (e.g.
+	 * when creating a new document with a partial filename set. */
+	gboolean success = FALSE;
+
 	// if the filetype doesn't have a tag parser or it is a new file
 	if (idx == -1 || doc_list[idx].file_type == NULL ||
+		app->tm_workspace == NULL ||
 		! filetype_has_tags(doc_list[idx].file_type) || ! doc_list[idx].file_name)
 	{
 		// set the default (empty) tag list
@@ -1678,32 +1802,34 @@ void document_update_tag_list(gint idx, gboolean update)
 
 	if (doc_list[idx].tm_file == NULL)
 	{
-#ifdef GEANY_OS_WIN32
-		doc_list[idx].tm_file = tm_source_file_new(
-				doc_list[idx].file_name, FALSE, doc_list[idx].file_type->name);
-#else
 		gchar *locale_filename = utils_get_locale_from_utf8(doc_list[idx].file_name);
+
 		doc_list[idx].tm_file = tm_source_file_new(
 				locale_filename, FALSE, doc_list[idx].file_type->name);
 		g_free(locale_filename);
-#endif
-		if (! doc_list[idx].tm_file) return;
-		tm_workspace_add_object(doc_list[idx].tm_file);
-		if (update)
-			tm_source_file_update(doc_list[idx].tm_file, TRUE, FALSE, TRUE);
-		treeviews_update_tag_list(idx, TRUE);
+
+		if (doc_list[idx].tm_file)
+		{
+			if (!tm_workspace_add_object(doc_list[idx].tm_file))
+			{
+				tm_work_object_free(doc_list[idx].tm_file);
+				doc_list[idx].tm_file = NULL;
+			}
+			else
+			{
+				if (update)
+					tm_source_file_update(doc_list[idx].tm_file, TRUE, FALSE, TRUE);
+				success = TRUE;
+			}
+		}
 	}
 	else
 	{
-		if (tm_source_file_update(doc_list[idx].tm_file, TRUE, FALSE, TRUE))
-		{
-			treeviews_update_tag_list(idx, TRUE);
-		}
-		else
-		{
+		success = tm_source_file_update(doc_list[idx].tm_file, TRUE, FALSE, TRUE);
+		if (! success)
 			geany_debug("tag list updating failed");
-		}
 	}
+	treeviews_update_tag_list(idx, success);
 }
 
 
@@ -1804,7 +1930,10 @@ void document_set_filetype(gint idx, filetype *type)
 	if (type == NULL || ! DOC_IDX_VALID(idx))
 		return;
 
-	geany_debug("%s : %s (%s)",	doc_list[idx].file_name, type->name, doc_list[idx].encoding);
+	geany_debug("%s : %s (%s)",
+		(doc_list[idx].file_name != NULL) ? doc_list[idx].file_name : "unknown",
+		(type->name != NULL) ? type->name : "unknown",
+		(doc_list[idx].encoding != NULL) ? doc_list[idx].encoding : "unknown");
 
 	ft_changed = (doc_list[idx].file_type != type);
 	if (ft_changed)	// filetype has changed
@@ -1814,10 +1943,10 @@ void document_set_filetype(gint idx, filetype *type)
 		// delete tm file object to force creation of a new one
 		if (doc_list[idx].tm_file != NULL)
 		{
-			tm_workspace_remove_object(doc_list[idx].tm_file, TRUE);
+			tm_workspace_remove_object(doc_list[idx].tm_file, TRUE, TRUE);
 			doc_list[idx].tm_file = NULL;
 		}
-		type->style_func_ptr(doc_list[idx].sci);	// set new styles
+		highlighting_set_styles(doc_list[idx].sci, type->id);
 		build_menu_update(idx);
 		colourise = TRUE;
 	}
@@ -1852,40 +1981,37 @@ gchar *document_get_eol_mode(gint idx)
 }
 
 
-void document_unfold_all(gint idx)
+static void fold_all(gint idx, gboolean want_fold)
 {
-	gint lines, pos, i;
+	gint lines, first, i;
 
-	if (idx == -1 || ! doc_list[idx].is_valid) return;
+	if (! DOC_IDX_VALID(idx) || ! editor_prefs.folding) return;
 
 	lines = sci_get_line_count(doc_list[idx].sci);
-	pos = sci_get_current_position(doc_list[idx].sci);
-
-	for (i = 0; i < lines; i++)
-	{
-		sci_ensure_line_is_visible(doc_list[idx].sci, i);
-	}
-}
-
-
-void document_fold_all(gint idx)
-{
-	gint lines, pos, i;
-
-	if (idx == -1 || ! doc_list[idx].is_valid) return;
-
-	lines = sci_get_line_count(doc_list[idx].sci);
-	pos = sci_get_current_position(doc_list[idx].sci);
+	first = sci_get_first_visible_line(doc_list[idx].sci);
 
 	for (i = 0; i < lines; i++)
 	{
 		gint level = sci_get_fold_level(doc_list[idx].sci, i);
 		if (level & SC_FOLDLEVELHEADERFLAG)
 		{
-			if (sci_get_fold_expanded(doc_list[idx].sci, i))
+			if (sci_get_fold_expanded(doc_list[idx].sci, i) == want_fold)
 					sci_toggle_fold(doc_list[idx].sci, i);
 		}
 	}
+	editor_scroll_to_line(doc_list[idx].sci, first, 0.0F);
+}
+
+
+void document_unfold_all(gint idx)
+{
+	fold_all(idx, FALSE);
+}
+
+
+void document_fold_all(gint idx)
+{
+	fold_all(idx, TRUE);
 }
 
 
@@ -1938,49 +2064,6 @@ void document_set_indicator(gint idx, gint line)
 }
 
 
-/* simple file print */
-void document_print(gint idx)
-{
-	gchar *cmdline;
-
-	if (! DOC_IDX_VALID(idx) || doc_list[idx].file_name == NULL)
-		return;
-
-	cmdline = g_strdup(prefs.tools_print_cmd);
-	cmdline = utils_str_replace(cmdline, "%f", doc_list[idx].file_name);
-
-	if (dialogs_show_question(
-			_("The file \"%s\" will be printed with the following command:\n\n%s"),
-			doc_list[idx].file_name, cmdline))
-	{
-		GError *error = NULL;
-
-#ifdef G_OS_WIN32
-		gchar *tmp_cmdline = cmdline;
-#else
-		// /bin/sh -c emulates the system() call and makes complex commands possible
-		// but only needed on non-win32 systems due to the lack of win32's shell capabilities
-		gchar *tmp_cmdline = g_strconcat("/bin/sh -c \"", cmdline, "\"", NULL);
-#endif
-
-		if (! g_spawn_command_line_async(tmp_cmdline, &error))
-		{
-			dialogs_show_msgbox(GTK_MESSAGE_ERROR, _("Printing of \"%s\" failed (return code: %s)."),
-								doc_list[idx].file_name, error->message);
-			g_error_free(error);
-		}
-		else
-		{
-			msgwin_status_add(_("File %s printed."), doc_list[idx].file_name);
-		}
-#ifndef G_OS_WIN32
-		g_free(tmp_cmdline);
-#endif
-	}
-	g_free(cmdline);
-}
-
-
 void document_replace_tabs(gint idx)
 {
 	gint search_pos, pos_in_line, current_tab_true_length;
@@ -2016,6 +2099,27 @@ void document_replace_tabs(gint idx)
 }
 
 
+void document_strip_line_trailing_spaces(gint idx, gint line)
+{
+	gint line_start = sci_get_position_from_line(doc_list[idx].sci, line);
+	gint line_end = sci_get_line_end_position(doc_list[idx].sci, line);
+	gint i = line_end - 1;
+	gchar ch = sci_get_char_at(doc_list[idx].sci, i);
+
+	while ((i >= line_start) && ((ch == ' ') || (ch == '\t')))
+	{
+		i--;
+		ch = sci_get_char_at(doc_list[idx].sci, i);
+	}
+	if (i < (line_end-1))
+	{
+		sci_target_start(doc_list[idx].sci, i + 1);
+		sci_target_end(doc_list[idx].sci, line_end);
+		sci_target_replace(doc_list[idx].sci, "", FALSE);
+	}
+}
+
+
 void document_strip_trailing_spaces(gint idx)
 {
 	gint max_lines = sci_get_line_count(doc_list[idx].sci);
@@ -2025,22 +2129,7 @@ void document_strip_trailing_spaces(gint idx)
 
 	for (line = 0; line < max_lines; line++)
 	{
-		gint line_start = sci_get_position_from_line(doc_list[idx].sci, line);
-		gint line_end = sci_get_line_end_position(doc_list[idx].sci, line);
-		gint i = line_end - 1;
-		gchar ch = sci_get_char_at(doc_list[idx].sci, i);
-
-		while ((i >= line_start) && ((ch == ' ') || (ch == '\t')))
-		{
-			i--;
-			ch = sci_get_char_at(doc_list[idx].sci, i);
-		}
-		if (i < (line_end-1))
-		{
-			sci_target_start(doc_list[idx].sci, i + 1);
-			sci_target_end(doc_list[idx].sci, line_end);
-			sci_target_replace(doc_list[idx].sci, "", FALSE);
-		}
+		document_strip_line_trailing_spaces(idx, line);
 	}
 	sci_end_undo_action(doc_list[idx].sci);
 }
