@@ -1,8 +1,8 @@
 /*
  *      document.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: document.h 1869 2007-09-11 16:05:03Z ntrel $
+ *  $Id: document.h 2150 2008-01-06 18:11:57Z eht16 $
  */
 
 
@@ -36,7 +36,7 @@ typedef struct FileEncoding
 } FileEncoding;
 
 
-/* structure for representing an open tab with all its related stuff. */
+/* Structure for representing an open tab with all its related stuff. */
 typedef struct document
 {
 	gboolean		 is_valid;
@@ -52,7 +52,7 @@ typedef struct document
 	GtkWidget		*tabmenu_label;
 	GtkWidget		*tag_tree;
 	GtkTreeStore	*tag_store;
-	GtkTreeIter		 iter;
+	GtkTreeIter		 iter;	// open files item for this document
 	gboolean		 readonly;
 	gboolean		 changed;
 	gboolean		 line_wrapping;
@@ -63,6 +63,7 @@ typedef struct document
 	GTrashStack		*undo_actions;
 	GTrashStack		*redo_actions;
 	FileEncoding	 saved_encoding;
+	gboolean		 use_tabs;
 } document;
 
 
@@ -185,11 +186,10 @@ void document_set_indicator(gint idx, gint line);
 
 void document_clear_indicators(gint idx);
 
-/* simple file print */
-void document_print(gint idx);
-
 void document_replace_tabs(gint idx);
 
+void document_strip_line_trailing_spaces(gint idx, gint line);
+
 void document_strip_trailing_spaces(gint idx);
 
 void document_ensure_final_newline(gint idx);
@@ -239,4 +239,8 @@ void document_colourise_new();
 
 void document_insert_colour(gint idx, const gchar *colour);
 
+void document_set_use_tabs(gint idx, gboolean use_tabs);
+
+void document_set_line_wrapping(gint idx, gboolean wrap);
+
 #endif
