@@ -1,8 +1,8 @@
 /*
  *      encodings.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: encodings.h 1376 2007-03-09 13:52:26Z eht16 $
+ *  $Id: encodings.h 2177 2008-01-16 16:50:10Z eht16 $
  */
 
 
@@ -57,16 +57,17 @@ typedef struct
 } GeanyEncoding;
 
 
-
 const GeanyEncoding* encodings_get_from_charset(const gchar *charset);
 const GeanyEncoding* encodings_get_from_index(gint idx);
 
 gchar* encodings_to_string(const GeanyEncoding* enc);
 const gchar* encodings_get_charset(const GeanyEncoding* enc);
+const gchar* encodings_get_charset_from_index(gint idx);
 
 void encodings_select_radio_item(const gchar *charset);
 
 void encodings_init(void);
+void encodings_finalize(void);
 
 gchar *encodings_convert_to_utf8(const gchar *buffer, gsize size, gchar **used_encoding);
 
@@ -167,4 +168,6 @@ GeanyEncoding encodings[GEANY_ENCODINGS_MAX];
 
 GeanyEncodingIndex encodings_scan_unicode_bom(const gchar *string, gsize len, guint *bom_len);
 
+GeanyEncodingIndex encodings_get_idx_from_charset(const gchar *charset);
+
 #endif
