@@ -1,8 +1,8 @@
 /*
  *      filetypes.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: filetypes.h 1800 2007-08-15 15:37:21Z ntrel $
+ * $Id: filetypes.h 2150 2008-01-06 18:11:57Z eht16 $
  */
 
 
@@ -119,7 +119,6 @@ struct filetype
 	gboolean  		  comment_use_indent;
 	struct build_programs	*programs;
 	struct build_actions	*actions;
-	void (*style_func_ptr) (ScintillaObject*);
 };
 
 extern filetype *filetypes[GEANY_MAX_FILE_TYPES];
@@ -144,7 +143,7 @@ filetype *filetypes_detect_from_filename(const gchar *utf8_filename);
 /* frees the array and all related pointers */
 void filetypes_free_types();
 
-void filetypes_get_config(GKeyFile *config, GKeyFile *configh, gint ft);
+void filetypes_load_config(gint ft_id);
 
 void filetypes_save_commands();
 
