@@ -1,7 +1,8 @@
 /*
  *      gb.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
+ *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,7 +18,11 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: gb.c 1059 2006-12-07 16:09:45Z eht16 $
+ * $Id: gb.c 2150 2008-01-06 18:11:57Z eht16 $
+ */
+
+/*
+ * GTK-Bandit.
  */
 
 #include <stdlib.h>
@@ -67,7 +72,7 @@ static void create_window(void)
 	GtkWidget *button5;
 
 	gb_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-	gtk_window_set_title(GTK_WINDOW(gb_window), "Happy Eastern!");
+	gtk_window_set_title(GTK_WINDOW(gb_window), "Happy Easter!");
 	gtk_window_set_resizable(GTK_WINDOW(gb_window), FALSE);
 	gtk_window_set_position(GTK_WINDOW(gb_window), GTK_WIN_POS_CENTER);
 	gtk_window_set_destroy_with_parent(GTK_WINDOW(gb_window), TRUE);
@@ -358,7 +363,7 @@ static void update_labels(GtkWidget *window, gint init, gint won)
 	}
 	gtk_label_set_text(GTK_LABEL(label3), info_texts[won]);
 
-	snprintf(pts, 50, "Points: %4d\tRound: %2d\n", points, lap);
+	g_snprintf(pts, 50, "Points: %4d\tRound: %2d\n", points, lap);
 	gtk_label_set_text(GTK_LABEL(label1), pts);
 
 }
