@@ -1,8 +1,8 @@
 /*
  *      geany.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: geany.h 1815 2007-08-23 11:34:06Z ntrel $
+ * $Id: geany.h 2218 2008-02-04 08:48:20Z eht16 $
  */
 
 /* Main header - should be included first in all source files.
@@ -42,10 +42,9 @@
 
 // for detailed description look in the documentation, things are not
 // listed in the documentation should not be changed ;-)
-#define GEANY_HOME_DIR					g_get_home_dir()
 #define GEANY_FILEDEFS_SUBDIR			"filedefs"
 #define GEANY_TEMPLATES_SUBDIR			"templates"
-#define GEANY_CODENAME					"Delurin"
+#define GEANY_CODENAME					"Vensell"
 #define GEANY_HOMEPAGE					"http://geany.uvena.de/"
 #define GEANY_PROJECT_EXT				"geany"
 #define GEANY_USE_WIN32_DIALOG			0
@@ -54,11 +53,12 @@
 #define GEANY_TOGGLE_MARK				"~ "
 #define GEANY_MAX_WORD_LENGTH			192
 #define GEANY_MAX_AUTOCOMPLETE_WORDS	30
-#define GEANY_MAX_AUTOCOMPLETE_HEIGHT	10
+#define GEANY_MAX_SYMBOLLIST_HEIGHT		10
+#define GEANY_MIN_SYMBOLLIST_CHARS		4
 #define GEANY_STRING_UNTITLED			_("untitled")
 #define GEANY_MSGWIN_HEIGHT				208
-#define GEANY_WINDOW_MINIMAL_WIDTH		620
-#define GEANY_WINDOW_MINIMAL_HEIGHT		440
+#define GEANY_WINDOW_MINIMAL_WIDTH		550
+#define GEANY_WINDOW_MINIMAL_HEIGHT		350
 #define GEANY_WINDOW_DEFAULT_WIDTH		900
 #define GEANY_WINDOW_DEFAULT_HEIGHT		600
 // some default settings which are used at the very first start of Geany to fill configuration file
@@ -91,6 +91,7 @@ typedef struct GeanyApp
 	gchar				*configdir;
 	gchar				*datadir;
 	gchar				*docdir;
+	gchar			   **active_plugins; // list of plugin filenames to load at startup
 	const TMWorkspace	*tm_workspace;
 	GeanyProject		*project;	// currently active project or NULL if none is open
 	gboolean			ignore_callback;	// should not be used in new code (use clicked instead of toggled signal)
@@ -100,7 +101,7 @@ typedef struct GeanyApp
 	GtkWidget			*toolbar;
 	GtkWidget			*treeview_notebook;
 	GtkWidget			*notebook;
-	GtkWidget			*statusbar;	// use ui_set_statusbar() or msgwin_status_add() to set
+	GtkWidget			*statusbar;	// use ui_set_statusbar() to set
 	GtkWidget			*popup_menu;
 }
 GeanyApp;
