@@ -1,8 +1,8 @@
 /*
  *      geanyobject.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2007 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: geanyobject.h 1816 2007-08-23 15:22:13Z ntrel $
+ * $Id: geanyobject.h 2150 2008-01-06 18:11:57Z eht16 $
  */
 
 
@@ -38,6 +38,9 @@ typedef enum
 	GCB_DOCUMENT_OPEN,
 	GCB_DOCUMENT_SAVE,
 	GCB_DOCUMENT_ACTIVATE,
+	GCB_PROJECT_OPEN,
+	GCB_PROJECT_SAVE,
+	GCB_PROJECT_CLOSE,
 	GCB_MAX
 } GeanyCallbackId;
 
@@ -71,10 +74,13 @@ struct _GeanyObjectClass
 	void (*document_open)(gint idx);
 	void (*document_save)(gint idx);
 	void (*document_activate)(gint idx);
+	void (*project_open)(GKeyFile *keyfile);
+	void (*project_save)(GKeyFile *keyfile);
+	void (*project_close)();
 };
 
 GType		geany_object_get_type	(void);
-GObject*	geany_object_new			(void);
+GObject*	geany_object_new		(void);
 
 G_END_DECLS
 
