@@ -1,8 +1,8 @@
 /*
  *      keyfile.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,13 +18,18 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: keyfile.c 1927 2007-10-02 16:20:44Z ntrel $
+ * $Id: keyfile.c 2202 2008-01-31 16:31:19Z ntrel $
  */
 
 /*
  * geany.conf preferences file loading and saving.
  */
 
+/*
+ * Session file format:
+ * filename_xx=pos;filetype UID;read only;encoding idx;use_tabs;auto_indent;line_wrapping;filename
+ */
+
 #include <stdlib.h>
 #include <string.h>
 
@@ -51,6 +56,8 @@
 #include "search.h"
 #include "project.h"
 #include "editor.h"
+#include "printing.h"
+#include "plugins.h"
 
 
 static gchar *scribble_text = NULL;
@@ -87,16 +94,13 @@ static void save_recent_files(GKeyFile *config)
 }
 
 
-static void save_session_files(GKeyFile *config)
+void configuration_save_session_files(GKeyFile *config)
 {
 	gint idx, npage;
 	gchar *tmp;
 	gchar entry[14];
 	guint i = 0, j = 0, max;
 
-	if (! cl_options.load_session)
-		return;
-
 	npage = gtk_notebook_get_current_page(GTK_NOTEBOOK(app->notebook));
 	g_key_file_set_integer(config, "files", "current_page", npage);
 
@@ -113,8 +117,15 @@ static void save_session_files(GKeyFile *config)
 			if (ft == NULL)	// can happen when saving a new file when quitting
 				ft = filetypes[GEANY_FILETYPES_ALL];
 			g_snprintf(entry, 13, "FILE_NAME_%d", j);
-			fname = g_strdup_printf("%d:%d:%s", sci_get_current_position(doc_list[idx].sci),
-				ft->uid, doc_list[idx].file_name);
+			fname = g_strdup_printf("%d;%d;%d;%d;%d;%d;%d;%s;",
+				sci_get_current_position(doc_list[idx].sci),
+				ft->uid,
+				doc_list[idx].readonly,
+				encodings_get_idx_from_charset(doc_list[idx].encoding),
+				doc_list[idx].use_tabs,
+				doc_list[idx].auto_indent,
+				doc_list[idx].line_wrapping,
+				doc_list[idx].file_name);
 			g_key_file_set_string(config, "files", entry, fname);
 			g_free(fname);
 			j++;
@@ -140,13 +151,28 @@ static void save_session_files(GKeyFile *config)
 }
 
 
+#ifdef HAVE_PLUGINS
+static void save_plugin_prefs(GKeyFile *config)
+{
+	g_key_file_set_boolean(config, "plugins", "load_plugins", prefs.load_plugins);
+	plugins_create_active_list();
+	if (app->active_plugins != NULL)
+		g_key_file_set_string_list(config, "plugins", "active_plugins",
+			(const gchar**)app->active_plugins, g_strv_length(app->active_plugins));
+	else
+		// use an empty dummy array to override maybe exisiting value
+		g_key_file_set_string_list(config, "plugins", "active_plugins", (const gchar*[1]){ "" }, 1);
+}
+#endif
+
+
 static void save_dialog_prefs(GKeyFile *config)
 {
 	/* Some of the key names are not consistent, but this is for backwards compatibility */
 
 	// general
 	g_key_file_set_boolean(config, PACKAGE, "pref_main_load_session", prefs.load_session);
-	g_key_file_set_boolean(config, PACKAGE, "load_plugins", prefs.load_plugins);
+	g_key_file_set_boolean(config, PACKAGE, "pref_main_project_session", project_prefs.project_session);
 	g_key_file_set_boolean(config, PACKAGE, "pref_main_save_winpos", prefs.save_winpos);
 	g_key_file_set_boolean(config, PACKAGE, "pref_main_confirm_exit", prefs.confirm_exit);
 	g_key_file_set_boolean(config, PACKAGE, "pref_main_suppress_search_dialogs", prefs.suppress_search_dialogs);
@@ -159,6 +185,7 @@ static void save_dialog_prefs(GKeyFile *config)
 	// interface
 	g_key_file_set_boolean(config, PACKAGE, "sidebar_symbol_visible", prefs.sidebar_symbol_visible);
 	g_key_file_set_boolean(config, PACKAGE, "sidebar_openfiles_visible", prefs.sidebar_openfiles_visible);
+	g_key_file_set_boolean(config, PACKAGE, "sidebar_openfiles_fullpath", prefs.sidebar_openfiles_fullpath);
 	g_key_file_set_string(config, PACKAGE, "editor_font", prefs.editor_font);
 	g_key_file_set_string(config, PACKAGE, "tagbar_font", prefs.tagbar_font);
 	g_key_file_set_string(config, PACKAGE, "msgwin_font", prefs.msgwin_font);
@@ -180,19 +207,22 @@ static void save_dialog_prefs(GKeyFile *config)
 	g_key_file_set_string(config, PACKAGE, "long_line_color", editor_prefs.long_line_color);
 
 	// editor
-	g_key_file_set_integer(config, PACKAGE, "autocompletion_max_height", editor_prefs.autocompletion_max_height);
+	g_key_file_set_integer(config, PACKAGE, "symbolcompletion_max_height", editor_prefs.symbolcompletion_max_height);
+	g_key_file_set_integer(config, PACKAGE, "symbolcompletion_min_chars", editor_prefs.symbolcompletion_min_chars);
 	g_key_file_set_boolean(config, PACKAGE, "use_folding", editor_prefs.folding);
 	g_key_file_set_boolean(config, PACKAGE, "unfold_all_children", editor_prefs.unfold_all_children);
 	g_key_file_set_integer(config, PACKAGE, "indent_mode", editor_prefs.indent_mode);
+	g_key_file_set_integer(config, PACKAGE, "check_detect_indent", editor_prefs.detect_tab_mode);
 	g_key_file_set_boolean(config, PACKAGE, "use_indicators", editor_prefs.use_indicators);
 	g_key_file_set_boolean(config, PACKAGE, "line_breaking", editor_prefs.line_wrapping);
 	g_key_file_set_boolean(config, PACKAGE, "auto_close_xml_tags", editor_prefs.auto_close_xml_tags);
-	g_key_file_set_boolean(config, PACKAGE, "auto_complete_constructs", editor_prefs.auto_complete_constructs);
+	g_key_file_set_boolean(config, PACKAGE, "complete_snippets", editor_prefs.complete_snippets);
 	g_key_file_set_boolean(config, PACKAGE, "auto_complete_symbols", editor_prefs.auto_complete_symbols);
 	g_key_file_set_integer(config, PACKAGE, "pref_editor_tab_width", editor_prefs.tab_width);
 	g_key_file_set_boolean(config, PACKAGE, "pref_editor_use_tabs", editor_prefs.use_tabs);
 	g_key_file_set_boolean(config, PACKAGE, "pref_editor_disable_dnd", editor_prefs.disable_dnd);
 	g_key_file_set_boolean(config, PACKAGE, "pref_editor_smart_home_key", editor_prefs.smart_home_key);
+	g_key_file_set_boolean(config, PACKAGE, "pref_editor_newline_strip", editor_prefs.newline_strip);
 
 	// files
 	g_key_file_set_string(config, PACKAGE, "pref_editor_default_new_encoding", encodings[prefs.default_new_encoding].charset);
@@ -210,6 +240,7 @@ static void save_dialog_prefs(GKeyFile *config)
 	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_search", prefs.toolbar_show_search);
 	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_goto", prefs.toolbar_show_goto);
 	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_zoom", prefs.toolbar_show_zoom);
+	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_indent", prefs.toolbar_show_indent);
 	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_undo", prefs.toolbar_show_undo);
 	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_navigation", prefs.toolbar_show_navigation);
 	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_compile", prefs.toolbar_show_compile);
@@ -230,10 +261,18 @@ static void save_dialog_prefs(GKeyFile *config)
 	g_key_file_set_string(config, "tools", "make_cmd", prefs.tools_make_cmd ? prefs.tools_make_cmd : "");
 	g_key_file_set_string(config, "tools", "term_cmd", prefs.tools_term_cmd ? prefs.tools_term_cmd : "");
 	g_key_file_set_string(config, "tools", "browser_cmd", prefs.tools_browser_cmd ? prefs.tools_browser_cmd : "");
-	g_key_file_set_string(config, "tools", "print_cmd", prefs.tools_print_cmd ? prefs.tools_print_cmd : "");
 	g_key_file_set_string(config, "tools", "grep_cmd", prefs.tools_grep_cmd ? prefs.tools_grep_cmd : "");
 	g_key_file_set_string(config, PACKAGE, "context_action_cmd", prefs.context_action_cmd);
 
+	// printing
+	g_key_file_set_string(config, "printing", "print_cmd", printing_prefs.external_print_cmd ? printing_prefs.external_print_cmd : "");
+	g_key_file_set_boolean(config, "printing", "use_gtk_printing", printing_prefs.use_gtk_printing);
+	g_key_file_set_boolean(config, "printing", "print_line_numbers", printing_prefs.print_line_numbers);
+	g_key_file_set_boolean(config, "printing", "print_page_numbers", printing_prefs.print_page_numbers);
+	g_key_file_set_boolean(config, "printing", "print_page_header", printing_prefs.print_page_header);
+	g_key_file_set_boolean(config, "printing", "page_header_basename", printing_prefs.page_header_basename);
+	g_key_file_set_string(config, "printing", "page_header_datefmt", printing_prefs.page_header_datefmt);
+
 	// VTE
 #ifdef HAVE_VTE
 	g_key_file_set_boolean(config, "VTE", "load_vte", vte_info.load_vte);
@@ -245,9 +284,11 @@ static void save_dialog_prefs(GKeyFile *config)
 		g_key_file_set_string(config, "VTE", "font", vc->font);
 		g_key_file_set_boolean(config, "VTE", "scroll_on_key", vc->scroll_on_key);
 		g_key_file_set_boolean(config, "VTE", "scroll_on_out", vc->scroll_on_out);
+		g_key_file_set_boolean(config, "VTE", "enable_bash_keys", vc->enable_bash_keys);
 		g_key_file_set_boolean(config, "VTE", "ignore_menu_bar_accel", vc->ignore_menu_bar_accel);
 		g_key_file_set_boolean(config, "VTE", "follow_path", vc->follow_path);
 		g_key_file_set_boolean(config, "VTE", "run_in_vte", vc->run_in_vte);
+		g_key_file_set_boolean(config, "VTE", "skip_run_script", vc->skip_run_script);
 		g_key_file_set_integer(config, "VTE", "scrollback_lines", vc->scrollback_lines);
 		g_key_file_set_string(config, "VTE", "font", vc->font);
 		g_key_file_set_string(config, "VTE", "shell", vc->shell);
@@ -291,7 +332,7 @@ static void save_ui_prefs(GKeyFile *config)
 				gtk_paned_get_position(GTK_PANED(lookup_widget(app->window, "vpaned1"))));
 	}
 
-	if (prefs.save_winpos && ! ui_prefs.fullscreen)
+	if (prefs.save_winpos)
 	{
 		gtk_window_get_position(GTK_WINDOW(app->window), &ui_prefs.geometry[0], &ui_prefs.geometry[1]);
 		gtk_window_get_size(GTK_WINDOW(app->window), &ui_prefs.geometry[2], &ui_prefs.geometry[3]);
@@ -327,9 +368,8 @@ static void save_hidden_prefs(GKeyFile *config)
 	write_hidden_pref_boolean(config, PACKAGE, "brace_match_ltgt", editor_prefs.brace_match_ltgt);
 	write_hidden_pref_boolean(config, PACKAGE, "use_gtk_word_boundaries", editor_prefs.use_gtk_word_boundaries);
 	write_hidden_pref_boolean(config, PACKAGE, "auto_complete_whilst_editing", editor_prefs.auto_complete_whilst_editing);
-#ifdef HAVE_VTE
-	if (vte_info.have_vte)
-		write_hidden_pref_boolean(config, "VTE", "enable_bash_keys", vc->enable_bash_keys);
+#if GTK_CHECK_VERSION(2, 12, 0)
+	write_hidden_pref_boolean(config, PACKAGE, "show_symbol_list_expanders", prefs.show_symbol_list_expanders);
 #endif
 }
 
@@ -343,11 +383,15 @@ void configuration_save()
 	g_key_file_load_from_file(config, configfile, G_KEY_FILE_NONE, NULL);
 
 	save_dialog_prefs(config);
+#ifdef HAVE_PLUGINS
+	save_plugin_prefs(config);
+#endif
 	save_hidden_prefs(config);
 	save_ui_prefs(config);
 	project_save_prefs(config);	// save project filename, etc.
 	save_recent_files(config);
-	save_session_files(config);
+	if (cl_options.load_session)
+		configuration_save_session_files(config);
 
 	// write the file
 	data = g_key_file_to_data(config, NULL, NULL);
@@ -358,15 +402,18 @@ void configuration_save()
 	g_free(configfile);
 }
 
-
-static void load_file_lists(GKeyFile *config)
+/*
+ * Load session list from the given keyfile, and store it in the global
+ * session_files variable for later file loading
+ * */
+void configuration_load_session_files(GKeyFile *config)
 {
 	gchar **recent_files;
 	guint i;
 	gsize len = 0;
 	gboolean have_session_files;
 	gchar entry[14];
-	gchar *tmp_string;
+	gchar **tmp_array;
 	GError *error = NULL;
 
 	session_notebook_page = utils_get_setting_integer(config, "files", "current_page", -1);
@@ -382,20 +429,24 @@ static void load_file_lists(GKeyFile *config)
 	}
 	g_strfreev(recent_files);
 
+	// the project may load another list than the main setting
+	if (session_files != NULL)
+		g_ptr_array_free(session_files, TRUE);
+
 	session_files = g_ptr_array_new();
 	have_session_files = TRUE;
 	i = 0;
 	while (have_session_files)
 	{
 		g_snprintf(entry, 13, "FILE_NAME_%d", i);
-		tmp_string = g_key_file_get_string(config, "files", entry, &error);
-		if (! tmp_string || error)
+		tmp_array = g_key_file_get_string_list(config, "files", entry, NULL, &error);
+		if (! tmp_array || error)
 		{
 			g_error_free(error);
 			error = NULL;
 			have_session_files = FALSE;
 		}
-		g_ptr_array_add(session_files, tmp_string);
+		g_ptr_array_add(session_files, tmp_array);
 		i++;
 	}
 }
@@ -419,7 +470,7 @@ static void load_dialog_prefs(GKeyFile *config)
 	prefs.suppress_search_dialogs = utils_get_setting_boolean(config, PACKAGE, "pref_main_suppress_search_dialogs", FALSE);
 	prefs.suppress_status_messages = utils_get_setting_boolean(config, PACKAGE, "pref_main_suppress_status_messages", FALSE);
 	prefs.load_session = utils_get_setting_boolean(config, PACKAGE, "pref_main_load_session", TRUE);
-	prefs.load_plugins = utils_get_setting_boolean(config, PACKAGE, "load_plugins", TRUE);
+	project_prefs.project_session = utils_get_setting_boolean(config, PACKAGE, "pref_main_project_session", TRUE);
 	prefs.save_winpos = utils_get_setting_boolean(config, PACKAGE, "pref_main_save_winpos", TRUE);
 	prefs.beep_on_errors = utils_get_setting_boolean(config, PACKAGE, "beep_on_errors", TRUE);
 	prefs.switch_msgwin_pages = utils_get_setting_boolean(config, PACKAGE, "switch_msgwin_pages", FALSE);
@@ -432,6 +483,7 @@ static void load_dialog_prefs(GKeyFile *config)
 	prefs.tab_pos_sidebar = utils_get_setting_integer(config, PACKAGE, "tab_pos_sidebar", GTK_POS_TOP);
 	prefs.sidebar_symbol_visible = utils_get_setting_boolean(config, PACKAGE, "sidebar_symbol_visible", TRUE);
 	prefs.sidebar_openfiles_visible = utils_get_setting_boolean(config, PACKAGE, "sidebar_openfiles_visible", TRUE);
+	prefs.sidebar_openfiles_fullpath = utils_get_setting_boolean(config, PACKAGE, "sidebar_openfiles_fullpath", FALSE);
 	prefs.statusbar_visible = utils_get_setting_boolean(config, PACKAGE, "statusbar_visible", TRUE);
 	prefs.tab_order_ltr = utils_get_setting_boolean(config, PACKAGE, "tab_order_ltr", TRUE);
 	prefs.show_notebook_tabs = utils_get_setting_boolean(config, PACKAGE, "show_notebook_tabs", TRUE);
@@ -439,21 +491,26 @@ static void load_dialog_prefs(GKeyFile *config)
 	prefs.editor_font = utils_get_setting_string(config, PACKAGE, "editor_font", GEANY_DEFAULT_FONT_EDITOR);
 	prefs.tagbar_font = utils_get_setting_string(config, PACKAGE, "tagbar_font", GEANY_DEFAULT_FONT_SYMBOL_LIST);
 	prefs.msgwin_font = utils_get_setting_string(config, PACKAGE, "msgwin_font", GEANY_DEFAULT_FONT_MSG_WINDOW);
+#if GTK_CHECK_VERSION(2, 12, 0)
+	prefs.show_symbol_list_expanders = utils_get_setting_boolean(config, PACKAGE, "show_symbol_list_expanders", TRUE);
+#endif
 
 	// display, editor
 	editor_prefs.long_line_type = utils_get_setting_integer(config, PACKAGE, "long_line_type", 0);
 	editor_prefs.long_line_color = utils_get_setting_string(config, PACKAGE, "long_line_color", "#C2EBC2");
 	editor_prefs.long_line_column = utils_get_setting_integer(config, PACKAGE, "long_line_column", 72);
-	editor_prefs.autocompletion_max_height = utils_get_setting_integer(config, PACKAGE, "autocompletion_max_height", GEANY_MAX_AUTOCOMPLETE_HEIGHT);
+	editor_prefs.symbolcompletion_min_chars = utils_get_setting_integer(config, PACKAGE, "symbolcompletion_min_chars", GEANY_MIN_SYMBOLLIST_CHARS);
+	editor_prefs.symbolcompletion_max_height = utils_get_setting_integer(config, PACKAGE, "symbolcompletion_max_height", GEANY_MAX_SYMBOLLIST_HEIGHT);
 	editor_prefs.line_wrapping = utils_get_setting_boolean(config, PACKAGE, "line_breaking", FALSE); // default is off for better performance
 	editor_prefs.indent_mode = utils_get_setting_integer(config, PACKAGE, "indent_mode", INDENT_CURRENTCHARS);
+	editor_prefs.detect_tab_mode = utils_get_setting_integer(config, PACKAGE, "check_detect_indent", FALSE);
 	editor_prefs.use_tab_to_indent = utils_get_setting_boolean(config, PACKAGE, "use_tab_to_indent", FALSE);
 	editor_prefs.use_indicators = utils_get_setting_boolean(config, PACKAGE, "use_indicators", TRUE);
 	editor_prefs.show_indent_guide = utils_get_setting_boolean(config, PACKAGE, "show_indent_guide", FALSE);
 	editor_prefs.show_white_space = utils_get_setting_boolean(config, PACKAGE, "show_white_space", FALSE);
 	editor_prefs.show_line_endings = utils_get_setting_boolean(config, PACKAGE, "show_line_endings", FALSE);
 	editor_prefs.auto_close_xml_tags = utils_get_setting_boolean(config, PACKAGE, "auto_close_xml_tags", TRUE);
-	editor_prefs.auto_complete_constructs = utils_get_setting_boolean(config, PACKAGE, "auto_complete_constructs", TRUE);
+	editor_prefs.complete_snippets = utils_get_setting_boolean(config, PACKAGE, "complete_snippets", TRUE);
 	editor_prefs.auto_complete_symbols = utils_get_setting_boolean(config, PACKAGE, "auto_complete_symbols", TRUE);
 	editor_prefs.folding = utils_get_setting_boolean(config, PACKAGE, "use_folding", TRUE);
 	editor_prefs.unfold_all_children = utils_get_setting_boolean(config, PACKAGE, "unfold_all_children", FALSE);
@@ -465,6 +522,7 @@ static void load_dialog_prefs(GKeyFile *config)
 	editor_prefs.use_tabs = utils_get_setting_boolean(config, PACKAGE, "pref_editor_use_tabs", TRUE);
 	editor_prefs.disable_dnd = utils_get_setting_boolean(config, PACKAGE, "pref_editor_disable_dnd", FALSE);
 	editor_prefs.smart_home_key = utils_get_setting_boolean(config, PACKAGE, "pref_editor_smart_home_key", TRUE);
+	editor_prefs.newline_strip = utils_get_setting_boolean(config, PACKAGE, "pref_editor_newline_strip", FALSE);
 	editor_prefs.use_gtk_word_boundaries = utils_get_setting_boolean(config, PACKAGE, "use_gtk_word_boundaries", TRUE);
 	editor_prefs.auto_complete_whilst_editing = utils_get_setting_boolean(config, PACKAGE, "auto_complete_whilst_editing", FALSE);
 
@@ -505,6 +563,7 @@ static void load_dialog_prefs(GKeyFile *config)
 	prefs.toolbar_show_search = utils_get_setting_boolean(config, PACKAGE, "pref_toolbar_show_search", TRUE);
 	prefs.toolbar_show_goto = utils_get_setting_boolean(config, PACKAGE, "pref_toolbar_show_goto", TRUE);
 	prefs.toolbar_show_zoom = utils_get_setting_boolean(config, PACKAGE, "pref_toolbar_show_zoom", FALSE);
+	prefs.toolbar_show_indent = utils_get_setting_boolean(config, PACKAGE, "pref_toolbar_show_indent", FALSE);
 	prefs.toolbar_show_compile = utils_get_setting_boolean(config, PACKAGE, "pref_toolbar_show_compile", TRUE);
 	prefs.toolbar_show_undo = utils_get_setting_boolean(config, PACKAGE, "pref_toolbar_show_undo", FALSE);
 	prefs.toolbar_show_navigation = utils_get_setting_boolean(config, PACKAGE, "pref_toolbar_show_navigation", TRUE);
@@ -542,10 +601,11 @@ static void load_dialog_prefs(GKeyFile *config)
 		vc->font = utils_get_setting_string(config, "VTE", "font", "Monospace 10");
 		vc->scroll_on_key = utils_get_setting_boolean(config, "VTE", "scroll_on_key", TRUE);
 		vc->scroll_on_out = utils_get_setting_boolean(config, "VTE", "scroll_on_out", TRUE);
+		vc->enable_bash_keys = utils_get_setting_boolean(config, "VTE", "enable_bash_keys", FALSE);
 		vc->ignore_menu_bar_accel = utils_get_setting_boolean(config, "VTE", "ignore_menu_bar_accel", FALSE);
 		vc->follow_path = utils_get_setting_boolean(config, "VTE", "follow_path", FALSE);
 		vc->run_in_vte = utils_get_setting_boolean(config, "VTE", "run_in_vte", FALSE);
-		vc->enable_bash_keys = utils_get_setting_boolean(config, "VTE", "enable_bash_keys", TRUE);
+		vc->skip_run_script = utils_get_setting_boolean(config, "VTE", "skip_run_script", FALSE);
 		vc->scrollback_lines = utils_get_setting_integer(config, "VTE", "scrollback_lines", 500);
 		vc->colour_fore = g_new0(GdkColor, 1);
 		vc->colour_back = g_new0(GdkColor, 1);
@@ -585,6 +645,13 @@ static void load_dialog_prefs(GKeyFile *config)
 	prefs.tools_browser_cmd = utils_get_setting_string(config, "tools", "browser_cmd", tmp_string);
 	g_free(tmp_string);
 
+	tmp_string = g_find_program_in_path(GEANY_DEFAULT_TOOLS_GREP);
+	prefs.tools_grep_cmd = utils_get_setting_string(config, "tools", "grep_cmd", tmp_string);
+	g_free(tmp_string);
+
+	prefs.context_action_cmd = utils_get_setting_string(config, PACKAGE, "context_action_cmd", "");
+
+	// printing
 	tmp_string2 = g_find_program_in_path(GEANY_DEFAULT_TOOLS_PRINTCMD);
 #ifdef G_OS_WIN32
 	// single quote paths on Win32 for g_spawn_command_line_async
@@ -592,18 +659,32 @@ static void load_dialog_prefs(GKeyFile *config)
 #else
 	tmp_string = g_strconcat(tmp_string2, " %f", NULL);
 #endif
-	prefs.tools_print_cmd = utils_get_setting_string(config, "tools", "print_cmd", tmp_string);
+	printing_prefs.external_print_cmd = utils_get_setting_string(config, "printing", "print_cmd", tmp_string);
 	g_free(tmp_string);
 	g_free(tmp_string2);
 
-	tmp_string = g_find_program_in_path(GEANY_DEFAULT_TOOLS_GREP);
-	prefs.tools_grep_cmd = utils_get_setting_string(config, "tools", "grep_cmd", tmp_string);
-	g_free(tmp_string);
-
-	prefs.context_action_cmd = utils_get_setting_string(config, PACKAGE, "context_action_cmd", "");
+#if GTK_CHECK_VERSION(2, 10, 0)
+	printing_prefs.use_gtk_printing = utils_get_setting_boolean(config, "printing", "use_gtk_printing", TRUE);
+#else
+	printing_prefs.use_gtk_printing = utils_get_setting_boolean(config, "printing", "use_gtk_printing", FALSE);
+#endif
+	printing_prefs.print_line_numbers = utils_get_setting_boolean(config, "printing", "print_line_numbers", TRUE);
+	printing_prefs.print_page_numbers = utils_get_setting_boolean(config, "printing", "print_page_numbers", TRUE);
+	printing_prefs.print_page_header = utils_get_setting_boolean(config, "printing", "print_page_header", TRUE);
+	printing_prefs.page_header_basename = utils_get_setting_boolean(config, "printing", "page_header_basename", FALSE);
+	printing_prefs.page_header_datefmt = utils_get_setting_string(config, "printing", "page_header_datefmt", "%c");
 }
 
 
+#ifdef HAVE_PLUGINS
+static void load_plugin_prefs(GKeyFile *config)
+{
+	prefs.load_plugins = utils_get_setting_boolean(config, "plugins", "load_plugins", TRUE);
+	app->active_plugins = g_key_file_get_string_list(config, "plugins", "active_plugins", NULL, NULL);
+}
+#endif
+
+
 static void load_ui_prefs(GKeyFile *config)
 {
 	gint *geo;
@@ -635,11 +716,15 @@ static void load_ui_prefs(GKeyFile *config)
 		ui_prefs.geometry[3] = geo[3];
 		ui_prefs.geometry[4] = geo[4];
 
-		// don't use insane values
-		for (i = 0; i < 4; i++)
+		// don't use insane values but when main windows was maximized last time, pos might be
+		// negative at least on Windows for some reason
+		if (ui_prefs.geometry[4] != 1)
 		{
-			if (ui_prefs.geometry[i] < -1)
-				ui_prefs.geometry[i] = -1;
+			for (i = 0; i < 4; i++)
+			{
+				if (ui_prefs.geometry[i] < -1)
+					ui_prefs.geometry[i] = -1;
+			}
 		}
 	}
 	hpan_position = utils_get_setting_integer(config, PACKAGE, "treeview_position", 156);
@@ -654,6 +739,47 @@ static void load_ui_prefs(GKeyFile *config)
 }
 
 
+/*
+ * Save current session in default configuration file
+ */
+void configuration_save_default_session()
+{
+	gchar *configfile = g_strconcat(app->configdir, G_DIR_SEPARATOR_S, "geany.conf", NULL);
+	gchar *data;
+	GKeyFile *config = g_key_file_new();
+
+	g_key_file_load_from_file(config, configfile, G_KEY_FILE_NONE, NULL);
+
+	if (cl_options.load_session)
+		configuration_save_session_files(config);
+
+	// write the file
+	data = g_key_file_to_data(config, NULL, NULL);
+	utils_write_file(configfile, data);
+	g_free(data);
+
+	g_key_file_free(config);
+	g_free(configfile);
+}
+
+
+/*
+ * Only reload the session part of the default configuration
+ */
+void configuration_reload_default_session()
+{
+	gchar *configfile = g_strconcat(app->configdir, G_DIR_SEPARATOR_S, "geany.conf", NULL);
+	GKeyFile *config = g_key_file_new();
+
+	g_key_file_load_from_file(config, configfile, G_KEY_FILE_NONE, NULL);
+
+	configuration_load_session_files(config);
+
+	g_key_file_free(config);
+	g_free(configfile);
+}
+
+
 gboolean configuration_load()
 {
 	gchar *configfile = g_strconcat(app->configdir, G_DIR_SEPARATOR_S, "geany.conf", NULL);
@@ -669,9 +795,12 @@ gboolean configuration_load()
 	g_key_file_load_from_file(config, configfile, G_KEY_FILE_NONE, NULL);
 
 	load_dialog_prefs(config);
+#ifdef HAVE_PLUGINS
+	load_plugin_prefs(config);
+#endif
 	load_ui_prefs(config);
 	project_load_prefs(config);
-	load_file_lists(config);
+	configuration_load_session_files(config);
 
 	g_key_file_free(config);
 	g_free(configfile);
@@ -679,63 +808,62 @@ gboolean configuration_load()
 }
 
 
-// open session files
+/* Open session files
+ * Note: notebook page switch handler and adding to recent files list is always disabled
+ * for all files opened within this function */
 gboolean configuration_open_files()
 {
 	gint i;
-	guint x, pos, y, len;
+	guint pos;
 	gboolean ret = FALSE, failure = FALSE;
 
+	// necessary to set it to TRUE for project session support
+	main_status.opening_session_files = TRUE;
+
 	document_delay_colourise();
 
 	i = prefs.tab_order_ltr ? 0 : (session_files->len - 1);
 	while (TRUE)
 	{
-		gchar *tmp = g_ptr_array_index(session_files, i);
+		gchar **tmp = g_ptr_array_index(session_files, i);
 
-		if (tmp && *tmp)
+		if (tmp != NULL && g_strv_length(tmp) == 8)
 		{
-			const gchar *file;
-			gchar *locale_filename, **array;
-			gint uid = -1;
-			x = 0;
-			y = 0;
+			gchar *locale_filename;
+			gint ft_uid, enc_idx;
+			gboolean ro, use_tabs, auto_indent, line_wrapping;
 
-			// yes it is :, it should be a ;, but now it is too late to change it
-			array = g_strsplit(tmp, ":", 3);
-			len = g_strv_length(array);
-
-			// read position
-			if (len > 0 && array[0]) pos = atoi(array[0]);
-			else pos = 0;
-
-			// read filetype (only if there are more than two fields, otherwise we have the old format)
-			if (len > 2 && array[1])
-			{
-				uid = atoi(array[1]);
-				file = array[2];
-			}
-			else file = array[1];
-
-			// try to get the locale equivalent for the filename, fallback to filename if error
-			locale_filename = utils_get_locale_from_utf8(file);
+			pos = atoi(tmp[0]);
+			ft_uid = atoi(tmp[1]);
+			ro = atoi(tmp[2]);
+			enc_idx = atoi(tmp[3]);
+			use_tabs = atoi(tmp[4]);
+			auto_indent = atoi(tmp[5]);
+			line_wrapping = atoi(tmp[6]);
+			// try to get the locale equivalent for the filename
+			locale_filename = utils_get_locale_from_utf8(tmp[7]);
 
 			if (g_file_test(locale_filename, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_SYMLINK))
 			{
-				filetype *ft = filetypes_get_from_uid(uid);
-				document_open_file_full(-1, locale_filename, pos, FALSE, ft, NULL);
+				filetype *ft = filetypes_get_from_uid(ft_uid);
+				gint new_idx = document_open_file_full(
+					-1, locale_filename, pos, ro, ft,
+					(enc_idx >= 0 && enc_idx < GEANY_ENCODINGS_MAX) ?
+						encodings[enc_idx].charset : NULL);
+				document_set_use_tabs(new_idx, use_tabs);
+				doc_list[new_idx].auto_indent = auto_indent;
+				document_set_line_wrapping(new_idx, line_wrapping);
 				ret = TRUE;
 			}
 			else
 			{
 				failure = TRUE;
-				geany_debug("Could not find file '%s'.", file);
+				geany_debug("Could not find file '%s'.", tmp[7]);
 			}
 
-			g_strfreev(array);
 			g_free(locale_filename);
 		}
-		g_free(tmp);
+		g_strfreev(tmp);
 
 		if (prefs.tab_order_ltr)
 		{
@@ -751,11 +879,20 @@ gboolean configuration_open_files()
 	document_colourise_new();
 
 	g_ptr_array_free(session_files, TRUE);
+	session_files = NULL;
 
 	if (failure)
-		msgwin_status_add(_("Failed to load one or more session files."));
+		ui_set_statusbar(TRUE, _("Failed to load one or more session files."));
 	else if (session_notebook_page >= 0)
+	{
+		// exlicitly allow notebook switch page callback to be called for window title,
+		// encoding settings and so other things
+		main_status.opening_session_files = FALSE;
+		/// TODO if session_notebook_page is equal to the current notebook tab(the last opened)
+		/// the notebook switch page callback isn't triggered and e.g. menu items are not updated
 		gtk_notebook_set_current_page(GTK_NOTEBOOK(app->notebook), session_notebook_page);
+	}
+	main_status.opening_session_files = FALSE;
 	return ret;
 }
 
@@ -895,13 +1032,10 @@ void configuration_read_filetype_extensions()
 }
 
 
-void configuration_read_autocompletions()
+void configuration_read_snippets()
 {
 	gsize i, j, len = 0, len_keys = 0;
-	gchar *sysconfigfile = g_strconcat(app->datadir, G_DIR_SEPARATOR_S,
-		"autocomplete.conf", NULL);
-	gchar *userconfigfile = g_strconcat(app->configdir, G_DIR_SEPARATOR_S,
-		"autocomplete.conf", NULL);
+	gchar *sysconfigfile, *userconfigfile;
 	gchar **groups_user, **groups_sys;
 	gchar **keys_user, **keys_sys;
 	gchar *value;
@@ -909,11 +1043,20 @@ void configuration_read_autocompletions()
 	GKeyFile *userconfig = g_key_file_new();
 	GHashTable *tmp;
 
+	sysconfigfile = g_strconcat(app->datadir, G_DIR_SEPARATOR_S, "snippets.conf", NULL);
+	userconfigfile = g_strconcat(app->configdir, G_DIR_SEPARATOR_S, "snippets.conf", NULL);
+
+	// check for old autocomplete.conf files (backwards compatibility)
+	if (! g_file_test(userconfigfile, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_SYMLINK))
+		setptr(userconfigfile,
+			g_strconcat(app->configdir, G_DIR_SEPARATOR_S, "autocomplete.conf", NULL));
+
+	// load the actual config files
 	g_key_file_load_from_file(sysconfig, sysconfigfile, G_KEY_FILE_NONE, NULL);
 	g_key_file_load_from_file(userconfig, userconfigfile, G_KEY_FILE_NONE, NULL);
 
 	// keys are strings, values are GHashTables, so use g_free and g_hash_table_destroy
-	editor_prefs.auto_completions =
+	editor_prefs.snippets =
 		g_hash_table_new_full(g_str_hash, g_str_equal, g_free, (GDestroyNotify) g_hash_table_destroy);
 
 	// first read all globally defined auto completions
@@ -923,7 +1066,7 @@ void configuration_read_autocompletions()
 		keys_sys = g_key_file_get_keys(sysconfig, groups_sys[i], &len_keys, NULL);
 		// create new hash table for the read section (=> filetype)
 		tmp = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
-		g_hash_table_insert(editor_prefs.auto_completions, g_strdup(groups_sys[i]), tmp);
+		g_hash_table_insert(editor_prefs.snippets, g_strdup(groups_sys[i]), tmp);
 
 		for (j = 0; j < len_keys; j++)
 		{
@@ -939,11 +1082,11 @@ void configuration_read_autocompletions()
 	{
 		keys_user = g_key_file_get_keys(userconfig, groups_user[i], &len_keys, NULL);
 
-		tmp = g_hash_table_lookup(editor_prefs.auto_completions, groups_user[i]);
+		tmp = g_hash_table_lookup(editor_prefs.snippets, groups_user[i]);
 		if (tmp == NULL)
 		{	// new key found, create hash table
 			tmp = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
-			g_hash_table_insert(editor_prefs.auto_completions, g_strdup(groups_user[i]), tmp);
+			g_hash_table_insert(editor_prefs.snippets, g_strdup(groups_user[i]), tmp);
 		}
 		for (j = 0; j < len_keys; j++)
 		{
