@@ -1,8 +1,8 @@
 /*
  *      main.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: main.c 1923 2007-10-01 12:22:44Z eht16 $
+ * $Id: main.c 2190 2008-01-25 13:14:38Z ntrel $
  */
 
 /*
@@ -40,6 +40,7 @@
 #endif
 
 #include "main.h"
+#include "prefix.h"
 #include "prefs.h"
 #include "interface.h"
 #include "support.h"
@@ -67,6 +68,7 @@
 #include "tools.h"
 #include "navqueue.h"
 #include "plugins.h"
+#include "printing.h"
 
 #ifdef HAVE_SOCKET
 # include "socket.h"
@@ -106,7 +108,10 @@ static gboolean generate_datafiles = FALSE;
 static gboolean generate_tags = FALSE;
 static gboolean no_preprocessing = FALSE;
 static gboolean ft_names = FALSE;
+static gboolean print_prefix = FALSE;
+#ifdef HAVE_PLUGINS
 static gboolean no_plugins = FALSE;
+#endif
 
 // in alphabetical order of short options
 static GOptionEntry entries[] =
@@ -124,7 +129,10 @@ static GOptionEntry entries[] =
 	{ "line", 'l', 0, G_OPTION_ARG_INT, &cl_options.goto_line, N_("Set initial line number for the first opened file"), NULL },
 	{ "no-msgwin", 'm', 0, G_OPTION_ARG_NONE, &no_msgwin, N_("Don't show message window at startup"), NULL },
 	{ "no-ctags", 'n', 0, G_OPTION_ARG_NONE, &ignore_global_tags, N_("Don't load auto completion data (see documentation)"), NULL },
+#ifdef HAVE_PLUGINS
 	{ "no-plugins", 'p', 0, G_OPTION_ARG_NONE, &no_plugins, N_("Don't load plugins"), NULL },
+#endif
+	{ "print-prefix", 0, 0, G_OPTION_ARG_NONE, &print_prefix, N_("Print Geany's installation prefix"), NULL },
 	{ "no-session", 's', G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, &cl_options.load_session, N_("don't load the previous session's files"), NULL },
 #ifdef HAVE_VTE
 	{ "no-terminal", 't', 0, G_OPTION_ARG_NONE, &no_vte, N_("Don't load terminal support"), NULL },
@@ -150,6 +158,7 @@ void geany_debug(gchar const *format, ...)
 	}
 }
 
+
 /* special things for the initial setup of the checkboxes and related stuff
  * an action on a setting is only performed if the setting is not equal to the program default
  * (all the following code is not perfect but it works for the moment) */
@@ -178,7 +187,7 @@ static void apply_settings(void)
 		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget(app->window, "menu_show_sidebar1")), FALSE);
 		app->ignore_callback = FALSE;
 	}
-	ui_treeviews_show_hide(TRUE);
+	ui_sidebar_show_hide();
 	// sets the icon style of the toolbar
 	switch (prefs.toolbar_icon_style)
 	{
@@ -267,8 +276,9 @@ static void main_init(void)
 	// inits
 	app->window				= NULL;
 	app->project			= NULL;
+	app->active_plugins		= NULL;
 	ui_widgets.open_fontsel		= NULL;
-	ui_widgets.open_colorsel		= NULL;
+	ui_widgets.open_colorsel	= NULL;
 	ui_widgets.open_filesel		= NULL;
 	ui_widgets.save_filesel		= NULL;
 	ui_widgets.prefs_dialog		= NULL;
@@ -277,7 +287,7 @@ static void main_init(void)
 	prefs.tab_order_ltr		= FALSE;
 	main_status.quitting			= FALSE;
 	app->ignore_callback	= FALSE;
-	app->tm_workspace				= tm_get_workspace();
+	app->tm_workspace		= tm_get_workspace();
 	ui_prefs.recent_queue				= g_queue_new();
 	main_status.opening_session_files		= FALSE;
 
@@ -289,13 +299,14 @@ static void main_init(void)
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(ui_widgets.recent_files_menuitem),
 							ui_widgets.recent_files_menubar);
 
-	// store important pointers in the GeanyApp structure
+	// store important pointers for later reference
 	app->toolbar = lookup_widget(app->window, "toolbar1");
 	app->treeview_notebook = lookup_widget(app->window, "notebook3");
 	app->notebook = lookup_widget(app->window, "notebook1");
 	app->statusbar = lookup_widget(app->window, "statusbar");
 	app->popup_menu = create_edit_menu1();
 	ui_widgets.toolbar_menu = create_toolbar_popup_menu1();
+	ui_widgets.print_page_setup = lookup_widget(app->window, "page_setup1");
 	ui_widgets.popup_goto_items[0] = lookup_widget(app->popup_menu, "goto_tag_definition1");
 	ui_widgets.popup_goto_items[1] = lookup_widget(app->popup_menu, "goto_tag_declaration1");
 	ui_widgets.popup_goto_items[2] = lookup_widget(app->popup_menu, "find_usage1");
@@ -324,6 +335,12 @@ static void main_init(void)
 	gtk_widget_set_name(app->window, "GeanyMainWindow");
 	gtk_widget_set_name(ui_widgets.toolbar_menu, "GeanyToolbarMenu");
 	gtk_widget_set_name(app->popup_menu, "GeanyEditMenu");
+
+#if ! GTK_CHECK_VERSION(2, 10, 0)
+	// hide Page setup menu item, it isn't supported with non-GTK printing
+	gtk_widget_hide(ui_widgets.print_page_setup);
+#endif
+
 }
 
 
@@ -364,8 +381,8 @@ static void setup_paths()
 
 	g_free(install_dir);
 #else
-	data_dir = g_strdup(PACKAGE_DATA_DIR "/" PACKAGE "/"); // e.g. /usr/share/geany
-	doc_dir = g_strdup(PACKAGE_DATA_DIR "/doc/" PACKAGE "/html/");
+	data_dir = g_strconcat(GEANY_DATADIR, "/geany/", NULL); // e.g. /usr/share/geany
+	doc_dir = g_strconcat(GEANY_DATADIR, "/doc/geany/html/", NULL);
 #endif
 
 	// convert path names to locale encoding
@@ -392,7 +409,7 @@ static void locale_init()
 	locale_dir = g_strconcat(install_dir, "\\lib\\locale", NULL);
 	g_free(install_dir);
 #else
-	locale_dir = g_strdup(PACKAGE_LOCALE_DIR);
+	locale_dir = g_strdup(GEANY_LOCALEDIR);
 #endif
 
 	bindtextdomain(GETTEXT_PACKAGE, locale_dir);
@@ -405,20 +422,27 @@ static void locale_init()
 
 static void parse_command_line_options(gint *argc, gchar ***argv)
 {
-	GOptionContext *context;
 	GError *error = NULL;
+	GOptionContext *context;
+
+	context = g_option_context_new(_("[FILES...]"));
+	g_option_context_add_main_entries(context, entries, GETTEXT_PACKAGE);
+	g_option_group_set_translation_domain(g_option_context_get_main_group(context), GETTEXT_PACKAGE);
+	g_option_context_add_group(context, gtk_get_option_group(FALSE));
+	g_option_context_parse(context, argc, argv, &error);
+	g_option_context_free(context);
 
 	// first initialise cl_options fields with default values
 	cl_options.load_session = TRUE;
 	cl_options.goto_line = -1;
 	cl_options.goto_column = -1;
 
-	context = g_option_context_new(_("[FILES...]"));
-	g_option_context_add_main_entries(context, entries, GETTEXT_PACKAGE);
-	g_option_group_set_translation_domain(g_option_context_get_main_group(context), GETTEXT_PACKAGE);
-	g_option_context_add_group(context, gtk_get_option_group(TRUE));
-	g_option_context_parse(context, argc, argv, &error);
-	g_option_context_free(context);
+	if (error != NULL)
+	{
+		g_printerr("Geany: %s\n", error->message);
+		g_error_free(error);
+		exit(1);
+	}
 
 	if (show_version)
 	{
@@ -431,6 +455,15 @@ static void parse_command_line_options(gint *argc, gchar ***argv)
 		exit(0);
 	}
 
+	if (print_prefix)
+	{
+		printf("%s\n", GEANY_PREFIX);
+		printf("%s\n", GEANY_DATADIR);
+		printf("%s\n", GEANY_LIBDIR);
+		printf("%s\n", GEANY_LOCALEDIR);
+		exit(0);
+	}
+
 #ifdef GEANY_DEBUG
 	app->debug_mode = TRUE;
 	geany_debug("debug mode built in (can't be disabled)");
@@ -438,13 +471,34 @@ static void parse_command_line_options(gint *argc, gchar ***argv)
 	app->debug_mode = debug_mode;
 #endif
 
+#ifdef G_OS_WIN32
+	win32_init_debug_code();
+#endif
+
 	if (alternate_config)
 	{
 		geany_debug("alternate config: %s", alternate_config);
 		app->configdir = alternate_config;
 	}
 	else
-		app->configdir = g_strconcat(GEANY_HOME_DIR, G_DIR_SEPARATOR_S, ".", PACKAGE, NULL);
+	{
+#ifdef G_OS_WIN32
+		gchar *appdata;
+
+		appdata = win32_get_appdata_folder();
+		if (appdata != NULL)
+		{
+			app->configdir = g_strconcat(appdata, G_DIR_SEPARATOR_S, "Geany", NULL);
+			g_free(appdata);
+		}
+		else
+		{
+			app->configdir = g_strconcat(g_get_home_dir(), G_DIR_SEPARATOR_S, "Geany", NULL);
+		}
+#else
+		app->configdir = g_strconcat(g_get_home_dir(), G_DIR_SEPARATOR_S, ".geany", NULL);
+#endif
+	}
 
 #ifdef GEANY_DEBUG
 	if (generate_datafiles)
@@ -487,6 +541,12 @@ static void parse_command_line_options(gint *argc, gchar ***argv)
 	vte_info.lib_vte = lib_vte;
 #endif
 	cl_options.ignore_global_tags = ignore_global_tags;
+
+	if (! gtk_init_check(NULL, NULL))
+	{	// check whether we have a valid X display and exit if not
+		g_printerr("Geany: cannot open display\n");
+		exit(1);
+	}
 }
 
 
@@ -500,7 +560,7 @@ static gint setup_config_dir()
 	app->configdir = utils_get_locale_from_utf8(app->configdir);
 	g_free(tmp);
 
-	mkdir_result = utils_make_settings_dir(app->configdir, app->datadir, app->docdir);
+	mkdir_result = utils_make_settings_dir();
 	if (mkdir_result != 0)
 	{
 		if (! dialogs_show_question(
@@ -561,7 +621,7 @@ static gboolean open_cl_files(gint argc, gchar **argv)
 
 			g_printerr(msg, filename);	// also print to the terminal
 			g_printerr("\n");
-			msgwin_status_add(msg, filename);
+			ui_set_statusbar(TRUE, msg, filename);
 		}
 		g_free(filename);
 	}
@@ -595,7 +655,9 @@ static void load_settings()
 #endif
 	if (no_msgwin) ui_prefs.msgwindow_visible = FALSE;
 
+#ifdef HAVE_PLUGINS
 	want_plugins = prefs.load_plugins && !no_plugins;
+#endif
 }
 
 
@@ -614,8 +676,6 @@ gint main(gint argc, gchar **argv)
 	locale_init();
 	parse_command_line_options(&argc, &argv);
 
-	gtk_set_locale();
-
 	signal(SIGTERM, signal_cb);
 #ifdef G_OS_UNIX
 	// SIGQUIT is used to kill spawned children and we get also this signal, so ignore
@@ -654,7 +714,6 @@ gint main(gint argc, gchar **argv)
 		VERSION,
 		gtk_major_version, gtk_minor_version, gtk_micro_version,
 		glib_major_version, glib_minor_version, glib_micro_version);
-	gtk_init(&argc, &argv);
 
 	// inits
 	main_init();
@@ -678,7 +737,7 @@ gint main(gint argc, gchar **argv)
 	document_init_doclist();
 	treeviews_init();
 	configuration_read_filetype_extensions();
-	configuration_read_autocompletions();
+	configuration_read_snippets();
 
 	// set window icon
 	{
@@ -705,9 +764,9 @@ gint main(gint argc, gchar **argv)
 #endif
 	ui_create_recent_menu();
 
-	msgwin_status_add(_("This is Geany %s."), VERSION);
+	ui_set_statusbar(TRUE, _("This is Geany %s."), VERSION);
 	if (config_dir_result != 0)
-		msgwin_status_add(_("Configuration directory could not be created (%s)."),
+		ui_set_statusbar(TRUE, _("Configuration directory could not be created (%s)."),
 			g_strerror(config_dir_result));
 
 	// apply all configuration options
@@ -727,7 +786,7 @@ gint main(gint argc, gchar **argv)
 		{
 			load_project_file();
 
-			// load session files
+			// load session files into tabs, as they are found in the session_files variable
 			if (! configuration_open_files())
 			{
 				ui_update_popup_copy_items(-1);
@@ -784,11 +843,12 @@ void main_quit()
 	if (want_plugins)
 		plugins_free();
 #endif
+
 	navqueue_free();
 	keybindings_free();
 	filetypes_save_commands();
 	filetypes_free_types();
-	styleset_free_styles();
+	highlighting_free_styles();
 	templates_free_templates();
 	msgwin_finalize();
 	search_finalize();
@@ -796,7 +856,7 @@ void main_quit()
 	document_finalize();
 	symbols_finalize();
 	editor_finalize();
-	if (app->project != NULL) project_close();
+	encodings_finalize();
 
 	tm_workspace_free(TM_WORK_OBJECT(app->tm_workspace));
 	g_free(app->configdir);
@@ -817,9 +877,11 @@ void main_quit()
 	g_free(prefs.tools_make_cmd);
 	g_free(prefs.tools_term_cmd);
 	g_free(prefs.tools_browser_cmd);
-	g_free(prefs.tools_print_cmd);
 	g_free(prefs.tools_grep_cmd);
+	g_free(printing_prefs.external_print_cmd);
+	g_free(printing_prefs.page_header_datefmt);
 	g_strfreev(ui_prefs.custom_commands);
+	g_strfreev(app->active_plugins);
 	while (! g_queue_is_empty(ui_prefs.recent_queue))
 	{
 		g_free(g_queue_pop_tail(ui_prefs.recent_queue));
