@@ -1,8 +1,8 @@
 /*
  *      msgwindow.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: msgwindow.c 1913 2007-09-27 11:39:21Z ntrel $
+ * $Id: msgwindow.c 2153 2008-01-09 13:36:22Z eht16 $
  */
 
 /*
@@ -58,9 +58,6 @@ typedef struct
 	gint file_idx;			// idx of the field where the filename is or -1
 } ParseData;
 
-static const GdkColor dark = {0, 58832, 58832, 58832};
-static const GdkColor white = {0, 65535, 65535, 65535};
-
 MessageWindow msgwindow;
 
 
@@ -109,11 +106,11 @@ static void prepare_status_tree_view(void)
 	GtkTreeViewColumn *column;
 	PangoFontDescription *pfd;
 
-	msgwindow.store_status = gtk_list_store_new(2, GDK_TYPE_COLOR, G_TYPE_STRING);
+	msgwindow.store_status = gtk_list_store_new(1, G_TYPE_STRING);
 	gtk_tree_view_set_model(GTK_TREE_VIEW(msgwindow.tree_status), GTK_TREE_MODEL(msgwindow.store_status));
 
 	renderer = gtk_cell_renderer_text_new();
-	column = gtk_tree_view_column_new_with_attributes(_("Status messages"), renderer, "background-gdk", 0, "text", 1, NULL);
+	column = gtk_tree_view_column_new_with_attributes(_("Status messages"), renderer, "text", 0, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(msgwindow.tree_status), column);
 
 	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(msgwindow.tree_status), FALSE);
@@ -136,14 +133,14 @@ static void prepare_msg_tree_view(void)
 	GtkTreeSelection *selection;
 	PangoFontDescription *pfd;
 
-	// doc idx, line, bg, fg, str
-	msgwindow.store_msg = gtk_list_store_new(5, G_TYPE_INT, G_TYPE_INT,
-		GDK_TYPE_COLOR, GDK_TYPE_COLOR, G_TYPE_STRING);
+	// doc idx, line, fg, str
+	msgwindow.store_msg = gtk_list_store_new(4, G_TYPE_INT, G_TYPE_INT,
+		GDK_TYPE_COLOR, G_TYPE_STRING);
 	gtk_tree_view_set_model(GTK_TREE_VIEW(msgwindow.tree_msg), GTK_TREE_MODEL(msgwindow.store_msg));
 
 	renderer = gtk_cell_renderer_text_new();
 	column = gtk_tree_view_column_new_with_attributes(NULL, renderer,
-		"background-gdk", 2, "foreground-gdk", 3, "text", 4, NULL);
+		"foreground-gdk", 2, "text", 3, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(msgwindow.tree_msg), column);
 
 	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(msgwindow.tree_msg), FALSE);
@@ -277,45 +274,33 @@ void msgwin_msg_add_fmt(gint msg_color, gint line, gint idx, const gchar *format
 void msgwin_msg_add(gint msg_color, gint line, gint idx, const gchar *string)
 {
 	GtkTreeIter iter;
-	static gint state = 0;
 	const GdkColor *color = get_color(msg_color);
+	gchar *tmp;
 
 	if (! ui_prefs.msgwindow_visible) msgwin_show_hide(TRUE);
 
+	// work around a strange problem when adding very long lines(greater than 4000 bytes)
+	// cut the string to a maximum of 1024 bytes and discard the rest
+	/// TODO find the real cause for the display problem / if it is GtkTreeView file a bug report
+	if (strlen(string) > 1024)
+		tmp = g_strndup(string, 1024);
+	else
+		tmp = g_strdup(string);
+
 	gtk_list_store_append(msgwindow.store_msg, &iter);
-	gtk_list_store_set(msgwindow.store_msg, &iter, 0, line, 1, idx, 2,
-		((state++ % 2) == 0) ? &white : &dark, 3, color, 4, string, -1);
+	gtk_list_store_set(msgwindow.store_msg, &iter, 0, line, 1, idx, 2, color, 3, tmp, -1);
 
 	gtk_widget_set_sensitive(lookup_widget(app->window, "next_message1"), TRUE);
+
+	g_free(tmp);
 }
 
 
 /* Log a status message *without* setting the status bar.
- * This is a temporary function for the plugin API for Geany 0.12.
- * In future, msgwin_status_add() will act like this. */
-void msgwin_status_add_new(const gchar *format, ...)
-{
-	gchar string[512];
-	va_list args;
-	gboolean suppress;
-
-	va_start(args, format);
-	g_vsnprintf(string, 512, format, args);
-	va_end(args);
-
-	// hack to prevent setting the status bar
-	suppress = prefs.suppress_status_messages;
-	prefs.suppress_status_messages = TRUE;
-	msgwin_status_add("%s", string);
-	prefs.suppress_status_messages = suppress;
-}
-
-
-// logs a status message (use ui_set_statusbar() to just display text on the statusbar)
+ * (Use ui_set_statusbar() to display text on the statusbar) */
 void msgwin_status_add(const gchar *format, ...)
 {
 	GtkTreeIter iter;
-	static gint state = 0;
 	gchar string[512];
 	gchar *statusmsg, *time_str;
 	va_list args;
@@ -324,10 +309,6 @@ void msgwin_status_add(const gchar *format, ...)
 	g_vsnprintf(string, 512, format, args);
 	va_end(args);
 
-	// display status message in status bar
-	if (! prefs.suppress_status_messages)
-		ui_set_statusbar("%s", string);
-
 	// add a timestamp to status messages
 	time_str = utils_get_current_time_string();
 	if (time_str == NULL)
@@ -338,8 +319,7 @@ void msgwin_status_add(const gchar *format, ...)
 
 	// add message to Status window
 	gtk_list_store_append(msgwindow.store_status, &iter);
-	gtk_list_store_set(msgwindow.store_status, &iter, 0,
-		((state++ % 2) == 0) ? &white : &dark, 1, statusmsg, -1);
+	gtk_list_store_set(msgwindow.store_status, &iter, 0, statusmsg, -1);
 	g_free(statusmsg);
 
 	if (main_status.main_window_realized)
@@ -377,6 +357,7 @@ on_compiler_treeview_copy_activate     (GtkMenuItem     *menuitem,
 	{
 		case MSG_STATUS:
 		tv = msgwindow.tree_status;
+		str_idx = 0;
 		break;
 
 		case MSG_COMPILER:
@@ -792,7 +773,7 @@ gboolean msgwin_goto_messages_file_line()
 		gint idx, line;
 		gchar *string;
 
-		gtk_tree_model_get(model, &iter, 0, &line, 1, &idx, 4, &string, -1);
+		gtk_tree_model_get(model, &iter, 0, &line, 1, &idx, 3, &string, -1);
 		if (line >= 0 && idx >= 0)
 		{
 			ret = utils_goto_line(idx, line);	// checks valid idx
