@@ -1,8 +1,8 @@
 /*
  *      notebook.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2006-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: notebook.c 1925 2007-10-02 14:23:11Z ntrel $
+ * $Id: notebook.c 2150 2008-01-06 18:11:57Z eht16 $
  */
 
 /*
@@ -32,6 +32,7 @@
 #include "ui_utils.h"
 #include "treeviews.h"
 #include "support.h"
+#include "callbacks.h"
 
 #define GEANY_DND_NOTEBOOK_TAB_TYPE	"geany_dnd_notebook_tab"
 
@@ -300,11 +301,23 @@ static void tab_count_changed()
 	}
 }
 
+gboolean notebook_tab_label_cb(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
+{
+	// toggle additional widgets on double click
+	if (event->type == GDK_2BUTTON_PRESS)
+		on_menu_toggle_all_additional_widgets1_activate(NULL, NULL);
+	// close tab on middle click
+	if (event->button == 2)
+		document_remove(gtk_notebook_page_num(GTK_NOTEBOOK(app->notebook), GTK_WIDGET(user_data)));
+
+	return FALSE;
+}
+
 
 /* Returns page number of notebook page, or -1 on error */
 gint notebook_new_tab(gint doc_idx)
 {
-	GtkWidget *hbox;
+	GtkWidget *hbox, *ebox;
 	gint tabnum;
 	gchar *title;
 	document *this = &(doc_list[doc_idx]);
@@ -317,8 +330,14 @@ gint notebook_new_tab(gint doc_idx)
 
 	this->tab_label = gtk_label_new(title);
 
+	ebox = gtk_event_box_new();
+	GTK_WIDGET_SET_FLAGS(ebox, GTK_NO_WINDOW);
+	g_signal_connect(G_OBJECT(ebox), "button-press-event",
+		G_CALLBACK(notebook_tab_label_cb), page);
+
 	hbox = gtk_hbox_new(FALSE, 2);
-	gtk_box_pack_start(GTK_BOX(hbox), this->tab_label, FALSE, FALSE, 0);
+	gtk_container_add(GTK_CONTAINER(ebox), this->tab_label);
+	gtk_box_pack_start(GTK_BOX(hbox), ebox, FALSE, FALSE, 0);
 
 	if (prefs.show_tab_cross)
 	{
@@ -351,8 +370,6 @@ gint notebook_new_tab(gint doc_idx)
 		g_signal_connect(G_OBJECT(btn), "clicked",
 			G_CALLBACK(notebook_tab_close_clicked_cb), page);
 	}
-	else
-		gtk_widget_set_size_request(hbox, -1, 18); // keep the familiar tab height
 
 	gtk_widget_show_all(hbox);
 
