@@ -1,8 +1,8 @@
 /*
  *      plugins.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2007 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2007-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: plugins.h 1734 2007-07-23 15:41:08Z ntrel $
+ * $Id: plugins.h 2150 2008-01-06 18:11:57Z eht16 $
  */
 
 
@@ -32,6 +32,9 @@ void plugins_init();
 
 void plugins_free();
 
+void plugins_create_active_list();
+
+void plugins_update_tools_menu();
 
 void plugins_update_document_sensitive(gboolean enabled);
 
