@@ -1,8 +1,8 @@
 /*
  *      project.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2007 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2007-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: project.c 1815 2007-08-23 11:34:06Z ntrel $
+ * $Id: project.c 2200 2008-01-31 14:35:03Z eht16 $
  */
 
 /*
@@ -28,6 +28,7 @@
 #include "geany.h"
 
 #include <string.h>
+#include <unistd.h>
 
 #include "project.h"
 #include "dialogs.h"
@@ -36,17 +37,21 @@
 #include "ui_utils.h"
 #include "msgwindow.h"
 #include "main.h"
+#include "keyfile.h"
 #ifdef G_OS_WIN32
 # include "win32.h"
 #endif
 #include "build.h"
+#include "document.h"
+#include "geanyobject.h"
+#include "callbacks.h"
 
 
-ProjectPrefs project_prefs = {NULL};
+ProjectPrefs project_prefs = { NULL, 0 };
 
 static struct
 {
-	gchar *project_file_path;
+	gchar *project_file_path; // in UTF-8
 } local_prefs = {NULL};
 
 
@@ -60,6 +65,7 @@ typedef struct _PropertyDialogElements
 	GtkWidget *description;
 	GtkWidget *file_name;
 	GtkWidget *base_path;
+	GtkWidget *make_in_base_path;
 	GtkWidget *run_cmd;
 	GtkWidget *patterns;
 } PropertyDialogElements;
@@ -67,17 +73,17 @@ typedef struct _PropertyDialogElements
 
 
 static gboolean update_config(const PropertyDialogElements *e);
-static void on_file_save_button_clicked(GtkButton *button, GtkWidget *entry);
-static void on_file_open_button_clicked(GtkButton *button, GtkWidget *entry);
+static void on_file_save_button_clicked(GtkButton *button, PropertyDialogElements *e);
+static void on_file_open_button_clicked(GtkButton *button, PropertyDialogElements *e);
 static gboolean close_open_project();
 static gboolean load_config(const gchar *filename);
-static gboolean write_config();
+static gboolean write_config(gboolean emit_signal);
 static void on_name_entry_changed(GtkEditable *editable, PropertyDialogElements *e);
 static void on_entries_changed(GtkEditable *editable, PropertyDialogElements *e);
 
 
-// avoid using __VA_ARGS__ because older gcc 2.x versions probably don't support C99
-#define SHOW_ERR(args...) dialogs_show_msgbox(GTK_MESSAGE_ERROR, args)
+#define SHOW_ERR(args) dialogs_show_msgbox(GTK_MESSAGE_ERROR, args)
+#define SHOW_ERR1(args,more) dialogs_show_msgbox(GTK_MESSAGE_ERROR, args, more)
 #define MAX_NAME_LEN 50
 // "projects" is part of the default project base path so be careful when translating
 // please avoid special characters and spaces, look at the source for details or ask Frank
@@ -138,7 +144,7 @@ void project_new()
 	gtk_entry_set_width_chars(GTK_ENTRY(e->file_name), 30);
 	button = gtk_button_new();
 	g_signal_connect((gpointer) button, "clicked",
-				G_CALLBACK(on_file_save_button_clicked), e->file_name);
+				G_CALLBACK(on_file_save_button_clicked), e);
 	image = gtk_image_new_from_stock("gtk-open", GTK_ICON_SIZE_BUTTON);
 	gtk_container_add(GTK_CONTAINER(button), image);
 	bbox = gtk_hbox_new(FALSE, 6);
@@ -153,7 +159,8 @@ void project_new()
 	e->base_path = gtk_entry_new();
 	gtk_tooltips_set_tip(tooltips, e->base_path,
 		_("Base directory of all files that make up the project. "
-		"This can be a new path, or an existing directory tree."), NULL);
+		"This can be a new path, or an existing directory tree. "
+		"You can use paths relative to the project filename."), NULL);
 	bbox = ui_path_box_new(_("Choose Project Base Path"),
 		GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, GTK_ENTRY(e->base_path));
 
@@ -199,13 +206,21 @@ static void run_open_dialog(GtkDialog *dialog)
 		{
 			gchar *utf8_filename = utils_get_utf8_from_locale(filename);
 
-			SHOW_ERR(_("Project file \"%s\" could not be loaded."), utf8_filename);
+			SHOW_ERR1(_("Project file \"%s\" could not be loaded."), utf8_filename);
 			gtk_widget_grab_focus(GTK_WIDGET(dialog));
 			g_free(utf8_filename);
 			g_free(filename);
 			goto retry;
 		}
 		g_free(filename);
+		if (project_prefs.project_session)
+		{
+			configuration_open_files();
+			// open a new file if no other file was opened
+			/// TODO refactor the following into a function to be used here and in main()
+			if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(app->notebook)) == 0)
+				document_new_file(NULL, NULL, NULL);
+		}
 	}
 }
 #endif
@@ -219,14 +234,26 @@ void project_open()
 #else
 	GtkWidget *dialog;
 	GtkFileFilter *filter;
+	gchar *locale_path;
 #endif
 	if (! close_open_project()) return;
 
 #ifdef G_OS_WIN32
-	file = win32_show_project_open_dialog(_("Open Project"), dir, FALSE);
+	file = win32_show_project_open_dialog(app->window, _("Open Project"), dir, FALSE, TRUE);
 	if (file != NULL)
 	{
-		load_config(file);
+		// try to load the config
+		if (! project_load_file(file))
+		{
+			SHOW_ERR1(_("Project file \"%s\" could not be loaded."), file);
+		}
+		if (project_prefs.project_session)
+		{
+			configuration_open_files();
+			// open a new file if no other file was opened
+			if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(app->notebook)) == 0)
+				document_new_file(NULL, NULL, NULL);
+		}
 		g_free(file);
 	}
 #else
@@ -256,7 +283,13 @@ void project_open()
 	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filter);
 	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(dialog), filter);
 
-	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), dir);
+	locale_path = utils_get_locale_from_utf8(dir);
+	if (g_file_test(locale_path, G_FILE_TEST_EXISTS) &&
+		g_file_test(locale_path, G_FILE_TEST_IS_DIR))
+	{
+		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), locale_path);
+	}
+	g_free(locale_path);
 
 	gtk_widget_show_all(dialog);
 	run_open_dialog(GTK_DIALOG(dialog));
@@ -273,14 +306,16 @@ static void update_ui()
 }
 
 
-void project_close()
+// open_default will make function reload default session files on close
+void project_close(gboolean open_default)
 {
+	gint i, max = gtk_notebook_get_n_pages(GTK_NOTEBOOK(app->notebook));
 	g_return_if_fail(app->project != NULL);
 
-	/// TODO handle open project files
+	ui_set_statusbar(TRUE, _("Project \"%s\" closed."), app->project->name);
 
-	write_config();
-	msgwin_status_add(_("Project \"%s\" closed."), app->project->name);
+	// use write_config() to save project session files
+	write_config(FALSE);
 
 	g_free(app->project->name);
 	g_free(app->project->description);
@@ -291,11 +326,37 @@ void project_close()
 	g_free(app->project);
 	app->project = NULL;
 
+	if (project_prefs.project_session)
+	{
+		// close all existing tabs first
+		for (i = 0; i < max; i++)
+		{
+			if (! document_remove(0))
+				break;
+		}
+
+		// after closing all tabs let's open the tabs found in the default config
+		if (open_default == TRUE && cl_options.load_session)
+		{
+			configuration_reload_default_session();
+			configuration_open_files();
+			// open a new file if no other file was opened
+			if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(app->notebook)) == 0)
+				document_new_file(NULL, NULL, NULL);
+		}
+	}
+
+	if (geany_object)
+	{
+		g_signal_emit_by_name(geany_object, "project-close");
+	}
+
+	tm_workspace_update(TM_WORK_OBJECT(app->tm_workspace), TRUE, TRUE, FALSE);
 	update_ui();
 }
 
 
-void project_properties()
+static void create_properties_dialog(PropertyDialogElements *e)
 {
 	GtkWidget *vbox;
 	GtkWidget *table;
@@ -305,11 +366,6 @@ void project_properties()
 	GtkWidget *label;
 	GtkWidget *swin;
 	GtkTooltips *tooltips = GTK_TOOLTIPS(lookup_widget(app->window, "tooltips"));
-	PropertyDialogElements *e = g_new(PropertyDialogElements, 1);
-	GeanyProject *p = app->project;
-	gint response;
-
-	g_return_if_fail(app->project != NULL);
 
 	e->dialog = gtk_dialog_new_with_buttons(_("Project Properties"), GTK_WINDOW(app->window),
 										 GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -319,8 +375,6 @@ void project_properties()
 
 	vbox = ui_dialog_vbox_new(GTK_DIALOG(e->dialog));
 
-	entries_modified = FALSE;
-
 	table = gtk_table_new(6, 2, FALSE);
 	gtk_table_set_row_spacings(GTK_TABLE(table), 5);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 10);
@@ -329,7 +383,7 @@ void project_properties()
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
 					(GtkAttachOptions) (GTK_FILL),
 					(GtkAttachOptions) (0), 0, 0);
-	gtk_misc_set_alignment(GTK_MISC(label), 1, 0);
+	gtk_misc_set_alignment(GTK_MISC(label), -1, 0);
 
 	e->name = gtk_entry_new();
 	gtk_entry_set_max_length(GTK_ENTRY(e->name), MAX_NAME_LEN);
@@ -341,7 +395,7 @@ void project_properties()
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
 					(GtkAttachOptions) (GTK_FILL),
 					(GtkAttachOptions) (0), 0, 0);
-	gtk_misc_set_alignment(GTK_MISC(label), 1, 0);
+	gtk_misc_set_alignment(GTK_MISC(label), -1, 0);
 
 	e->file_name = gtk_entry_new();
 	gtk_editable_set_editable(GTK_EDITABLE(e->file_name), FALSE);	// read-only
@@ -353,7 +407,7 @@ void project_properties()
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
 					(GtkAttachOptions) (GTK_FILL),
 					(GtkAttachOptions) (GTK_FILL), 0, 0);
-	gtk_misc_set_alignment(GTK_MISC(label), 1, 0);
+	gtk_misc_set_alignment(GTK_MISC(label), -1, 0);
 
 	e->description = gtk_text_view_new();
 	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(e->description), GTK_WRAP_WORD);
@@ -370,23 +424,29 @@ void project_properties()
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4,
 					(GtkAttachOptions) (GTK_FILL),
 					(GtkAttachOptions) (0), 0, 0);
-	gtk_misc_set_alignment(GTK_MISC(label), 1, 0);
+	gtk_misc_set_alignment(GTK_MISC(label), -1, 0);
 
 	e->base_path = gtk_entry_new();
 	gtk_tooltips_set_tip(tooltips, e->base_path,
-		_("Directory to run Make All from. "
-		"Leave blank to use the default command."), NULL);
+		_("Base directory of all files that make up the project. "
+		"This can be a new path, or an existing directory tree. "
+		"You can use paths relative to the project filename."), NULL);
 	bbox = ui_path_box_new(_("Choose Project Base Path"),
 		GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, GTK_ENTRY(e->base_path));
 	gtk_table_attach(GTK_TABLE(table), bbox, 1, 2, 3, 4,
 					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
 					(GtkAttachOptions) (0), 0, 0);
 
-	label = gtk_label_new(_("Run command:"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 4, 5,
+	e->make_in_base_path = gtk_check_button_new_with_label(_("Make in base path"));
+	gtk_table_attach(GTK_TABLE(table), e->make_in_base_path, 0, 3, 4, 5,
 					(GtkAttachOptions) (GTK_FILL),
 					(GtkAttachOptions) (0), 0, 0);
-	gtk_misc_set_alignment(GTK_MISC(label), 1, 0);
+
+	label = gtk_label_new(_("Run command:"));
+	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 5, 6,
+					(GtkAttachOptions) (GTK_FILL),
+					(GtkAttachOptions) (0), 0, 0);
+	gtk_misc_set_alignment(GTK_MISC(label), -1, 0);
 
 	e->run_cmd = gtk_entry_new();
 	gtk_tooltips_set_tip(tooltips, e->run_cmd,
@@ -401,17 +461,17 @@ void project_properties()
 	bbox = gtk_hbox_new(FALSE, 6);
 	gtk_box_pack_start_defaults(GTK_BOX(bbox), e->run_cmd);
 	gtk_box_pack_start(GTK_BOX(bbox), button, FALSE, FALSE, 0);
-	gtk_table_attach(GTK_TABLE(table), bbox, 1, 2, 4, 5,
+	gtk_table_attach(GTK_TABLE(table), bbox, 1, 2, 5, 6,
 					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
 					(GtkAttachOptions) (0), 0, 0);
 
 #if 0
 	label = gtk_label_new(_("File patterns:"));
 	// <small>Separate multiple patterns by a new line</small>
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 5, 6,
+	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 6, 7,
 					(GtkAttachOptions) (GTK_FILL),
 					(GtkAttachOptions) (GTK_FILL), 0, 0);
-	gtk_misc_set_alignment(GTK_MISC(label), 1, 0);
+	gtk_misc_set_alignment(GTK_MISC(label), -1, 0);
 
 	e->patterns = gtk_text_view_new();
 	swin = gtk_scrolled_window_new(NULL, NULL);
@@ -419,12 +479,26 @@ void project_properties()
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swin),
 				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(swin), GTK_WIDGET(e->patterns));
-	gtk_table_attach(GTK_TABLE(table), swin, 1, 2, 5, 6,
+	gtk_table_attach(GTK_TABLE(table), swin, 1, 2, 6, 7,
 					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
 					(GtkAttachOptions) (0), 0, 0);
 #endif
 
 	gtk_container_add(GTK_CONTAINER(vbox), table);
+}
+
+
+void project_properties()
+{
+	PropertyDialogElements *e = g_new(PropertyDialogElements, 1);
+	GeanyProject *p = app->project;
+	gint response;
+
+	g_return_if_fail(app->project != NULL);
+
+	entries_modified = FALSE;
+
+	create_properties_dialog(e);
 
 	// fill the elements with the appropriate data
 	gtk_entry_set_text(GTK_ENTRY(e->name), p->name);
@@ -458,6 +532,8 @@ void project_properties()
 
 	gtk_entry_set_text(GTK_ENTRY(e->file_name), p->file_name);
 	gtk_entry_set_text(GTK_ENTRY(e->base_path), p->base_path);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(e->make_in_base_path),
+		p->make_in_base_path);
 	if (p->run_cmd != NULL)
 		gtk_entry_set_text(GTK_ENTRY(e->run_cmd), p->run_cmd);
 
@@ -485,11 +561,11 @@ static gboolean close_open_project()
 {
 	if (app->project != NULL)
 	{
-		if (dialogs_show_question_full(NULL, GTK_STOCK_OK, GTK_STOCK_CANCEL,
+		if (dialogs_show_question_full(NULL, GTK_STOCK_CLOSE, GTK_STOCK_CANCEL,
 			_("Do you want to close it before proceeding?"),
-			_("The '%s' project is already open. "), app->project->name))
+			_("The '%s' project is already open."), app->project->name))
 		{
-			project_close();
+			project_close(FALSE);
 			return TRUE;
 		}
 		else
@@ -505,7 +581,9 @@ static gboolean close_open_project()
 static gboolean update_config(const PropertyDialogElements *e)
 {
 	const gchar *name, *file_name, *base_path;
+	gchar *locale_filename;
 	gint name_len;
+	gint err_code;
 	gboolean new_project = FALSE;
 	GeanyProject *p;
 
@@ -521,7 +599,7 @@ static gboolean update_config(const PropertyDialogElements *e)
 	}
 	else if (name_len > MAX_NAME_LEN)
 	{
-		SHOW_ERR(_("The specified project name is too long (max. %d characters)."), MAX_NAME_LEN);
+		SHOW_ERR1(_("The specified project name is too long (max. %d characters)."), MAX_NAME_LEN);
 		gtk_widget_grab_focus(e->name);
 		return FALSE;
 	}
@@ -534,10 +612,27 @@ static gboolean update_config(const PropertyDialogElements *e)
 		return FALSE;
 	}
 
+	// finally test whether the given project file can be written
+	locale_filename = utils_get_locale_from_utf8(file_name);
+	if ((err_code = utils_is_file_writeable(locale_filename)) != 0)
+	{
+		SHOW_ERR1(_("Project file could not be written (%s)."), g_strerror(err_code));
+		gtk_widget_grab_focus(e->file_name);
+		return FALSE;
+	}
+
 	base_path = gtk_entry_get_text(GTK_ENTRY(e->base_path));
 	if (NZV(base_path))
 	{	// check whether the given directory actually exists
 		gchar *locale_path = utils_get_locale_from_utf8(base_path);
+
+		if (! g_path_is_absolute(locale_path))
+		{	// relative base path, so add base dir of project file name
+			gchar *dir = g_path_get_dirname(locale_filename);
+			setptr(locale_path, g_strconcat(dir, G_DIR_SEPARATOR_S, locale_path, NULL));
+			g_free(dir);
+		}
+
 		if (! g_file_test(locale_path, G_FILE_TEST_IS_DIR))
 		{
 			if (dialogs_show_question_full(NULL, GTK_STOCK_OK, GTK_STOCK_CANCEL,
@@ -556,14 +651,7 @@ static gboolean update_config(const PropertyDialogElements *e)
 		}
 		g_free(locale_path);
 	}
-
-	// finally test whether the given project file can be written
-	if (utils_write_file(file_name, "") != 0)
-	{
-		SHOW_ERR(_("Project file could not be written."));
-		gtk_widget_grab_focus(e->file_name);
-		return FALSE;
-	}
+	g_free(locale_filename);
 
 	if (app->project == NULL)
 	{
@@ -579,7 +667,7 @@ static gboolean update_config(const PropertyDialogElements *e)
 	p->file_name = g_strdup(file_name);
 
 	if (p->base_path != NULL) g_free(p->base_path);
-	p->base_path = g_strdup(base_path);
+	p->base_path = g_strdup(NZV(base_path) ? base_path : "./"); // use "." if base_path is empty
 
 	if (! new_project)	// save properties specific fields
 	{
@@ -587,6 +675,8 @@ static gboolean update_config(const PropertyDialogElements *e)
 		//gchar *tmp;
 		GtkTextBuffer *buffer;
 
+		p->make_in_base_path = gtk_toggle_button_get_active(
+			GTK_TOGGLE_BUTTON(e->make_in_base_path));
 		if (p->run_cmd != NULL) g_free(p->run_cmd);
 		p->run_cmd = g_strdup(gtk_entry_get_text(GTK_ENTRY(e->run_cmd)));
 
@@ -608,11 +698,11 @@ static gboolean update_config(const PropertyDialogElements *e)
 		g_free(tmp);
 #endif
 	}
-	write_config();
+	write_config(TRUE);
 	if (new_project)
-		msgwin_status_add(_("Project \"%s\" created."), p->name);
+		ui_set_statusbar(TRUE, _("Project \"%s\" created."), p->name);
 	else
-		msgwin_status_add(_("Project \"%s\" saved."), p->name);
+		ui_set_statusbar(TRUE, _("Project \"%s\" saved."), p->name);
 
 	return TRUE;
 }
@@ -629,6 +719,18 @@ static void run_dialog(GtkWidget *dialog, GtkWidget *entry)
 	{
 		if (g_file_test(locale_filename, G_FILE_TEST_EXISTS))
 			gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog), utf8_filename);
+		else // if the file doesn't yet exist, use at least the current directory
+		{
+			gchar *locale_dir = g_path_get_dirname(locale_filename);
+			gchar *name = g_path_get_basename(utf8_filename);
+
+			if (g_file_test(locale_dir, G_FILE_TEST_EXISTS))
+				gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), locale_dir);
+			gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(dialog), name);
+
+			g_free(name);
+			g_free(locale_dir);
+		}
 	}
 	else
 	if (gtk_file_chooser_get_action(GTK_FILE_CHOOSER(dialog)) != GTK_FILE_CHOOSER_ACTION_OPEN)
@@ -653,14 +755,14 @@ static void run_dialog(GtkWidget *dialog, GtkWidget *entry)
 #endif
 
 
-static void on_file_save_button_clicked(GtkButton *button, GtkWidget *entry)
+static void on_file_save_button_clicked(GtkButton *button, PropertyDialogElements *e)
 {
 #ifdef G_OS_WIN32
-	gchar *path = win32_show_project_open_dialog(_("Choose Project Filename"),
-						gtk_entry_get_text(GTK_ENTRY(entry)), TRUE);
+	gchar *path = win32_show_project_open_dialog(e->dialog, _("Choose Project Filename"),
+						gtk_entry_get_text(GTK_ENTRY(e->file_name)), TRUE, TRUE);
 	if (path != NULL)
 	{
-		gtk_entry_set_text(GTK_ENTRY(entry), path);
+		gtk_entry_set_text(GTK_ENTRY(e->file_name), path);
 		g_free(path);
 	}
 #else
@@ -677,19 +779,19 @@ static void on_file_save_button_clicked(GtkButton *button, GtkWidget *entry)
 	gtk_window_set_type_hint(GTK_WINDOW(dialog), GDK_WINDOW_TYPE_HINT_DIALOG);
 	gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT);
 
-	run_dialog(dialog, entry);
+	run_dialog(dialog, e->file_name);
 #endif
 }
 
 
-static void on_file_open_button_clicked(GtkButton *button, GtkWidget *entry)
+static void on_file_open_button_clicked(GtkButton *button, PropertyDialogElements *e)
 {
 #ifdef G_OS_WIN32
-	gchar *path = win32_show_project_open_dialog(_("Choose Project Run Command"),
-						gtk_entry_get_text(GTK_ENTRY(entry)), FALSE);
+	gchar *path = win32_show_project_open_dialog(e->dialog, _("Choose Project Run Command"),
+						gtk_entry_get_text(GTK_ENTRY(e->run_cmd)), FALSE, FALSE);
 	if (path != NULL)
 	{
-		gtk_entry_set_text(GTK_ENTRY(entry), path);
+		gtk_entry_set_text(GTK_ENTRY(e->run_cmd), path);
 		g_free(path);
 	}
 #else
@@ -706,7 +808,7 @@ static void on_file_open_button_clicked(GtkButton *button, GtkWidget *entry)
 	gtk_window_set_type_hint(GTK_WINDOW(dialog), GDK_WINDOW_TYPE_HINT_DIALOG);
 	gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT);
 
-	run_dialog(dialog, entry);
+	run_dialog(dialog, e->run_cmd);
 #endif
 }
 
@@ -759,14 +861,14 @@ gboolean project_load_file(const gchar *locale_file_name)
 
 	if (load_config(locale_file_name))
 	{
-		msgwin_status_add(_("Project \"%s\" opened."), app->project->name);
+		ui_set_statusbar(TRUE, _("Project \"%s\" opened."), app->project->name);
 		return TRUE;
 	}
 	else
 	{
 		gchar *utf8_filename = utils_get_utf8_from_locale(locale_file_name);
 
-		msgwin_status_add(_("Project file \"%s\" could not be loaded."), utf8_filename);
+		ui_set_statusbar(TRUE, _("Project file \"%s\" could not be loaded."), utf8_filename);
 		g_free(utf8_filename);
 	}
 	return FALSE;
@@ -786,7 +888,7 @@ static gboolean load_config(const gchar *filename)
 	g_return_val_if_fail(app->project == NULL && filename != NULL, FALSE);
 
 	config = g_key_file_new();
-	if (! g_key_file_load_from_file(config, filename, G_KEY_FILE_KEEP_COMMENTS, NULL))
+	if (! g_key_file_load_from_file(config, filename, G_KEY_FILE_NONE, NULL))
 	{
 		g_key_file_free(config);
 		return FALSE;
@@ -798,9 +900,26 @@ static gboolean load_config(const gchar *filename)
 	p->description = utils_get_setting_string(config, "project", "description", "");
 	p->file_name = utils_get_utf8_from_locale(filename);
 	p->base_path = utils_get_setting_string(config, "project", "base_path", "");
+	p->make_in_base_path = utils_get_setting_boolean(config, "project", "make_in_base_path", TRUE);
 	p->run_cmd = utils_get_setting_string(config, "project", "run_cmd", "");
 	p->file_patterns = g_key_file_get_string_list(config, "project", "file_patterns", NULL, NULL);
 
+	if (project_prefs.project_session)
+	{
+		// save current (non-project) session (it could has been changed since program startup)
+		configuration_save_default_session();
+		// now close all open files
+		/// TODO make this a general, non-callback function, use it also in project_close()
+		///      and remove include of callbacks.h
+		on_close_all1_activate(NULL, NULL);
+		// read session files so they can be opened with configuration_open_files()
+		configuration_load_session_files(config);
+	}
+
+	if (geany_object)
+	{
+		g_signal_emit_by_name(geany_object, "project-open", config);
+	}
 	g_key_file_free(config);
 
 	update_ui();
@@ -808,8 +927,12 @@ static gboolean load_config(const gchar *filename)
 }
 
 
-// Returns: TRUE if project file was written successfully.
-static gboolean write_config()
+/* Write the project settings as well as the project session files into its configuration files.
+ * emit_signal defines whether the project-save signal should be emitted. When write_config()
+ * is called while closing a project, this is used to skip emitting the signal because
+ * project-close will be emitted afterwards.
+ * Returns: TRUE if project file was written successfully. */
+static gboolean write_config(gboolean emit_signal)
 {
 	GeanyProject *p;
 	GKeyFile *config;
@@ -824,19 +947,28 @@ static gboolean write_config()
 	config = g_key_file_new();
 	// try to load an existing config to keep manually added comments
 	filename = utils_get_locale_from_utf8(p->file_name);
-	g_key_file_load_from_file(config, filename, G_KEY_FILE_KEEP_COMMENTS, NULL);
+	g_key_file_load_from_file(config, filename, G_KEY_FILE_NONE, NULL);
 
 	g_key_file_set_string(config, "project", "name", p->name);
 	g_key_file_set_string(config, "project", "base_path", p->base_path);
 
 	if (p->description)
 		g_key_file_set_string(config, "project", "description", p->description);
+	g_key_file_set_boolean(config, "project", "make_in_base_path", p->make_in_base_path);
 	if (p->run_cmd)
 		g_key_file_set_string(config, "project", "run_cmd", p->run_cmd);
 	if (p->file_patterns)
 		g_key_file_set_string_list(config, "project", "file_patterns",
 			(const gchar**) p->file_patterns, g_strv_length(p->file_patterns));
 
+	// store the session files into the project too
+	if (project_prefs.project_session)
+		configuration_save_session_files(config);
+
+	if (geany_object && emit_signal)
+	{
+		g_signal_emit_by_name(geany_object, "project-save", config);
+	}
 	// write the file
 	data = g_key_file_to_data(config, NULL, NULL);
 	ret = (utils_write_file(filename, data) == 0);
@@ -849,15 +981,47 @@ static gboolean write_config()
 }
 
 
-const gchar *project_get_make_dir()
+/* Constructs the project's base path which is used for "Make all" and "Execute".
+ * The result is an absolute string in UTF-8 encoding which is either the same as
+ * base path if it is absolute or it is built out of project file name's dir and base_path.
+ * If there is no project or project's base_path is invalid, NULL will be returned.
+ * The returned string should be freed when no longer needed. */
+gchar *project_get_base_path()
 {
-	if (app->project != NULL && NZV(app->project->base_path))
-		return app->project->base_path;
-	else
-		return NULL;
+	GeanyProject *project = app->project;
+
+	if (project && NZV(project->base_path))
+	{
+		if (g_path_is_absolute(project->base_path))
+			return g_strdup(project->base_path);
+		else
+		{	// build base_path out of project file name's dir and base_path
+			gchar *path;
+			gchar *dir = g_path_get_dirname(project->file_name);
+
+			path = g_strconcat(dir, G_DIR_SEPARATOR_S, project->base_path, NULL);
+			g_free(dir);
+			return path;
+		}
+	}
+	return NULL;
 }
 
 
+/* Returns: NULL if the default path should be used, or a UTF-8 path.
+ * Maybe in future this will support a separate project make path from base path. */
+gchar *project_get_make_dir()
+{
+	GeanyProject *project = app->project;
+
+	if (project && ! project->make_in_base_path)
+		return NULL;
+	else
+		return project_get_base_path();
+}
+
+
+/* This is to save project-related global settings, NOT project file settings. */
 void project_save_prefs(GKeyFile *config)
 {
 	GeanyProject *project = app->project;
@@ -885,7 +1049,7 @@ void project_load_prefs(GKeyFile *config)
 		"project_file_path", NULL);
 	if (local_prefs.project_file_path == NULL)
 	{
-		local_prefs.project_file_path = g_strconcat(GEANY_HOME_DIR,
+		local_prefs.project_file_path = g_strconcat(g_get_home_dir(),
 			G_DIR_SEPARATOR_S, PROJECT_DIR, NULL);
 	}
 }
@@ -911,8 +1075,7 @@ void project_apply_prefs()
 	const gchar *str;
 
 	str = gtk_entry_get_text(GTK_ENTRY(path_entry));
-	g_free(local_prefs.project_file_path);
-	local_prefs.project_file_path = g_strdup(str);
+	setptr(local_prefs.project_file_path, g_strdup(str));
 }
 
 
