@@ -1,9 +1,8 @@
 /*
  *      sciwrappers.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
- *
+ *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
  *      the Free Software Foundation; either version 2 of the License, or
@@ -18,7 +17,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: sciwrappers.c 1904 2007-09-25 16:44:33Z ntrel $
+ * $Id: sciwrappers.c 2150 2008-01-06 18:11:57Z eht16 $
  */
 
 /*
@@ -167,7 +166,7 @@ void sci_set_lines_wrapped(ScintillaObject* sci, gboolean set )
 	if (set)
 	{
 		SSM(sci,SCI_SETWRAPMODE,SC_WRAP_WORD,0);
-		SSM(sci, SCI_SETWRAPVISUALFLAGS, SC_WRAPVISUALFLAG_END, 0);
+		SSM(sci, SCI_SETWRAPVISUALFLAGS, SC_WRAPVISUALFLAG_END | SC_WRAPVISUALFLAG_START, 0);
 	}
 	else
 		SSM(sci,SCI_SETWRAPMODE,SC_WRAP_NONE,0);
