@@ -1,8 +1,8 @@
 /*
  *      search.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2006-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: search.c 1887 2007-09-17 16:33:17Z ntrel $
+ * $Id: search.c 2199 2008-01-29 16:34:16Z ntrel $
  */
 
 /*
@@ -208,28 +208,6 @@ static void send_find_dialog_response(GtkButton *button, gpointer user_data)
 }
 
 
-static gchar *get_default_text(gint idx)
-{
-	gchar *s = NULL;
-
-	if (sci_get_lines_selected(doc_list[idx].sci) == 1)
-	{
-		gint len = sci_get_selected_text_length(doc_list[idx].sci);
-
-		s = g_malloc(len + 1);
-		sci_get_selected_text(doc_list[idx].sci, s);
-	}
-	else if (sci_get_lines_selected(doc_list[idx].sci) == 0)
-	{	// use the word at current cursor position
-		gchar word[GEANY_MAX_WORD_LENGTH];
-
-		editor_find_current_word(doc_list[idx].sci, -1, word, sizeof(word), NULL);
-		if (word[0] != '\0') s = g_strdup(word);
-	}
-	return s;
-}
-
-
 // store text, clear search flags so we can use Search->Find Next/Previous
 static void setup_find_next(const gchar *text)
 {
@@ -269,7 +247,7 @@ void search_find_selection(gint idx, gboolean search_backwards)
 	}
 #endif
 
-	if (!s)	{ s=get_default_text(idx); }
+	if (!s)	{ s=editor_get_default_selection(idx, NULL); }
 	if (s)
 	{
 		setup_find_next(s);	// allow find next/prev
@@ -286,7 +264,7 @@ void search_show_find_dialog()
 
 	g_return_if_fail(DOC_IDX_VALID(idx));
 
-	sel = get_default_text(idx);
+	sel = editor_get_default_selection(idx, NULL);
 
 	if (widgets.find_dialog == NULL)
 	{
@@ -338,7 +316,7 @@ void search_show_find_dialog()
 			add_find_checkboxes(GTK_DIALOG(widgets.find_dialog)));
 
 		// Now add the multiple match options
-		exp = gtk_expander_new(_("Find All"));
+		exp = gtk_expander_new_with_mnemonic(_("_Find All"));
 		bbox = gtk_hbutton_box_new();
 
 		button = gtk_button_new_with_mnemonic(_("_Mark"));
@@ -405,7 +383,7 @@ void search_show_replace_dialog()
 
 	if (idx == -1 || ! doc_list[idx].is_valid) return;
 
-	sel = get_default_text(idx);
+	sel = editor_get_default_selection(idx, NULL);
 
 	if (widgets.replace_dialog == NULL)
 	{
@@ -434,7 +412,7 @@ void search_show_replace_dialog()
 		label_find = gtk_label_new_with_mnemonic(_("_Search for:"));
 		gtk_misc_set_alignment(GTK_MISC(label_find), 0, 0.5);
 
-		label_replace = gtk_label_new_with_mnemonic(_("Re_place with:"));
+		label_replace = gtk_label_new_with_mnemonic(_("Replace wit_h:"));
 		gtk_misc_set_alignment(GTK_MISC(label_replace), 0, 0.5);
 
 		entry_find = gtk_combo_box_entry_new_text();
@@ -481,7 +459,7 @@ void search_show_replace_dialog()
 			add_find_checkboxes(GTK_DIALOG(widgets.replace_dialog)));
 
 		// Now add the multiple replace options
-		exp = gtk_expander_new(_("Replace All"));
+		exp = gtk_expander_new_with_mnemonic(_("Re_place All"));
 		bbox = gtk_hbutton_box_new();
 
 		button = gtk_button_new_with_mnemonic(_("In Se_lection"));
@@ -543,7 +521,9 @@ static void on_extra_options_toggled(GtkToggleButton *togglebutton, gpointer use
 }
 
 
-void search_show_find_in_files_dialog()
+/* dir is the directory to search in (UTF-8 encoding), maybe NULL to determine it the usual way
+ * by using the current file's path */
+void search_show_find_in_files_dialog(const gchar *dir)
 {
 	static GtkWidget *combo = NULL;
 	static GtkWidget *dir_combo;
@@ -552,7 +532,7 @@ void search_show_find_in_files_dialog()
 	gchar *sel = NULL;
 	gchar *cur_dir;
 
-	if (idx == -1 || ! doc_list[idx].is_valid) return;
+	if (! DOC_IDX_VALID(idx)) return;
 
 	if (widgets.find_in_files_dialog == NULL)
 	{
@@ -608,7 +588,7 @@ void search_show_find_in_files_dialog()
 		g_object_unref(G_OBJECT(size_group));	// auto destroy the size group
 
 		rbox = gtk_vbox_new(FALSE, 0);
-		rbtn = gtk_radio_button_new_with_mnemonic(NULL, _("_Fixed strings"));
+		rbtn = gtk_radio_button_new_with_mnemonic(NULL, _("Fixed s_trings"));
 		// Make fixed strings the default to speed up searching all files in directory.
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rbtn), TRUE);
 		g_object_set_data_full(G_OBJECT(widgets.find_in_files_dialog), "radio_fgrep",
@@ -700,19 +680,22 @@ void search_show_find_in_files_dialog()
 				G_CALLBACK(gtk_widget_hide_on_delete), NULL);
 
 		gtk_widget_show_all(widgets.find_in_files_dialog);
-		sel = get_default_text(idx);
+		sel = editor_get_default_selection(idx, NULL);
 	}
 
 	entry = GTK_BIN(combo)->child;
 	// only set selection if the dialog is not already visible, or has just been created
 	if (! sel && ! GTK_WIDGET_VISIBLE(widgets.find_in_files_dialog))
-		sel = get_default_text(idx);
+		sel = editor_get_default_selection(idx, NULL);
 	if (sel)
 		gtk_entry_set_text(GTK_ENTRY(entry), sel);
 	g_free(sel);
 
 	entry = GTK_BIN(dir_combo)->child;
-	cur_dir = utils_get_current_file_dir();
+	if (NZV(dir))
+		cur_dir = g_strdup(dir);
+	else
+		cur_dir = utils_get_current_file_dir_utf8();
 	if (cur_dir)
 	{
 		gtk_entry_set_text(GTK_ENTRY(entry), cur_dir);
@@ -805,6 +788,26 @@ on_find_entry_activate(GtkEntry *entry, gpointer user_data)
 }
 
 
+static gint get_search_flags(GtkWidget *dialog)
+{
+	gboolean fl1, fl2, fl3, fl4;
+
+	fl1 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
+				lookup_widget(dialog, "check_case")));
+	fl2 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
+				lookup_widget(dialog, "check_word")));
+	fl3 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
+				lookup_widget(dialog, "check_regexp")));
+	fl4 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
+				lookup_widget(dialog, "check_wordstart")));
+
+	return (fl1 ? SCFIND_MATCHCASE : 0) |
+		(fl2 ? SCFIND_WHOLEWORD : 0) |
+		(fl3 ? SCFIND_REGEXP | SCFIND_POSIX : 0) |
+		(fl4 ? SCFIND_WORDSTART : 0);
+}
+
+
 static void
 on_find_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 {
@@ -814,17 +817,9 @@ on_find_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 	{
 		gint idx = document_get_cur_idx();
 		gboolean search_replace_escape;
-		gboolean
-			fl1 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-						lookup_widget(GTK_WIDGET(widgets.find_dialog), "check_case"))),
-			fl2 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-						lookup_widget(GTK_WIDGET(widgets.find_dialog), "check_word"))),
-			fl3 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-						lookup_widget(GTK_WIDGET(widgets.find_dialog), "check_regexp"))),
-			fl4 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-						lookup_widget(GTK_WIDGET(widgets.find_dialog), "check_wordstart"))),
-			check_close = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
+		gboolean check_close = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
 						lookup_widget(GTK_WIDGET(widgets.find_dialog), "check_close")));
+
 		search_replace_escape = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
 						lookup_widget(GTK_WIDGET(widgets.find_dialog), "check_escape")));
 		search_data.backwards = FALSE;
@@ -843,10 +838,8 @@ on_find_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 		}
 
 		ui_combo_box_add_to_history(GTK_COMBO_BOX(user_data), search_data.text);
-		search_data.flags = (fl1 ? SCFIND_MATCHCASE : 0) |
-				(fl2 ? SCFIND_WHOLEWORD : 0) |
-				(fl3 ? SCFIND_REGEXP | SCFIND_POSIX: 0) |
-				(fl4 ? SCFIND_WORDSTART : 0);
+
+		search_data.flags = get_search_flags(widgets.find_dialog);
 
 		switch (response)
 		{
@@ -873,9 +866,9 @@ on_find_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 					gint count = search_mark(idx, search_data.text, search_data.flags);
 
 					if (count == 0)
-						ui_set_statusbar(_("No matches found for \"%s\"."), search_data.text);
+						ui_set_statusbar(FALSE, _("No matches found for \"%s\"."), search_data.text);
 					else
-						ui_set_statusbar(_("Found %d matches for \"%s\"."), count,
+						ui_set_statusbar(FALSE, _("Found %d matches for \"%s\"."), count,
 							search_data.text);
 				}
 				break;
@@ -901,7 +894,6 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 	GtkWidget *entry_replace = lookup_widget(GTK_WIDGET(widgets.replace_dialog), "entry_replace");
 	gint search_flags_re;
 	gboolean search_backwards_re, search_replace_escape_re;
-	gboolean fl1, fl2, fl3, fl4;
 	gboolean close_window;
 	gchar *find, *replace;
 
@@ -911,14 +903,6 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 		return;
 	}
 
-	fl1 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-				lookup_widget(GTK_WIDGET(widgets.replace_dialog), "check_case")));
-	fl2 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-				lookup_widget(GTK_WIDGET(widgets.replace_dialog), "check_word")));
-	fl3 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-				lookup_widget(GTK_WIDGET(widgets.replace_dialog), "check_regexp")));
-	fl4 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-				lookup_widget(GTK_WIDGET(widgets.replace_dialog), "check_wordstart")));
 	close_window = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
 				lookup_widget(GTK_WIDGET(widgets.replace_dialog), "check_close")));
 	search_backwards_re = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
@@ -928,7 +912,10 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 	find = g_strdup(gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry_find)))));
 	replace = g_strdup(gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry_replace)))));
 
-	if ((response != GEANY_RESPONSE_FIND) && fl1 && (strcasecmp(find, replace) == 0))
+	search_flags_re = get_search_flags(widgets.replace_dialog);
+
+	if ((response != GEANY_RESPONSE_FIND) && (search_flags_re & SCFIND_MATCHCASE)
+		&& (strcmp(find, replace) == 0))
 	{
 		utils_beep();
 		gtk_widget_grab_focus(GTK_WIDGET(GTK_BIN(lookup_widget(widgets.replace_dialog, "entry_find"))->child));
@@ -946,11 +933,6 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 		return;
 	}
 
-	search_flags_re = (fl1 ? SCFIND_MATCHCASE : 0) |
-					  (fl2 ? SCFIND_WHOLEWORD : 0) |
-					  (fl3 ? SCFIND_REGEXP | SCFIND_POSIX : 0) |
-					  (fl4 ? SCFIND_WORDSTART : 0);
-
 	switch (response)
 	{
 		case GEANY_RESPONSE_REPLACE_AND_FIND:
@@ -994,7 +976,7 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 				if (document_replace_all(ix, find, replace, search_flags_re,
 					search_replace_escape_re)) count++;
 			}
-			ui_set_statusbar(_("Replaced text in %u files."), count);
+			ui_set_statusbar(FALSE, _("Replaced text in %u files."), count);
 			// show which docs had replacements:
 			gtk_notebook_set_current_page(GTK_NOTEBOOK(msgwindow.notebook), MSG_STATUS);
 
@@ -1095,7 +1077,7 @@ on_find_in_files_dialog_response(GtkDialog *dialog, gint response, gpointer user
 					lookup_widget(widgets.find_in_files_dialog, "entry_extra"))));
 
 		if (utf8_dir == NULL || utils_str_equal(utf8_dir, ""))
-			ui_set_statusbar(_("Invalid directory for find in files."));
+			ui_set_statusbar(FALSE, _("Invalid directory for find in files."));
 		else if (search_text && *search_text)
 		{
 			gchar *locale_dir;
@@ -1113,7 +1095,7 @@ on_find_in_files_dialog_response(GtkDialog *dialog, gint response, gpointer user
 			g_string_free(opts, TRUE);
 		}
 		else
-			ui_set_statusbar(_("No text to find."));
+			ui_set_statusbar(FALSE, _("No text to find."));
 	}
 	else
 		gtk_widget_hide(widgets.find_in_files_dialog);
@@ -1126,7 +1108,7 @@ search_find_in_files(const gchar *search_text, const gchar *dir, const gchar *op
 	gchar **argv_prefix, **argv, **opts_argv;
 	guint opts_argv_len, i;
 	GPid child_pid;
-	gint stdout_fd, stdin_fd;
+	gint stdout_fd;
 	GError *error = NULL;
 	gboolean ret = FALSE;
 
@@ -1134,7 +1116,7 @@ search_find_in_files(const gchar *search_text, const gchar *dir, const gchar *op
 
 	if (! g_file_test(prefs.tools_grep_cmd, G_FILE_TEST_IS_EXECUTABLE))
 	{
-		msgwin_status_add(_("Cannot execute grep tool '%s';"
+		ui_set_statusbar(TRUE, _("Cannot execute grep tool '%s';"
 			" check the path setting in Preferences."), prefs.tools_grep_cmd);
 		return FALSE;
 	}
@@ -1182,10 +1164,10 @@ search_find_in_files(const gchar *search_text, const gchar *dir, const gchar *op
 	if (! g_spawn_async_with_pipes(dir, (gchar**)argv, NULL,
 		G_SPAWN_STDERR_TO_DEV_NULL | G_SPAWN_DO_NOT_REAP_CHILD,
 		NULL, NULL, &child_pid,
-		&stdin_fd, &stdout_fd, NULL, &error))
+		NULL, &stdout_fd, NULL, &error))
 	{
 		geany_debug("%s: g_spawn_async_with_pipes() failed: %s", __func__, error->message);
-		msgwin_status_add(_("Process failed (%s)"), error->message);
+		ui_set_statusbar(TRUE, _("Process failed (%s)"), error->message);
 		g_error_free(error);
 		ret = FALSE;
 	}
@@ -1227,7 +1209,7 @@ static gchar **search_get_argv(const gchar **argv_prefix, const gchar *dir)
 	list = utils_get_file_list(dir, &list_len, &error);
 	if (error)
 	{
-		msgwin_status_add(_("Could not open directory (%s)"), error->message);
+		ui_set_statusbar(TRUE, _("Could not open directory (%s)"), error->message);
 		g_error_free(error);
 		return NULL;
 	}
@@ -1287,8 +1269,9 @@ static void search_close_pid(GPid child_pid, gint status, gpointer user_data)
 				gint count = gtk_tree_model_iter_n_children(
 					GTK_TREE_MODEL(msgwindow.store_msg), NULL) - 1;
 
-				msgwin_msg_add_fmt(COLOR_BLUE, -1, -1, _("Search completed with %d matches."), count);
-				ui_set_statusbar(_("Search completed with %d matches."), count);
+				msgwin_msg_add_fmt(COLOR_BLUE, -1, -1,
+					_("Search completed with %d matches."), count);
+				ui_set_statusbar(FALSE, _("Search completed with %d matches."), count);
 				break;
 			}
 			case 1:
@@ -1296,7 +1279,7 @@ static void search_close_pid(GPid child_pid, gint status, gpointer user_data)
 				color = COLOR_BLUE;
 			default:
 				msgwin_msg_add(color, -1, -1, msg);
-				ui_set_statusbar("%s", msg);
+				ui_set_statusbar(FALSE, "%s", msg);
 				break;
 		}
 	}
@@ -1373,14 +1356,14 @@ void search_find_usage(const gchar *search_text, gint flags, gboolean in_session
 
 	if (! found) // no matches were found
 	{
-		ui_set_statusbar(_("No matches found for \"%s\"."), search_text);
+		ui_set_statusbar(FALSE, _("No matches found for \"%s\"."), search_text);
 		msgwin_msg_add_fmt(COLOR_BLUE, -1, -1, _("No matches found for \"%s\"."), search_text);
 	}
 	else
 	{
 		gint count = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(msgwindow.store_msg), NULL);
 
-		ui_set_statusbar(_("Found %d matches for \"%s\"."), count, search_text);
+		ui_set_statusbar(FALSE, _("Found %d matches for \"%s\"."), count, search_text);
 		msgwin_msg_add_fmt(COLOR_BLUE, -1, -1, _("Found %d matches for \"%s\"."), count,
 			search_text);
 	}
