@@ -1,8 +1,8 @@
 /*
  *      symbols.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2006-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: symbols.c 1934 2007-10-09 12:04:15Z eht16 $
+ * $Id: symbols.c 2150 2008-01-06 18:11:57Z eht16 $
  */
 
 /*
@@ -34,6 +34,7 @@
 #include <string.h>
 #include <stdlib.h>
 
+#include "prefix.h"
 #include "symbols.h"
 #include "utils.h"
 #include "filetypes.h"
@@ -92,7 +93,8 @@ void symbols_global_tags_loaded(gint file_type_idx)
 	TagFileInfo *tfi;
 	gint tag_type;
 
-	if (cl_options.ignore_global_tags) return;
+	if (cl_options.ignore_global_tags || app->tm_workspace == NULL)
+		return;
 
 	load_user_tags(file_type_idx);
 
@@ -198,6 +200,7 @@ const gchar *symbols_get_context_separator(gint ft_id)
 	{
 		case GEANY_FILETYPES_C:	// for C++ .h headers or C structs
 		case GEANY_FILETYPES_CPP:
+		//~ case GEANY_FILETYPES_RUBY: // not sure what to use atm
 		{
 			static gchar cc[] = "::";
 
@@ -273,7 +276,7 @@ static TMTag *find_workspace_tag(const gchar *tag_name, gint type)
 	const GPtrArray *tags;
 	TMTag *tmtag;
 
-	if (app->tm_workspace->work_objects != NULL)
+	if (app->tm_workspace != NULL && app->tm_workspace->work_objects != NULL)
 	{
 		for (j = 0; j < app->tm_workspace->work_objects->len; j++)
 		{
@@ -461,12 +464,16 @@ tag_list_add_groups(GtkTreeStore *tree_store, ...)
 
 	if (icon_theme == NULL)
 	{
+#ifndef G_OS_WIN32
+		gchar *path = g_strconcat(GEANY_DATADIR, "/icons", NULL);
+#endif
 		gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &x, &y);
 		icon_theme = gtk_icon_theme_get_default();
 #ifdef G_OS_WIN32
 		gtk_icon_theme_append_search_path(icon_theme, "share\\icons");
 #else
-		gtk_icon_theme_append_search_path(icon_theme, PACKAGE_DATA_DIR "/icons");
+		gtk_icon_theme_append_search_path(icon_theme, path);
+		g_free(path);
 #endif
 	}
 
@@ -488,7 +495,7 @@ tag_list_add_groups(GtkTreeStore *tree_store, ...)
 		if (G_IS_OBJECT(icon))
 		{
 			gtk_tree_store_set(tree_store, iter, SYMBOLS_COLUMN_ICON, icon,
-			                   SYMBOLS_COLUMN_NAME, title, -1);
+				SYMBOLS_COLUMN_NAME, title, -1);
 			g_object_unref(icon);
 		}
 		else
@@ -572,9 +579,10 @@ static void init_tag_list(gint idx)
 		case GEANY_FILETYPES_PHP:
 		{
 			tag_list_add_groups(tag_store,
+				&(tv_iters.tag_struct), _("Interfaces"), "classviewer-struct",
 				&(tv_iters.tag_class), _("Classes"), "classviewer-class",
 				&(tv_iters.tag_function), _("Functions"), "classviewer-method",
-				&(tv_iters.tag_macro), _("Constants"), NULL,
+				&(tv_iters.tag_macro), _("Constants"), "classviewer-macro",
 				&(tv_iters.tag_variable), _("Variables"), "classviewer-var",
 				NULL);
 				//&(tv_iters.tag_struct), _("Label"),
@@ -595,12 +603,10 @@ static void init_tag_list(gint idx)
 		case GEANY_FILETYPES_RUBY:
 		{
 			tag_list_add_groups(tag_store,
+				&(tv_iters.tag_namespace), _("Modules"), NULL,
 				&(tv_iters.tag_class), _("Classes"), "classviewer-class",
-				&(tv_iters.tag_member), _("Singletons"), NULL,
-				&(tv_iters.tag_macro), _("Mixins"), NULL,
+				&(tv_iters.tag_member), _("Singletons"), "classviewer-struct",
 				&(tv_iters.tag_function), _("Methods"), "classviewer-method",
-				&(tv_iters.tag_struct), _("Members"), "classviewer-member",
-				&(tv_iters.tag_variable), _("Variables"), "classviewer-var",
 				NULL);
 				//&(tv_iters.tag_namespace), _("Begin"),
 				//&(tv_iters.tag_other), _("Other"), NULL);
@@ -639,7 +645,7 @@ static void init_tag_list(gint idx)
 		{
 			tag_list_add_groups(tag_store,
 				&(tv_iters.tag_namespace), _("Package"), NULL,
-				&(tv_iters.tag_struct), _("Interfaces"), NULL,
+				&(tv_iters.tag_struct), _("Interfaces"), "classviewer-struct",
 				&(tv_iters.tag_class), _("Classes"), "classviewer-class",
 				&(tv_iters.tag_function), _("Methods"), "classviewer-method",
 				&(tv_iters.tag_member), _("Members"), "classviewer-member",
@@ -655,12 +661,24 @@ static void init_tag_list(gint idx)
 				&(tv_iters.tag_struct), _("Interfaces"), "classviewer-struct",
 				&(tv_iters.tag_class), _("Classes"), "classviewer-class",
 				&(tv_iters.tag_function), _("Methods"), "classviewer-method",
-				&(tv_iters.tag_type), _("Types"), NULL,
+				&(tv_iters.tag_type), _("Types"), "classviewer-macro",
 				&(tv_iters.tag_variable), _("Variables"), "classviewer-var",
 				&(tv_iters.tag_other), _("Other"), "classviewer-other",
 				NULL);
 			break;
 		}
+		case GEANY_FILETYPES_BASIC:
+		{
+			tag_list_add_groups(tag_store,
+				&(tv_iters.tag_function), _("Functions"), "classviewer-method",
+				&(tv_iters.tag_variable), _("Variables"), "classviewer-var",
+				&(tv_iters.tag_macro), _("Constants"), "classviewer-macro",
+				&(tv_iters.tag_struct), _("Types"), "classviewer-namespace",
+				&(tv_iters.tag_namespace), _("Labels"), "classviewer-member",
+				&(tv_iters.tag_other), _("Other"), "classviewer-other",
+				NULL);
+			break;
+		}
 		case GEANY_FILETYPES_D:
 		default:
 		{
@@ -897,13 +915,15 @@ int symbols_generate_global_tags(int argc, char **argv, gboolean want_preprocess
 			command = NULL;	// don't preprocess
 
 		geany_debug("Generating %s tags file.", ft->name);
-		status = tm_workspace_create_global_tags(command,
+		tm_get_workspace();
+		status = tm_workspace_create_global_tags(app->configdir, command,
 												 (const char **) (argv + 2),
 												 argc - 2, tags_file, ft->lang);
 		g_free(command);
 		if (! status)
 		{
-			g_printerr(_("Failed to create tags file.\n"));
+			g_printerr(_("Failed to create tags file, perhaps because no tags "
+				"were found.\n"));
 			return 1;
 		}
 	}
@@ -950,9 +970,9 @@ void symbols_show_load_tags_dialog()
 			ft = detect_global_tags_filetype(utf8_fname);
 
 			if (ft != NULL && tm_workspace_load_global_tags(fname, ft->lang))
-				msgwin_status_add(_("Loaded %s tags file '%s'."), ft->name, utf8_fname);
+				ui_set_statusbar(TRUE, _("Loaded %s tags file '%s'."), ft->name, utf8_fname);
 			else
-				msgwin_status_add(_("Could not load tags file '%s'."), utf8_fname);
+				ui_set_statusbar(TRUE, _("Could not load tags file '%s'."), utf8_fname);
 
 			g_free(utf8_fname);
 			g_free(fname);
@@ -1061,6 +1081,11 @@ gboolean symbols_goto_tag(const gchar *name, gboolean definition)
 	{
 		gint new_idx = document_find_by_filename(
 			tmtag->atts.entry.file->work_object.file_name, TRUE);
+		// not found in opened document, should open
+		if (new_idx == -1)
+		{
+			new_idx = document_open_file(tmtag->atts.entry.file->work_object.file_name, FALSE, NULL, NULL);
+		}
 
 		if (navqueue_goto_line(new_idx, tmtag->atts.entry.line))
 			return TRUE;
@@ -1068,9 +1093,9 @@ gboolean symbols_goto_tag(const gchar *name, gboolean definition)
 	// if we are here, there was no match and we are beeping ;-)
 	utils_beep();
 	if (type == forward_types)
-		ui_set_statusbar(_("Forward declaration \"%s\" not found."), name);
+		ui_set_statusbar(FALSE, _("Forward declaration \"%s\" not found."), name);
 	else
-		ui_set_statusbar(_("Definition of \"%s\" not found."), name);
+		ui_set_statusbar(FALSE, _("Definition of \"%s\" not found."), name);
 	return FALSE;
 }
 
