@@ -1,8 +1,8 @@
 /*
  *      templates.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: templates.c 1433 2007-03-31 11:39:56Z ntrel $
+ * $Id: templates.c 2150 2008-01-06 18:11:57Z eht16 $
  */
 
 /*
@@ -32,11 +32,17 @@
 #include "geany.h"
 
 #include "templates.h"
+#include "prefs.h"
 #include "support.h"
 #include "utils.h"
 #include "document.h"
+#include "filetypes.h"
+#include "ui_utils.h"
 
 
+#define TEMPLATE_DATE_FORMAT "%Y-%m-%d"
+#define TEMPLATE_DATETIME_FORMAT "%d.%m.%Y %H:%M:%S %Z"
+
 // default templates, only for initial tempate file creation on first start of Geany
 static const gchar templates_gpl_notice[] = "\
 This program is free software; you can redistribute it and/or modify\n\
@@ -140,7 +146,10 @@ int main(char[][] args)\n\
 }\n\
 ";
 
-static const gchar templates_filetype_php[] = "{fileheader}\n\
+static const gchar templates_filetype_php[] = "\
+<?php\n\
+{fileheader}\
+?>\n\n\
 <!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\"\n\
   \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n\
 <html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n\
@@ -206,6 +215,15 @@ end\n\
 x = StdClass.new\n\
 ";
 
+static const gchar templates_filetype_python[] = "#!/usr/bin/env python\n#\n\
+{fileheader}\n\n\
+\n\
+def main():\n\
+	\n\
+	return 0\n\
+\n\
+if __name__ == '__main__': main()\n\
+";
 
 static gchar *templates[GEANY_MAX_TEMPLATES];
 static gchar *ft_templates[GEANY_MAX_FILE_TYPES] = {NULL};
@@ -289,6 +307,7 @@ static void init_ft_templates(const gchar *year, const gchar *date)
 			case GEANY_FILETYPES_PHP:	TEMPLATES_CREATE_FILE(fname, templates_filetype_php); break;
 			case GEANY_FILETYPES_HTML:	TEMPLATES_CREATE_FILE(fname, templates_filetype_html); break;
 			case GEANY_FILETYPES_RUBY:	TEMPLATES_CREATE_FILE(fname, templates_filetype_ruby); break;
+			case GEANY_FILETYPES_PYTHON:    TEMPLATES_CREATE_FILE(fname, templates_filetype_python); break;
 			default: break;
 		}
 		TEMPLATES_READ_FILE(fname, &ft_templates[ft_id]);
@@ -305,7 +324,11 @@ static void
 on_new_with_template                   (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	document_new_file(NULL, (filetype*) user_data);
+	filetype *ft = user_data;
+	gchar *template = templates_get_template_new_file(ft);
+
+	document_new_file(NULL, ft, template);
+	g_free(template);
 }
 
 
@@ -315,22 +338,22 @@ static void create_new_menu_items()
 	GtkWidget *template_menu = lookup_widget(app->window, "menu_new_with_template1_menu");
 	filetype_id ft_id;
 
-	for (ft_id = 0; ft_id < GEANY_FILETYPES_ALL; ft_id++)
+	for (ft_id = 0; ft_id < GEANY_MAX_FILE_TYPES; ft_id++)
 	{
 		GtkWidget *tmp_menu, *tmp_button;
 		filetype *ft = filetypes[ft_id];
 		const gchar *label = ft->title;
 
 		if (ft_templates[ft_id] == NULL)
-		{
 			continue;
-		}
+		if (ft_id == GEANY_FILETYPES_ALL)
+			label = _("None");
 		tmp_menu = gtk_menu_item_new_with_label(label);
 		tmp_button = gtk_menu_item_new_with_label(label);
 		gtk_widget_show(tmp_menu);
 		gtk_widget_show(tmp_button);
 		gtk_container_add(GTK_CONTAINER(template_menu), tmp_menu);
-		gtk_container_add(GTK_CONTAINER(app->new_file_menu), tmp_button);
+		gtk_container_add(GTK_CONTAINER(ui_widgets.new_file_menu), tmp_button);
 		g_signal_connect((gpointer) tmp_menu, "activate",
 			G_CALLBACK(on_new_with_template), (gpointer) ft);
 		g_signal_connect((gpointer) tmp_button, "activate",
@@ -341,12 +364,8 @@ static void create_new_menu_items()
 
 void templates_init(void)
 {
-	time_t tp = time(NULL);
-	const struct tm *tm = localtime(&tp);
-	gchar *year = g_malloc0(5);
-	gchar *date = utils_get_date();
-
-	strftime(year, 5, "%Y", tm);
+	gchar *year = utils_get_date_time("%Y", NULL);
+	gchar *date = utils_get_date_time(TEMPLATE_DATE_FORMAT, NULL);
 
 	init_general_templates(year, date);
 	init_ft_templates(year, date);
@@ -401,6 +420,7 @@ static gchar *make_comment_block(const gchar *comment_text, gint filetype_idx, g
 		}
 
 		case GEANY_FILETYPES_JS:
+		case GEANY_FILETYPES_HAXE:
 		{
 			line_prefix = "//";
 			break;
@@ -412,6 +432,7 @@ static gchar *make_comment_block(const gchar *comment_text, gint filetype_idx, g
 			break;
 		}
 
+		case GEANY_FILETYPES_HASKELL:
 		case GEANY_FILETYPES_VHDL:
 		case GEANY_FILETYPES_LUA:
 		{
@@ -431,6 +452,12 @@ static gchar *make_comment_block(const gchar *comment_text, gint filetype_idx, g
 			break;
 		}
 
+		case GEANY_FILETYPES_BASIC:
+		{
+			line_prefix = "'";
+			break;
+		}
+
 		case GEANY_FILETYPES_PASCAL:
 		{
 			frame_start = "{\n";
@@ -441,8 +468,8 @@ static gchar *make_comment_block(const gchar *comment_text, gint filetype_idx, g
 
 		case GEANY_FILETYPES_PHP:
 		{
-			frame_start = "<?\n/*\n";
-			frame_end = " */\n?>\n";
+			frame_start = "/*\n";
+			frame_end = " */\n";
 			line_prefix = " *";
 			break;
 		}
@@ -505,16 +532,20 @@ gchar *templates_get_template_licence(gint filetype_idx, gint licence_type)
 }
 
 
-static gchar *get_file_header(filetype *ft, const gchar *fname)
+gchar *templates_get_template_fileheader(gint filetype_idx, const gchar *fname)
 {
 	gchar *template = g_strdup(templates[GEANY_TEMPLATE_FILEHEADER]);
 	gchar *shortname;
 	gchar *result;
-	gchar *date = utils_get_date_time();
+	gchar *date = utils_get_date_time(TEMPLATE_DATETIME_FORMAT, NULL);
+	filetype_id ft_id = filetype_idx;
+	filetype *ft = filetypes[ft_id];
+
+	filetypes_load_config(ft_id);	// load any user extension setting
 
 	if (fname == NULL)
 	{
-		if (FILETYPE_ID(ft) == GEANY_FILETYPES_ALL)
+		if (ft_id == GEANY_FILETYPES_ALL)
 			shortname = g_strdup(GEANY_STRING_UNTITLED);
 		else
 			shortname = g_strconcat(GEANY_STRING_UNTITLED, ".", ft->extension, NULL);
@@ -523,14 +554,11 @@ static gchar *get_file_header(filetype *ft, const gchar *fname)
 		shortname = g_path_get_basename(fname);
 
 	template = utils_str_replace(template, "{filename}", shortname);
-
 	template = utils_str_replace(template, "{gpl}", templates[GEANY_TEMPLATE_GPL]);
-
 	template = utils_str_replace(template, "{bsd}", templates[GEANY_TEMPLATE_BSD]);
-
 	template = utils_str_replace(template, "{datetime}", date);
 
-	result = make_comment_block(template, FILETYPE_ID(ft), 8);
+	result = make_comment_block(template, ft_id, 8);
 
 	g_free(template);
 	g_free(shortname);
@@ -539,19 +567,6 @@ static gchar *get_file_header(filetype *ft, const gchar *fname)
 }
 
 
-gchar *templates_get_template_fileheader(gint idx)
-{
-	gchar *fname;
-	filetype *ft;
-
-	g_return_val_if_fail(DOC_IDX_VALID(idx), NULL);
-	ft = doc_list[idx].file_type;
-	fname = doc_list[idx].file_name;
-
-	return get_file_header(ft, fname);
-}
-
-
 static gchar *get_file_template(filetype *ft)
 {
 	filetype_id ft_id = FILETYPE_ID(ft);
@@ -560,7 +575,7 @@ static gchar *get_file_template(filetype *ft)
 }
 
 
-gchar *templates_get_template_new_file(filetype *ft)
+gchar *templates_get_template_new_file(struct filetype *ft)
 {
 	gchar *ft_template = NULL;
 	gchar *file_header = NULL;
@@ -568,7 +583,7 @@ gchar *templates_get_template_new_file(filetype *ft)
 	if (FILETYPE_ID(ft) == GEANY_FILETYPES_ALL)
 		return get_file_template(ft);
 
-	file_header = get_file_header(ft, NULL);	// file template only used for new files
+	file_header = templates_get_template_fileheader(ft->id, NULL);	// file template only used for new files
 	ft_template = get_file_template(ft);
 	ft_template = utils_str_replace(ft_template, "{fileheader}", file_header);
 	g_free(file_header);
@@ -585,8 +600,8 @@ gchar *templates_get_template_generic(gint template)
 gchar *templates_get_template_function(gint filetype_idx, const gchar *func_name)
 {
 	gchar *template = g_strdup(templates[GEANY_TEMPLATE_FUNCTION]);
-	gchar *date = utils_get_date();
-	gchar *datetime = utils_get_date_time();
+	gchar *date = utils_get_date_time(TEMPLATE_DATE_FORMAT, NULL);
+	gchar *datetime = utils_get_date_time(TEMPLATE_DATETIME_FORMAT, NULL);
 	gchar *result;
 
 	template = utils_str_replace(template, "{date}", date);
@@ -604,7 +619,7 @@ gchar *templates_get_template_function(gint filetype_idx, const gchar *func_name
 
 gchar *templates_get_template_changelog(void)
 {
-	gchar *date = utils_get_date_time();
+	gchar *date = utils_get_date_time(TEMPLATE_DATETIME_FORMAT, NULL);
 	gchar *result = g_strdup(templates[GEANY_TEMPLATE_CHANGELOG]);
 	result = utils_str_replace(result, "{date}", date);
 
@@ -627,13 +642,14 @@ static gchar *templates_replace_all(gchar *text, const gchar *year, const gchar
 {
 	text = utils_str_replace(text, "{year}", year);
 	text = utils_str_replace(text, "{date}", date);
-	text = utils_str_replace(text, "{version}", app->pref_template_version);
-	text = utils_str_replace(text, "{initial}", app->pref_template_initial);
-	text = utils_str_replace(text, "{developer}", app->pref_template_developer);
-	text = utils_str_replace(text, "{mail}", app->pref_template_mail);
-	text = utils_str_replace(text, "{company}", app->pref_template_company);
+	text = utils_str_replace(text, "{version}", prefs.template_version);
+	text = utils_str_replace(text, "{initial}", prefs.template_initial);
+	text = utils_str_replace(text, "{developer}", prefs.template_developer);
+	text = utils_str_replace(text, "{mail}", prefs.template_mail);
+	text = utils_str_replace(text, "{company}", prefs.template_company);
 	text = utils_str_replace(text, "{untitled}", GEANY_STRING_UNTITLED);
 	text = utils_str_replace(text, "{geanyversion}", "Geany " VERSION);
 
 	return text;
 }
+
