@@ -1,8 +1,8 @@
 /*
  *      tools.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2006-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: tools.c 1815 2007-08-23 11:34:06Z ntrel $
+ * $Id: tools.c 2150 2008-01-06 18:11:57Z eht16 $
  */
 
 /*
@@ -163,7 +163,7 @@ void tools_execute_custom_command(gint idx, const gchar *command)
 		return;
 
 	argv = g_strsplit(command, " ", -1);
-	msgwin_status_add(_("Passing data and executing custom command: %s"), command);
+	ui_set_statusbar(TRUE, _("Passing data and executing custom command: %s"), command);
 
 	if (g_spawn_async_with_pipes(NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
 						NULL, NULL, &pid, &stdin_fd, &stdout_fd, &stderr_fd, &error))
