@@ -1,8 +1,8 @@
 /*
  *      tools.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2006-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,17 +18,20 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: tools.h 1334 2007-02-25 14:26:55Z eht16 $
+ * $Id: tools.h 2150 2008-01-06 18:11:57Z eht16 $
  */
 
 
 #ifndef GEANY_TOOLS_H
 #define GEANY_TOOLS_H 1
 
-void tools_show_dialog_insert_special_chars();
-
 void tools_create_insert_custom_command_menu_items();
 
 void tools_execute_custom_command(gint idx, const gchar *command);
 
+
+void tools_word_count();
+
+void tools_color_chooser(gchar *color);
+
 #endif
