@@ -1,8 +1,8 @@
 /*
  *      utils.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: utils.h 1914 2007-09-27 15:09:54Z ntrel $
+ * $Id: utils.h 2151 2008-01-06 19:59:01Z eht16 $
  */
 
 
@@ -55,11 +55,6 @@ gboolean utils_goto_line(gint idx, gint line);
 
 gint utils_write_file(const gchar *filename, const gchar *text);
 
-/**
- * (stolen from anjuta and modified)
- * Search backward through size bytes looking for a '<', then return the tag if any
- * @return The tag name
- */
 gchar *utils_find_open_xml_tag(const gchar sel[], gint size, gboolean check_tag);
 
 gboolean utils_check_disk_status(gint idx, gboolean force);
@@ -71,10 +66,7 @@ gint utils_get_current_function(gint idx, const gchar **tagname);
 gint utils_get_eol_char_len(gint idx);
 
 /* returns the end-of-line character(s) of the specified editor */
-gchar *utils_get_eol_char(gint idx);
-
-/* mainly debug function, to get TRUE or FALSE as ascii from a gboolean */
-gchar *utils_btoa(gboolean sbool);
+const gchar *utils_get_eol_char(gint idx);
 
 gboolean utils_atob(const gchar *str);
 
@@ -97,7 +89,7 @@ gchar utils_brace_opposite(gchar ch);
 
 gchar *utils_get_hostname();
 
-gint utils_make_settings_dir(const gchar *dir, const gchar *data_dir, const gchar *doc_dir);
+gint utils_make_settings_dir();
 
 
 gboolean utils_string_replace_all(GString *str, const gchar *needle, const gchar *replace);
@@ -107,9 +99,7 @@ gchar *utils_str_replace(gchar *haystack, const gchar *needle, const gchar *repl
 gint utils_strpos(const gchar* haystack, const gchar * needle);
 
 
-gchar *utils_get_date_time();
-
-gchar *utils_get_date();
+gchar *utils_get_date_time(const gchar *format, time_t *time_to_use);
 
 gchar *utils_get_initials(gchar *name);
 
@@ -125,7 +115,9 @@ void utils_replace_filename(gint idx);
 
 gchar *utils_get_hex_from_color(GdkColor *color);
 
-gchar *utils_get_current_file_dir();
+const gchar *utils_get_default_dir_utf8();
+
+gchar *utils_get_current_file_dir_utf8();
 
 void utils_beep();
 
@@ -180,4 +172,6 @@ GSList *utils_get_file_list(const gchar *path, guint *length, GError **error);
 
 gboolean utils_str_has_upper(const gchar *str);
 
+gint utils_is_file_writeable(const gchar *locale_filename);
+
 #endif
