@@ -1,8 +1,8 @@
 /*
  *      vte.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: vte.c 1929 2007-10-03 11:25:28Z ntrel $
+ * $Id: vte.c 2158 2008-01-10 17:11:19Z ntrel $
  */
 
 /*
@@ -41,12 +41,14 @@
 #include "utils.h"
 #include "document.h"
 #include "callbacks.h"
+#include "geanywraplabel.h"
 
 
 VteInfo vte_info;
 
 extern gchar **environ;
 static pid_t pid;
+static gboolean clean = TRUE;
 static GModule *module = NULL;
 static struct VteFunctions *vf;
 static gboolean popup_menu_created = FALSE;
@@ -63,10 +65,14 @@ static const gchar VTE_WORDCHARS[] = "-A-Za-z0-9,./?%&#:_";
 static void create_vte();
 static void vte_start(GtkWidget *widget);
 static gboolean vte_button_pressed(GtkWidget *widget, GdkEventButton *event, gpointer user_data);
+static gboolean vte_keyrelease(GtkWidget *widget, GdkEventKey *event, gpointer data);
 static gboolean vte_keypress(GtkWidget *widget, GdkEventKey *event, gpointer data);
 static void vte_register_symbols(GModule *module);
 static void vte_popup_menu_clicked(GtkMenuItem *menuitem, gpointer user_data);
 static GtkWidget *vte_create_popup_menu(void);
+void vte_commit(VteTerminal *vte, gchar *arg1, guint arg2, gpointer user_data);
+void vte_drag_data_received(GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y,
+							GtkSelectionData *data, guint info, guint ltime);
 
 
 enum
@@ -74,7 +80,22 @@ enum
 	POPUP_COPY,
 	POPUP_PASTE,
 	POPUP_CHANGEPATH,
-	POPUP_PREFERENCES
+	POPUP_RESTARTTERMINAL,
+	POPUP_PREFERENCES,
+	TARGET_UTF8_STRING = 0,
+	TARGET_TEXT,
+	TARGET_COMPOUND_TEXT,
+	TARGET_STRING,
+	TARGET_TEXT_PLAIN,
+};
+
+static const GtkTargetEntry dnd_targets[] =
+{
+  { "UTF8_STRING", 0, TARGET_UTF8_STRING },
+  { "TEXT", 0, TARGET_TEXT },
+  { "COMPOUND_TEXT", 0, TARGET_COMPOUND_TEXT },
+  { "STRING", 0, TARGET_STRING },
+  { "text/plain", 0, TARGET_TEXT_PLAIN },
 };
 
 
@@ -141,11 +162,18 @@ void vte_init(void)
 		return;
 	}
 
-	if (vte_info.lib_vte && strlen(vte_info.lib_vte))
+	if (vte_info.lib_vte && vte_info.lib_vte[0] != '\0')
 	{
 		module = g_module_open(vte_info.lib_vte, G_MODULE_BIND_LAZY);
 	}
+#ifdef VTE_MODULE_PATH
 	else
+	{
+		module = g_module_open(VTE_MODULE_PATH, G_MODULE_BIND_LAZY);
+	}
+#endif
+
+	if (module == NULL)
 	{
 		gint i;
 		const gchar *sonames[] = {  "libvte.so", "libvte.so.4",
@@ -198,16 +226,19 @@ static void create_vte()
 	gtk_widget_set_size_request(GTK_WIDGET(vte), 10, 10);
 	vf->vte_terminal_set_size(VTE_TERMINAL(vte), 30, 1);
 
-	//vf->vte_terminal_set_encoding(VTE_TERMINAL(vte), "UTF-8");
 	vf->vte_terminal_set_mouse_autohide(VTE_TERMINAL(vte), TRUE);
 	vf->vte_terminal_set_word_chars(VTE_TERMINAL(vte), VTE_WORDCHARS);
 
+    gtk_drag_dest_set(vte, GTK_DEST_DEFAULT_ALL,
+		dnd_targets, G_N_ELEMENTS(dnd_targets), GDK_ACTION_COPY);
+
 	g_signal_connect(G_OBJECT(vte), "child-exited", G_CALLBACK(vte_start), NULL);
 	g_signal_connect(G_OBJECT(vte), "button-press-event", G_CALLBACK(vte_button_pressed), NULL);
 	g_signal_connect(G_OBJECT(vte), "event", G_CALLBACK(vte_keypress), NULL);
+	g_signal_connect(G_OBJECT(vte), "key-release-event", G_CALLBACK(vte_keyrelease), NULL);
+	g_signal_connect(G_OBJECT(vte), "commit", G_CALLBACK(vte_commit), NULL);
 	g_signal_connect(G_OBJECT(vte), "motion-notify-event", G_CALLBACK(on_motion_event), NULL);
-	//g_signal_connect(G_OBJECT(vte), "drag-data-received", G_CALLBACK(vte_drag_data_received), NULL);
-	//g_signal_connect(G_OBJECT(vte), "drag-drop", G_CALLBACK(vte_drag_drop), NULL);
+	g_signal_connect(G_OBJECT(vte), "drag-data-received", G_CALLBACK(vte_drag_data_received), NULL);
 
 	vte_start(vte);
 
@@ -238,9 +269,24 @@ void vte_close(void)
 }
 
 
+static gboolean vte_keyrelease(GtkWidget *widget, GdkEventKey *event, gpointer data)
+{
+	if (event->keyval == GDK_Return ||
+			 event->keyval == GDK_ISO_Enter ||
+			 event->keyval == GDK_KP_Enter)
+	{
+		clean = TRUE; // assume any text on the prompt has been executed when pressing Enter/Return
+	}
+	return FALSE;
+}
+
+
 static gboolean vte_keypress(GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
-	if (event->type != GDK_KEY_RELEASE || vc->enable_bash_keys)
+	if (vc->enable_bash_keys)
+		return FALSE;	// Ctrl-[CD] will be handled by the VTE itself
+
+	if (event->type != GDK_KEY_RELEASE)
 		return FALSE;
 
 	if ((event->keyval == GDK_c ||
@@ -266,6 +312,12 @@ static gboolean vte_keypress(GtkWidget *widget, GdkEventKey *event, gpointer dat
 }
 
 
+void vte_commit(VteTerminal *vte, gchar *arg1, guint arg2, gpointer user_data)
+{
+	clean = FALSE;
+}
+
+
 static void vte_start(GtkWidget *widget)
 {
 	VteTerminal *vte = VTE_TERMINAL(widget);
@@ -285,6 +337,21 @@ static void vte_start(GtkWidget *widget)
 	}
 	else
 		pid = 0; // use 0 as invalid pid
+
+	clean = TRUE;
+}
+
+
+static void vte_restart(GtkWidget *widget)
+{
+		vte_get_working_directory(); // try to keep the working directory when restarting the VTE
+	 	if (pid > 0)
+		{
+			kill(pid, SIGINT);
+			pid = 0;
+		}
+		vf->vte_terminal_reset(VTE_TERMINAL(widget), TRUE, TRUE);
+		vte_start(widget);
 }
 
 
@@ -299,6 +366,7 @@ static gboolean vte_button_pressed(GtkWidget *widget, GdkEventButton *event, gpo
 			popup_menu_created = TRUE;
 		}
 
+		gtk_widget_grab_focus(vc->vte);
 		gtk_menu_popup(GTK_MENU(vc->menu), NULL, NULL, NULL, NULL, event->button, event->time);
 	}
 
@@ -313,9 +381,7 @@ static void vte_register_symbols(GModule *mod)
 	g_module_symbol(mod, "vte_terminal_fork_command", (void*)&vf->vte_terminal_fork_command);
 	g_module_symbol(mod, "vte_terminal_set_word_chars", (void*)&vf->vte_terminal_set_word_chars);
 	g_module_symbol(mod, "vte_terminal_set_mouse_autohide", (void*)&vf->vte_terminal_set_mouse_autohide);
-	g_module_symbol(mod, "vte_terminal_set_encoding", (void*)&vf->vte_terminal_set_encoding);
 	g_module_symbol(mod, "vte_terminal_reset", (void*)&vf->vte_terminal_reset);
-	g_module_symbol(mod, "vte_terminal_set_cursor_blinks", (void*)&vf->vte_terminal_set_cursor_blinks);
 	g_module_symbol(mod, "vte_terminal_get_type", (void*)&vf->vte_terminal_get_type);
 	g_module_symbol(mod, "vte_terminal_set_scroll_on_output", (void*)&vf->vte_terminal_set_scroll_on_output);
 	g_module_symbol(mod, "vte_terminal_set_scroll_on_keystroke", (void*)&vf->vte_terminal_set_scroll_on_keystroke);
@@ -370,6 +436,11 @@ static void vte_popup_menu_clicked(GtkMenuItem *menuitem, gpointer user_data)
 				vte_cwd(doc_list[idx].file_name, TRUE);
 			break;
 		}
+		case POPUP_RESTARTTERMINAL:
+		{
+			vte_restart(vc->vte);
+			break;
+		}
 		case POPUP_PREFERENCES:
 		{
 			GtkWidget *notebook;
@@ -401,19 +472,19 @@ static GtkWidget *vte_create_popup_menu(void)
 	gtk_container_add(GTK_CONTAINER(menu), item);
 	g_signal_connect((gpointer)item, "activate", G_CALLBACK(vte_popup_menu_clicked), GINT_TO_POINTER(POPUP_PASTE));
 
-	item = gtk_image_menu_item_new_with_label("Change current working directory");
-	gtk_widget_show(item);
-	gtk_container_add(GTK_CONTAINER(menu), item);
-	g_signal_connect((gpointer)item, "activate", G_CALLBACK(vte_popup_menu_clicked), GINT_TO_POINTER(POPUP_CHANGEPATH));
-
 	item = gtk_separator_menu_item_new();
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
 
-	item = gtk_image_menu_item_new_from_stock("gtk-preferences", NULL);
+	item = gtk_image_menu_item_new_with_label(_("Set Path From Document"));
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	g_signal_connect((gpointer)item, "activate", G_CALLBACK(vte_popup_menu_clicked), GINT_TO_POINTER(POPUP_PREFERENCES));
+	g_signal_connect((gpointer)item, "activate", G_CALLBACK(vte_popup_menu_clicked), GINT_TO_POINTER(POPUP_CHANGEPATH));
+
+	item = gtk_image_menu_item_new_with_label(_("Restart Terminal"));
+	gtk_widget_show(item);
+	gtk_container_add(GTK_CONTAINER(menu), item);
+	g_signal_connect((gpointer)item, "activate", G_CALLBACK(vte_popup_menu_clicked), GINT_TO_POINTER(POPUP_RESTARTTERMINAL));
 
 	item = gtk_separator_menu_item_new();
 	gtk_widget_show(item);
@@ -425,17 +496,35 @@ static GtkWidget *vte_create_popup_menu(void)
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
 
-	msgwin_menu_add_common_items(GTK_MENU(menu));
-
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), vc->im_submenu);
 
+	item = gtk_separator_menu_item_new();
+	gtk_widget_show(item);
+	gtk_container_add(GTK_CONTAINER(menu), item);
+
+	item = gtk_image_menu_item_new_from_stock("gtk-preferences", NULL);
+	gtk_widget_show(item);
+	gtk_container_add(GTK_CONTAINER(menu), item);
+	g_signal_connect((gpointer)item, "activate", G_CALLBACK(vte_popup_menu_clicked), GINT_TO_POINTER(POPUP_PREFERENCES));
+
+	msgwin_menu_add_common_items(GTK_MENU(menu));
+
 	return menu;
 }
 
 
-void vte_send_cmd(const gchar *cmd)
+/* if the command could be executed, TRUE is returned, FALSE otherwise (i.e. there was some text
+ * on the prompt). */
+gboolean vte_send_cmd(const gchar *cmd)
 {
-	vf->vte_terminal_feed_child(VTE_TERMINAL(vc->vte), cmd, strlen(cmd));
+	if (clean)
+	{
+		vf->vte_terminal_feed_child(VTE_TERMINAL(vc->vte), cmd, strlen(cmd));
+		clean = TRUE; // vte_terminal_feed_child() also marks the vte as not clean
+		return TRUE;
+	}
+	else
+		return FALSE;
 }
 
 
@@ -480,21 +569,31 @@ const gchar* vte_get_working_directory()
 }
 
 
-// if force is set to TRUE, it will always change the cwd
+/* Changes the current working directory of the VTE to the path of the given filename.
+ * filename is expected to be in UTF-8 encoding.
+ * filename can also be a path, then it is used directly.
+ * If force is set to TRUE, it will always change the cwd
+ * */
 void vte_cwd(const gchar *filename, gboolean force)
 {
 	if (vte_info.have_vte && (vc->follow_path || force) && filename != NULL)
 	{
 		gchar *path;
 
-		path = g_path_get_dirname(filename);
-		vte_get_working_directory();	// refresh vte_info.dir
+		if (g_file_test(filename, G_FILE_TEST_IS_DIR))
+			path = g_strdup(filename);
+		else
+			path = g_path_get_dirname(filename);
+
+		vte_get_working_directory(); // refresh vte_info.dir
 		if (! utils_str_equal(path, vte_info.dir))
 		{
 			// use g_shell_quote to avoid problems with spaces, '!' or something else in path
 			gchar *quoted_path = g_shell_quote(path);
 			gchar *cmd = g_strconcat("cd ", quoted_path, "\n", NULL);
-			vte_send_cmd(cmd);
+			if (! vte_send_cmd(cmd))
+				ui_set_statusbar(FALSE,
+		_("Could not change the directory in the VTE because it probably contains a command."));
 			g_free(quoted_path);
 			g_free(cmd);
 		}
@@ -502,37 +601,31 @@ void vte_cwd(const gchar *filename, gboolean force)
 	}
 }
 
-/*
-void vte_drag_data_received(GtkWidget *widget, GdkDragContext  *drag_context, gint x, gint y,
-							GtkSelectionData *data, guint info, guint time)
+
+void vte_drag_data_received(GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y,
+							GtkSelectionData *data, guint info, guint ltime)
 {
-	geany_debug("length: %d, format: %d, action: %d", data->length, data->format, drag_context->action);
-	if ((data->length >= 0) && (data->format == 8))
+	if (info == TARGET_TEXT_PLAIN)
 	{
-		if (drag_context->action == GDK_ACTION_ASK)
-		{
-			gint accept = TRUE;
-			// should I check the incoming data?
-			if (accept)
-				drag_context->action = GDK_ACTION_COPY;
-		}
-		gtk_drag_finish(drag_context, TRUE, FALSE, time);
-		return;
+		if (data->format == 8 && data->length > 0)
+			vf->vte_terminal_feed_child(VTE_TERMINAL(widget),
+				(const gchar*) data->data, data->length);
 	}
-	gtk_drag_finish(drag_context, FALSE, FALSE, time);
+	else
+	{
+		gchar *text = (gchar*) gtk_selection_data_get_text(data);
+		if (NZV(text))
+			vf->vte_terminal_feed_child(VTE_TERMINAL(widget), text, strlen(text));
+		g_free (text);
+	}
+	gtk_drag_finish(drag_context, TRUE, FALSE, ltime);
 }
 
 
-gboolean vte_drag_drop(GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y, guint time,
-					   gpointer user_data)
+static void check_run_in_vte_toggled(GtkToggleButton *togglebutton, GtkWidget *user_data)
 {
-
-	GdkAtom *atom;
-	gtk_drag_get_data(widget, drag_context, atom, time);
-	geany_debug("%s", GDK_ATOM_TO_POINTER(atom));
-	return TRUE;
+	gtk_widget_set_sensitive(user_data, gtk_toggle_button_get_active(togglebutton));
 }
-*/
 
 
 void vte_append_preferences_tab()
@@ -541,8 +634,9 @@ void vte_append_preferences_tab()
 	{
 		GtkWidget *notebook, *vbox, *label, *alignment, *table, *frame, *box;
 		GtkWidget *font_term, *color_fore, *color_back, *spin_scrollback, *entry_emulation;
-		GtkWidget *check_scroll_key, *check_scroll_out, *check_follow_path, *check_ignore_menu_key;
-		GtkWidget *check_run_in_vte, *entry_shell, *button_shell, *image_shell;
+		GtkWidget *check_scroll_key, *check_scroll_out, *check_follow_path;
+		GtkWidget *check_enable_bash_keys, *check_ignore_menu_key;
+		GtkWidget *check_run_in_vte, *check_skip_script, *entry_shell, *button_shell, *image_shell;
 		GtkTooltips *tooltips;
 		GtkObject *spin_scrollback_adj;
 
@@ -556,7 +650,7 @@ void vte_append_preferences_tab()
 		label = gtk_label_new(_("Terminal"));
 		vte_prefs_tab_num = gtk_notebook_append_page(GTK_NOTEBOOK(notebook), frame, label);
 
-		label = gtk_label_new(_("These settings for the virtual terminal emulator widget (VTE) only apply if the VTE library could be loaded."));
+		label = geany_wrap_label_new(_("These settings for the virtual terminal emulator widget (VTE) only apply if the VTE library could be loaded."));
 		gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 6);
 		gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_FILL);
 		gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
@@ -666,6 +760,12 @@ void vte_append_preferences_tab()
 		gtk_button_set_focus_on_click(GTK_BUTTON(check_scroll_out), FALSE);
 		gtk_container_add(GTK_CONTAINER(box), check_scroll_out);
 
+		check_enable_bash_keys = gtk_check_button_new_with_mnemonic(_("Override Geany keybindings"));
+		gtk_tooltips_set_tip(tooltips, check_enable_bash_keys,
+			_("Allows the VTE to receive keyboard shortcuts (apart from focus commands)."), NULL);
+		gtk_button_set_focus_on_click(GTK_BUTTON(check_enable_bash_keys), FALSE);
+		gtk_container_add(GTK_CONTAINER(box), check_enable_bash_keys);
+
 		check_ignore_menu_key = gtk_check_button_new_with_mnemonic(_("Disable menu shortcut key (F10 by default)"));
 		gtk_tooltips_set_tip(tooltips, check_ignore_menu_key, _("This option disables the keybinding to popup the menu bar (default is F10). Disabling it can be useful if you use, for example, Midnight Commander within the VTE."), NULL);
 		gtk_button_set_focus_on_click(GTK_BUTTON(check_ignore_menu_key), FALSE);
@@ -676,10 +776,22 @@ void vte_append_preferences_tab()
 		gtk_button_set_focus_on_click(GTK_BUTTON(check_follow_path), FALSE);
 		gtk_container_add(GTK_CONTAINER(box), check_follow_path);
 
+		// create check_skip_script checkbox before the check_skip_script checkbox to be able to
+		// use the object for the toggled handler of check_skip_script checkbox
+		check_skip_script = gtk_check_button_new_with_mnemonic(_("Don't use run script"));
+		gtk_tooltips_set_tip(tooltips, check_skip_script, _("Don't use the simple run script which is usually used to display the exit status of the executed program."), NULL);
+		gtk_button_set_focus_on_click(GTK_BUTTON(check_skip_script), FALSE);
+		gtk_widget_set_sensitive(check_skip_script, vc->run_in_vte);
+
 		check_run_in_vte = gtk_check_button_new_with_mnemonic(_("Execute programs in VTE"));
 		gtk_tooltips_set_tip(tooltips, check_run_in_vte, _("Run programs in VTE instead of opening a terminal emulation window. Please note, programs executed in VTE cannot be stopped."), NULL);
 		gtk_button_set_focus_on_click(GTK_BUTTON(check_run_in_vte), FALSE);
 		gtk_container_add(GTK_CONTAINER(box), check_run_in_vte);
+		g_signal_connect((gpointer) check_run_in_vte, "toggled",
+			G_CALLBACK(check_run_in_vte_toggled), check_skip_script);
+
+		// now add the check_skip_script checkbox after the check_run_in_vte checkbox
+		gtk_container_add(GTK_CONTAINER(box), check_skip_script);
 
 		gtk_box_pack_start(GTK_BOX(vbox), box, FALSE, FALSE, 0);
 
@@ -699,12 +811,16 @@ void vte_append_preferences_tab()
 				gtk_widget_ref(check_scroll_key),	(GDestroyNotify) gtk_widget_unref);
 		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "check_scroll_out",
 				gtk_widget_ref(check_scroll_out),	(GDestroyNotify) gtk_widget_unref);
+		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "check_enable_bash_keys",
+				gtk_widget_ref(check_enable_bash_keys),	(GDestroyNotify) gtk_widget_unref);
 		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "check_ignore_menu_key",
 				gtk_widget_ref(check_ignore_menu_key),	(GDestroyNotify) gtk_widget_unref);
 		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "check_follow_path",
 				gtk_widget_ref(check_follow_path),	(GDestroyNotify) gtk_widget_unref);
 		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "check_run_in_vte",
 				gtk_widget_ref(check_run_in_vte),	(GDestroyNotify) gtk_widget_unref);
+		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "check_skip_script",
+				gtk_widget_ref(check_skip_script),	(GDestroyNotify) gtk_widget_unref);
 
 		gtk_widget_show_all(frame);
 
