@@ -1,8 +1,8 @@
 /*
  *      win32.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: win32.h 1663 2007-07-04 17:08:53Z eht16 $
+ *  $Id: win32.h 2150 2008-01-06 18:11:57Z eht16 $
  */
 
 
@@ -27,7 +27,7 @@
 
 void win32_show_pref_file_dialog(GtkEntry *item);
 
-gboolean win32_show_file_dialog(gboolean file_open);
+gboolean win32_show_file_dialog(gboolean file_open, const gchar *initial_dir);
 
 void win32_show_font_dialog(void);
 
@@ -44,13 +44,19 @@ gint win32_message_dialog_unsaved(const gchar *msg);
 /* Just a simple wrapper function to open a browser window */
 void win32_open_browser(const gchar *uri);
 
-/* Shows a file open dialog.
- * If allow_new_file is set, the file to be opened doesn't have to exist.
- * The selected file name is returned. */
-gchar *win32_show_project_open_dialog(const gchar *title, const gchar *initial_dir, gboolean allow_new_file);
+gchar *win32_show_project_open_dialog(GtkWidget *parent, const gchar *title,
+								      const gchar *initial_dir, gboolean allow_new_file,
+								      gboolean project_file_filter);
 
 /* Shows a folder selection dialog.
  * The selected folder name is returned. */
-gchar *win32_show_project_folder_dialog(const gchar *title, const gchar *initial_dir);
+gchar *win32_show_project_folder_dialog(GtkWidget *parent, const gchar *title,
+										const gchar *initial_dir);
+
+gint win32_check_write_permission(const gchar *dir);
+
+void win32_init_debug_code();
+
+gchar *win32_get_appdata_folder();
 
 #endif
