@@ -45,7 +45,7 @@
 *   DATA DECLARATIONS
 */
 
-enum { NumTokens = 8 };
+enum { NumTokens = 12 };
 
 typedef enum eException {
     ExceptionNone, ExceptionEOF, ExceptionFormattingError,
@@ -114,7 +114,8 @@ typedef enum eTokenType {
     TOKEN_PAREN_NAME,	/* a single name in parentheses */
     TOKEN_SEMICOLON,	/* the semicolon character */
     TOKEN_SPEC,		/* a storage class specifier, qualifier, type, etc. */
-    TOKEN_STAR,   /* pointer detection */
+    TOKEN_STAR,		/* pointer detection */
+    TOKEN_ARRAY,	/* array detection */
     TOKEN_COUNT
 } tokenType;
 
@@ -167,13 +168,10 @@ typedef struct sTokenInfo {
     tokenType     type;
     keywordId     keyword;
     vString*      name;		/* the name of the token */
-    int           pointerOrder; /* 0=no pointer, 1=pointer, 2=pointer
-				   to pointer, etc */
     unsigned long lineNumber;	/* line number of tag */
     fpos_t        filePosition;	/* file position of line containing name */
 } tokenInfo;
 
-
 typedef enum eImplementation {
     IMP_DEFAULT,
     IMP_ABSTRACT,
@@ -191,8 +189,6 @@ typedef struct sStatementInfo {
     boolean	haveQualifyingName;  /* do we have a name we are considering? */
     boolean	gotParenName;	/* was a name inside parentheses parsed yet? */
     boolean	gotArgs;	/* was a list of parameters parsed yet? */
-    int         pointerOrder;   /* 0=no pointer, 1=pointer, 2=pointer
-				   to pointer, etc */
     impType	implementation;	/* abstract or concrete implementation? */
     unsigned int tokenIndex;	/* currently active token */
     tokenInfo*	token [((int) NumTokens)];
@@ -233,7 +229,6 @@ typedef enum eTagType {
 } tagType;
 
 typedef struct sParenInfo {
-    int pointerOrder;
     boolean isParamList;
     boolean isKnrParamList;
     boolean isNameCandidate;
@@ -359,7 +354,7 @@ static const keywordDesc KeywordTable [] = {
 	{ "goto",           KEYWORD_GOTO,           { 1, 1, 1, 1, 0 } },
 	{ "if",             KEYWORD_IF,             { 1, 1, 1, 1, 0 } },
 	{ "implements",     KEYWORD_IMPLEMENTS,     { 0, 0, 0, 1, 0 } },
-	{ "import",         KEYWORD_IMPORT,         { 0, 0, 0, 1, 0 } },
+	{ "import",         KEYWORD_IMPORT,         { 0, 1, 0, 1, 0 } },	// D
 	{ "inline",         KEYWORD_INLINE,         { 0, 1, 0, 0, 0 } },
 	{ "inout",          KEYWORD_INOUT,          { 0, 0, 0, 0, 1 } },
 	{ "input",          KEYWORD_INPUT,          { 0, 0, 0, 0, 1 } },
@@ -374,7 +369,7 @@ static const keywordDesc KeywordTable [] = {
 	{ "m_state",        KEYWORD_M_STATE,        { 0, 0, 0, 0, 1 } },
 	{ "m_trans",        KEYWORD_M_TRANS,        { 0, 0, 0, 0, 1 } },
 	{ "mutable",        KEYWORD_MUTABLE,        { 0, 1, 0, 0, 0 } },
-	{ "module",         KEYWORD_MODULE,         { 0, 1, 0, 0, 0 } },
+	{ "module",         KEYWORD_MODULE,         { 0, 1, 0, 0, 0 } },	// D
 	{ "namespace",      KEYWORD_NAMESPACE,      { 0, 1, 1, 0, 0 } },
 	{ "native",         KEYWORD_NATIVE,         { 0, 0, 0, 1, 0 } },
 	{ "new",            KEYWORD_NEW,            { 0, 1, 1, 1, 0 } },
@@ -404,7 +399,7 @@ static const keywordDesc KeywordTable [] = {
 	{ "synchronized",   KEYWORD_SYNCHRONIZED,   { 0, 0, 0, 1, 0 } },
 	{ "task",           KEYWORD_TASK,           { 0, 0, 0, 0, 1 } },
 	{ "template",       KEYWORD_TEMPLATE,       { 0, 1, 0, 0, 0 } },
-	{ "this",           KEYWORD_THIS,           { 0, 1, 1, 1, 0 } },
+	{ "this",           KEYWORD_THIS,           { 0, 0, 1, 1, 0 } },	// allow D ctor tags
 	{ "throw",          KEYWORD_THROW,          { 0, 1, 1, 1, 0 } },
 	{ "throws",         KEYWORD_THROWS,         { 0, 0, 0, 1, 0 } },
 	{ "trans",          KEYWORD_TRANS,          { 0, 0, 0, 0, 1 } },
@@ -432,6 +427,7 @@ static const keywordDesc KeywordTable [] = {
 */
 static void createTags (const unsigned int nestLevel, statementInfo *const parent);
 static void copyToken (tokenInfo *const dest, const tokenInfo *const src);
+static const char *getVarType (const statementInfo *const st);
 
 /*
 *   FUNCTION DEFINITIONS
@@ -441,7 +437,7 @@ static void copyToken (tokenInfo *const dest, const tokenInfo *const src);
 #if DEBUG_C
 static char *tokenTypeName[] = {
     "none", "args", "'}'", "'{'", "','", "'::'", "keyword", "name",
-    "package", "paren-name", "';'",	"spec", "*","count"
+    "package", "paren-name", "';'",	"spec", "*", "[]", "count"
 };
 
 static char *tagScopeNames[] = {
@@ -509,7 +505,6 @@ static void initToken (tokenInfo* const token)
 {
     token->type		= TOKEN_NONE;
     token->keyword	= KEYWORD_NONE;
-    token->pointerOrder = 0;
     token->lineNumber	= getSourceLineNumber ();
     token->filePosition	= getInputFilePosition ();
     vStringClear (token->name);
@@ -608,7 +603,7 @@ static const char *tokenString (const tokenType type)
 {
     static const char *const names [] = {
 	"none", "args", "}", "{", "comma", "double colon", "keyword", "name",
-	"package", "paren-name", "semicolon", "specifier", "*","count"
+	"package", "paren-name", "semicolon", "specifier", "*", "[]", "count"
     };
     Assert (sizeof (names) / sizeof (names [0]) == TOKEN_COUNT);
     Assert ((int) type < TOKEN_COUNT);
@@ -822,7 +817,6 @@ static void reinitStatement (statementInfo *const st, const boolean partial)
 	    st->declaration = DECL_NONE;
     }
     st->gotParenName	= FALSE;
-    st->pointerOrder    = 0;
     st->implementation	= IMP_DEFAULT;
     st->gotArgs		= FALSE;
     st->gotName		= FALSE;
@@ -1074,10 +1068,50 @@ static void addOtherFields (tagEntryInfo* const tag, const tagType type,
 	{
 		if (((TOKEN_NAME == st->firstToken->type) || isDataTypeKeyword(st->firstToken))
 			  && (0 != strcmp(vStringValue(st->firstToken->name), tag->name)))
-				tag->extensionFields.varType = vStringValue(st->firstToken->name);
+				tag->extensionFields.varType = getVarType(st);
 	}
 }
 
+static const char *getVarType (const statementInfo *const st)
+{
+	static vString *vt = NULL;
+	unsigned int i;
+
+	if (! st->gotArgs)
+		return vStringValue(st->firstToken->name);	/* ignore non-functions */
+
+	if (vt == NULL)
+		vt = vStringNew();
+	else
+		vStringClear(vt);
+
+	for (i = 0; i < st->tokenIndex; i++)
+	{
+		tokenInfo *t = st->token[i];
+
+		switch (t->type)
+		{
+			case TOKEN_NAME:	/* user typename */
+				if (strcmp(vStringValue(t->name), vStringValue(st->firstToken->name)) != 0)
+					continue;
+				break;
+			case TOKEN_KEYWORD:
+				if (t->keyword != KEYWORD_EXTERN && t->keyword != KEYWORD_STATIC)	/* uninteresting keywords */
+					break;
+				continue;
+			case TOKEN_STAR: vStringCatS(vt, " *"); continue;
+			case TOKEN_ARRAY: vStringCatS(vt, "[]"); continue;
+			default: continue;
+		}
+		if (vStringLength(vt) > 0)
+			if (isalpha(vStringValue(vt)[vStringLength(vt) - 1]))
+				vStringPut(vt, ' ');
+		vStringCat(vt, t->name);
+	}
+	vStringTerminate(vt);
+	return vStringValue(vt);
+}
+
 static void addContextSeparator (vString *const scope)
 {
     if (isLanguage (Lang_c)  ||  isLanguage (Lang_cpp))
@@ -1202,7 +1236,6 @@ static void makeTag (const tokenInfo *const token,
 	e.kindName	= tagName (type);
 	e.kind		= tagLetter (type);
 	e.type = type;
-	e.pointerOrder = token->pointerOrder;
 
 	findScopeHierarchy (scope, st);
 	addOtherFields (&e, type, st, scope);
@@ -1251,13 +1284,13 @@ static void qualifyFunctionTag (const statementInfo *const st,
 {
     if (isType (nameToken, TOKEN_NAME))
     {
-	const tagType type = (isLanguage (Lang_java) || isLanguage (Lang_csharp))
-								? TAG_METHOD : TAG_FUNCTION;
-	const boolean isFileScope =
-			(boolean) (st->member.access == ACCESS_PRIVATE ||
-			(!isMember (st)  &&  st->scope == SCOPE_STATIC));
+		const tagType type = (isLanguage (Lang_java) || isLanguage (Lang_csharp))
+									? TAG_METHOD : TAG_FUNCTION;
+		const boolean isFileScope =
+				(boolean) (st->member.access == ACCESS_PRIVATE ||
+				(!isMember (st)  &&  st->scope == SCOPE_STATIC));
 
-	makeTag (nameToken, st, isFileScope, type);
+		makeTag (nameToken, st, isFileScope, type);
     }
 }
 
@@ -1265,7 +1298,7 @@ static void qualifyFunctionDeclTag (const statementInfo *const st,
 				    const tokenInfo *const nameToken)
 {
     if (! isType (nameToken, TOKEN_NAME))
-	;
+		;
     else if (isLanguage (Lang_java) || isLanguage (Lang_csharp))
 		qualifyFunctionTag (st, nameToken);
     else if (st->scope == SCOPE_TYPEDEF)
@@ -1655,7 +1688,6 @@ static void copyToken (tokenInfo *const dest, const tokenInfo *const src)
     dest->keyword      = src->keyword;
     dest->filePosition = src->filePosition;
     dest->lineNumber   = src->lineNumber;
-    dest->pointerOrder = src->pointerOrder;
     vStringCopy (dest->name, src->name);
 }
 
@@ -2051,7 +2083,6 @@ static int parseParens (statementInfo *const st, parenInfo *const info)
 	    case '&':
 	    case '*':
 		/* DEBUG_PRINT("parseParens, po++\n"); */
-		info->pointerOrder++;
 		info->isKnrParamList = FALSE;
 		if (identifierCount == 0)
 		    info->isParamList = FALSE;
@@ -2176,7 +2207,6 @@ static int parseParens (statementInfo *const st, parenInfo *const info)
 
 static void initParenInfo (parenInfo *const info)
 {
-    info->pointerOrder          = 0;
     info->isParamList		= TRUE;
     info->isKnrParamList	= TRUE;
     info->isNameCandidate	= TRUE;
@@ -2215,8 +2245,6 @@ static void analyzeParens (statementInfo *const st)
 	    	token->type = TOKEN_NAME;
 	    	processName (st);
 	    	st->gotParenName = TRUE;
-	    	if (! (c == '('  &&  info.nestedArgs))
-				st->pointerOrder = info.pointerOrder;
 		}
 		else if (! st->gotArgs  &&  info.isParamList)
 		{
@@ -2274,8 +2302,9 @@ static void processColon (statementInfo *const st)
 	else
 	{
 		cppUngetc (c);
-		if ((isLanguage (Lang_cpp) || isLanguage (Lang_csharp))  &&
-			inheritingDeclaration (st->declaration))
+		if (((isLanguage (Lang_cpp) || isLanguage (Lang_csharp))  &&
+			inheritingDeclaration (st->declaration)) ||
+			isLanguage (Lang_d))
 		{
 			readParents (st, ':');
 		}
@@ -2413,7 +2442,7 @@ static void nextToken (statementInfo *const st)
 		    case ';': setToken (st, TOKEN_SEMICOLON);			break;
 		    case '<': skipToMatch ("<>");				break;
 	    	case '=': processInitializer (st);				break;
-		    case '[': skipToMatch ("[]");				break;
+		    case '[': setToken (st, TOKEN_ARRAY); skipToMatch ("[]");	break;
 		    case '{': setToken (st, TOKEN_BRACE_OPEN);			break;
 	    	case '}': setToken (st, TOKEN_BRACE_CLOSE);			break;
 	    	default:  parseGeneralToken (st, c);				break;
@@ -2538,28 +2567,6 @@ static void nest (statementInfo *const st, const unsigned int nestLevel)
     setToken (st, TOKEN_BRACE_CLOSE);
 }
 
-static int getTokenPointerOrder (statementInfo *const st, int prev)
-{
-    const tokenInfo *prev_tmp = prevToken (st, prev);
-    int pointer_order = 0;
-
-    /* ... , *const ptr; */
-    if (isVariableKeyword ( prev_tmp ))
-    {
-	prev++;
-	prev_tmp = prevToken (st, prev);
-    }
-
-    while (isType ( prev_tmp, TOKEN_STAR ))
-    {
-	pointer_order++;
-	prev++;
-	prev_tmp = prevToken (st, prev);
-    }
-
-    return pointer_order;
-}
-
 static void tagCheck (statementInfo *const st)
 {
     const tokenInfo *const token = activeToken (st);
@@ -2581,79 +2588,65 @@ static void tagCheck (statementInfo *const st)
 	case TOKEN_BRACE_OPEN:
 	    if (isType (prev, TOKEN_ARGS))
 	    {
-		if (st->haveQualifyingName)
-		{
-		    st->declaration = DECL_FUNCTION;
-		    if (isType (prev2, TOKEN_NAME))
-			copyToken (st->blockName, prev2);
-
-		    if (!isLanguage (Lang_java))
-		    {
-			((tokenInfo *)prev2)->pointerOrder = getTokenPointerOrder (st, 3);
-		    }
-		    qualifyFunctionTag (st, prev2);
-		}
+			if (st->haveQualifyingName)
+			{
+				st->declaration = DECL_FUNCTION;
+				if (isType (prev2, TOKEN_NAME))
+					copyToken (st->blockName, prev2);
+				qualifyFunctionTag (st, prev2);
+			}
 	    }
 	    else if (isContextualStatement (st))
 	    {
-		tokenInfo *name_token = (tokenInfo *)prev;
-		if (isType (name_token, TOKEN_NAME))
-		{
-		    copyToken (st->blockName, name_token);
-		}
-		else if (isLanguage (Lang_csharp))
-				makeTag (prev, st, FALSE, TAG_PROPERTY);
-		else
-		{
-		    tokenInfo *contextual_token = (tokenInfo *)prev;
-		    if(isContextualKeyword (contextual_token))
-		    {
-				char buffer[64];
-
-				name_token = newToken ();
-				copyToken (name_token, contextual_token);
-
-				sprintf(buffer, "anon_%s_%d", name_token->name->buffer, contextual_fake_count++);
-				vStringClear(name_token->name);
-				vStringCatS(name_token->name, buffer);
-
-				name_token->type = TOKEN_NAME;
-				name_token->keyword	= KEYWORD_NONE;
-
-				advanceToken (st);
-				contextual_token = activeToken (st);
-				copyToken (contextual_token, token);
-				copyToken ((tokenInfo *const)token, name_token);
+			tokenInfo *name_token = (tokenInfo *)prev;
+			if (isType (name_token, TOKEN_NAME))
+			{
 				copyToken (st->blockName, name_token);
-				copyToken (st->firstToken, name_token);
-		    }
-		}
-		qualifyBlockTag (st, name_token);
+			}
+			else if (isLanguage (Lang_csharp))
+					makeTag (prev, st, FALSE, TAG_PROPERTY);
+			else
+			{
+				tokenInfo *contextual_token = (tokenInfo *)prev;
+				if(isContextualKeyword (contextual_token))
+				{
+					char buffer[64];
+
+					name_token = newToken ();
+					copyToken (name_token, contextual_token);
+
+					sprintf(buffer, "anon_%s_%d", name_token->name->buffer, contextual_fake_count++);
+					vStringClear(name_token->name);
+					vStringCatS(name_token->name, buffer);
+
+					name_token->type = TOKEN_NAME;
+					name_token->keyword	= KEYWORD_NONE;
+
+					advanceToken (st);
+					contextual_token = activeToken (st);
+					copyToken (contextual_token, token);
+					copyToken ((tokenInfo *const)token, name_token);
+					copyToken (st->blockName, name_token);
+					copyToken (st->firstToken, name_token);
+				}
+			}
+			qualifyBlockTag (st, name_token);
 	    }
 	    break;
+	case TOKEN_ARRAY:
 	case TOKEN_SEMICOLON:
 	case TOKEN_COMMA:
 	    if (insideEnumBody (st))
-		;
+			;
 	    else if (isType (prev, TOKEN_NAME))
 	    {
-		if (isContextualKeyword (prev2))
-		    makeTag (prev, st, TRUE, TAG_EXTERN_VAR);
-		else
-		{
-		    if (!isLanguage (Lang_java))
-		    {
-			((tokenInfo *)prev)->pointerOrder = getTokenPointerOrder (st, 2);
-		    }
-			qualifyVariableTag (st, prev);
-	    }
+			if (isContextualKeyword (prev2))
+				makeTag (prev, st, TRUE, TAG_EXTERN_VAR);
+			else
+				qualifyVariableTag (st, prev);
 	    }
 	    else if (isType (prev, TOKEN_ARGS)  &&  isType (prev2, TOKEN_NAME))
 	    {
-			if (!isLanguage (Lang_java))
-			{
-			    ((tokenInfo *)prev2)->pointerOrder = getTokenPointerOrder (st, 3);
-			}
 		    qualifyFunctionDeclTag (st, prev2);
 	    }
 	    break;
@@ -2775,14 +2768,14 @@ static void initializeDParser (const langType language)
 {
     contextual_fake_count = 0;
     Lang_d = language;
-    buildKeywordHash (language, 1);
+    buildKeywordHash (language, 1);	// C++ keywords
 }
 
 static void initializeFeriteParser (const langType language)
 {
     contextual_fake_count = 0;
     Lang_ferite = language;
-    buildKeywordHash (language, 1);
+    buildKeywordHash (language, 1);	// C++ keywords
 }
 
 static void initializeCsharpParser (const langType language)
