@@ -38,7 +38,12 @@
     DiffParser, \
     VhdlParser, \
     LuaParser, \
-    JavaScriptParser
+    JavaScriptParser, \
+    HaskellParser, \
+    CsharpParser, \
+    FreeBasicParser,\
+    HaxeParser,\
+    RestParser
 
 /*
 langType of each parser
@@ -66,6 +71,11 @@ langType of each parser
 21	VhdlParser
 22	LuaParser
 23	JavaScriptParser
+24	HaskellParser
+25	CsharpParser
+26	FreeBasicParser
+27  HaxeParser
+28  RestParser
 */
 #endif	/* _PARSERS_H */
 
