@@ -137,7 +137,7 @@ gboolean tm_tag_init(TMTag *tag, TMSourceFile *file, const tagEntryInfo *tag_ent
 		tag->name = g_strdup(tag_entry->name);
 		tag->type = get_tag_type(tag_entry->kindName);
 		tag->atts.entry.local = tag_entry->isFileScope;
-		tag->atts.entry.pointerOrder = tag_entry->pointerOrder;
+		tag->atts.entry.pointerOrder = 0;	/* backward compatibility (use var_type instead) */
 		tag->atts.entry.line = tag_entry->lineNumber;
 		if (NULL != tag_entry->extensionFields.arglist)
 			tag->atts.entry.arglist = g_strdup(tag_entry->extensionFields.arglist);
@@ -728,7 +728,7 @@ static const char *tm_tag_access_name(TMTag *tag)
 
 void tm_tag_print(TMTag *tag, FILE *fp)
 {
-	const char *access, *impl, *type;
+	const char *laccess, *impl, *type;
 	if (!tag || !fp)
 		return;
 	if (tm_tag_file_t == tag->type)
@@ -736,11 +736,11 @@ void tm_tag_print(TMTag *tag, FILE *fp)
 		fprintf(fp, "%s\n", tag->name);
 		return;
 	}
-	access = tm_tag_access_name(tag);
+	laccess = tm_tag_access_name(tag);
 	impl = tm_tag_impl_name(tag);
 	type = tm_tag_type_name(tag);
-	if (access)
-		fprintf(fp, "%s ", access);
+	if (laccess)
+		fprintf(fp, "%s ", laccess);
 	if (impl)
 		fprintf(fp, "%s ", impl);
 	if (type)
