@@ -20,15 +20,6 @@
 # include <glob.h>
 #endif
 #include <glib/gstdio.h>
-// handling of P_tmpdir, should be something like /tmp, take the root directory under Win32,
-// and assume /tmp on non-Win32 systems where P_tmpdir is not set
-#ifndef P_tmpdir
-# ifdef G_OS_WIN32
-#  define P_tmpdir "\\"
-# else
-#  define P_tmpdir "/tmp"
-# endif
-#endif
 
 #include "tm_tag.h"
 #include "tm_workspace.h"
@@ -38,28 +29,20 @@
 static TMWorkspace *theWorkspace = NULL;
 guint workspace_class_id = 0;
 
-static gboolean tm_create_workspace(void)
+static gboolean tm_create_workspace()
 {
-#ifdef G_OS_WIN32
-	char *file_name = g_strdup_printf("%s_%s_%ld.%d", P_tmpdir, PACKAGE, time(NULL), getpid());
-#else
-	char *file_name = g_strdup_printf("%s/%s_%ld.%d", P_tmpdir, PACKAGE, time(NULL), getpid());
-#endif
-
 	workspace_class_id = tm_work_object_register(tm_workspace_free, tm_workspace_update
 		  , tm_workspace_find_object);
 	theWorkspace = g_new(TMWorkspace, 1);
 	if (FALSE == tm_work_object_init(TM_WORK_OBJECT(theWorkspace),
-		  workspace_class_id, file_name, TRUE))
+		  workspace_class_id, NULL, TRUE))
 	{
-		g_free(file_name);
 		g_free(theWorkspace);
 		theWorkspace = NULL;
 		g_warning("Failed to initialize workspace");
 		return FALSE;
 	}
 
-	g_free(file_name);
 	theWorkspace->global_tags = NULL;
 	theWorkspace->work_objects = NULL;
 	return TRUE;
@@ -90,14 +73,13 @@ void tm_workspace_free(gpointer workspace)
 				tm_tag_free(theWorkspace->global_tags->pdata[i]);
 			g_ptr_array_free(theWorkspace->global_tags, TRUE);
 		}
-		unlink(theWorkspace->work_object.file_name);
 		tm_work_object_destroy(TM_WORK_OBJECT(theWorkspace));
 		g_free(theWorkspace);
 		theWorkspace = NULL;
 	}
 }
 
-const TMWorkspace *tm_get_workspace(void)
+const TMWorkspace *tm_get_workspace()
 {
 	if (NULL == theWorkspace)
 		tm_create_workspace();
@@ -106,8 +88,9 @@ const TMWorkspace *tm_get_workspace(void)
 
 gboolean tm_workspace_add_object(TMWorkObject *work_object)
 {
+	// theWorkspace should already have been created otherwise something went wrong
 	if (NULL == theWorkspace)
-		tm_create_workspace();
+		return FALSE;
 	if (NULL == theWorkspace->work_objects)
 		theWorkspace->work_objects = g_ptr_array_new();
 	g_ptr_array_add(theWorkspace->work_objects, work_object);
@@ -115,12 +98,14 @@ gboolean tm_workspace_add_object(TMWorkObject *work_object)
 	return TRUE;
 }
 
-gboolean tm_workspace_remove_object(TMWorkObject *w, gboolean do_free)
+gboolean tm_workspace_remove_object(TMWorkObject *w, gboolean do_free, gboolean update)
 {
 	guint i;
 	if ((NULL == theWorkspace) || (NULL == theWorkspace->work_objects)
 		  || (NULL == w))
 		return FALSE;
+
+
 	for (i=0; i < theWorkspace->work_objects->len; ++i)
 	{
 		if (theWorkspace->work_objects->pdata[i] == w)
@@ -128,10 +113,12 @@ gboolean tm_workspace_remove_object(TMWorkObject *w, gboolean do_free)
 			if (do_free)
 				tm_work_object_free(w);
 			g_ptr_array_remove_index_fast(theWorkspace->work_objects, i);
-			tm_workspace_update(TM_WORK_OBJECT(theWorkspace), TRUE, FALSE, FALSE);
+			if (update)
+				tm_workspace_update(TM_WORK_OBJECT(theWorkspace), TRUE, FALSE, FALSE);
 			return TRUE;
 		}
 	}
+
 	return FALSE;
 }
 
@@ -149,7 +136,7 @@ gboolean tm_workspace_load_global_tags(const char *tags_file, gint mode)
 	if (NULL == (fp = g_fopen(tags_file, "r")))
 		return FALSE;
 	if (NULL == theWorkspace)
-		tm_create_workspace();
+		return FALSE;
 	if (NULL == theWorkspace->global_tags)
 		theWorkspace->global_tags = g_ptr_array_new();
 	while (NULL != (tag = tm_tag_new_from_file(NULL, fp, mode)))
@@ -248,8 +235,8 @@ static gint get_global_tag_type_mask(gint lang)
 	}
 }
 
-gboolean tm_workspace_create_global_tags(const char *pre_process, const char **includes
-  , int includes_count, const char *tags_file, int lang)
+gboolean tm_workspace_create_global_tags(const char *config_dir, const char *pre_process,
+	const char **includes, int includes_count, const char *tags_file, int lang)
 {
 #ifdef HAVE_GLOB_H
 	glob_t globbuf;
@@ -264,14 +251,14 @@ gboolean tm_workspace_create_global_tags(const char *pre_process, const char **i
 	GHashTable *includes_files_hash;
 	GList *includes_files = NULL;
 #ifdef G_OS_WIN32
-	char *temp_file = g_strdup_printf("%s_%d_%ld_1.cpp", P_tmpdir, getpid(), time(NULL));
-	char *temp_file2 = g_strdup_printf("%s_%d_%ld_2.cpp", P_tmpdir, getpid(), time(NULL));
+	char *temp_file = g_strdup_printf("%s\\_%d_%ld_1.cpp", config_dir, getpid(), time(NULL));
+	char *temp_file2 = g_strdup_printf("%s\\_%d_%ld_2.cpp", config_dir, getpid(), time(NULL));
 #else
-	char *temp_file = g_strdup_printf("%s/%d_%ld_1.cpp", P_tmpdir, getpid(), time(NULL));
-	char *temp_file2 = g_strdup_printf("%s/%d_%ld_2.cpp", P_tmpdir, getpid(), time(NULL));
+	char *temp_file = g_strdup_printf("%s/%d_%ld_1.cpp", config_dir, getpid(), time(NULL));
+	char *temp_file2 = g_strdup_printf("%s/%d_%ld_2.cpp", config_dir, getpid(), time(NULL));
 #endif
 
-	if (NULL == (fp = g_fopen(temp_file, "w")))
+	if (NULL == theWorkspace || NULL == (fp = g_fopen(temp_file, "w")))
 		return FALSE;
 
 	includes_files_hash = g_hash_table_new_full (tm_file_inode_hash,
@@ -366,7 +353,7 @@ gboolean tm_workspace_create_global_tags(const char *pre_process, const char **i
 #endif
 		system(command);
 		g_free(command);
-		unlink(temp_file);
+		g_unlink(temp_file);
 		g_free(temp_file);
 	}
 	else
@@ -379,10 +366,10 @@ gboolean tm_workspace_create_global_tags(const char *pre_process, const char **i
 	source_file = tm_source_file_new(temp_file2, TRUE, tm_source_file_get_lang_name(lang));
 	if (NULL == source_file)
 	{
-		unlink(temp_file2);
+		g_unlink(temp_file2);
 		return FALSE;
 	}
-	unlink(temp_file2);
+	g_unlink(temp_file2);
 	g_free(temp_file2);
 	if ((NULL == source_file->tags_array) || (0 == source_file->tags_array->len))
 	{
